#ifndef CUTE_TDPDF_H
#define CUTE_TDPDF_H

#include "CuTe.h"
#include "TDPDF_Plot.h"
//#include "config_CuTe.h"

#ifndef __USE_GNU
	inline double exp10(const double x){ return exp(x*log(10.));}
#endif


namespace CuTe {

using NS_TDPDF::TD_PDF_TO_PLOT;
using NS_TDPDF::TD_PDF_POL_PLOT;
using NS_TDPDF::SOPS_TDPDF;
using NS_Polynom::max;

using std::string;
using std::vector;



class TDPDF_Plot {
protected:
	TD_PDF_TO_PLOT tdPDF;
	TD_PDF_POL_PLOT tdpol;

	bool pol;

	const SOPS_TDPDF *OP;
	NS_File_Path::File_Path outfile;


	int ofprec,ofwidth;

	vector<double> xi_values;
	int xi_N;
	vector<double> xT_or_qT_values;
	int xT_or_qT_N;


	inline int _set_values(vector<double> &out, const vector<double> &in, int steps){
//		DOUT dout("gPDF_Plot::_set_values");
//		using NS_Utils::operator <<;
//		dout()	<<"steps="<<steps<<endl;
//		dout()	<<"in="<<in<<endl;
		if(steps<=1){
			out	= in;
		}else{
			int N_in(in.size());
			int i_next(1);
			out.clear();
			--steps;

			double inv_steps(1./static_cast<double>(steps));

			double D;
			double val,next;

			val	= in[0];
			while(i_next<N_in){

				out.push_back(val);

				next= in[i_next++];
				D	= (next - val)*inv_steps;

				for(int i_steps=1; i_steps<steps; ++i_steps){
					out.push_back(val+=D);
				}
				val	= next;
			}
			out.push_back(val);
		}

//		dout()	<<"out="<<out<<endl;
		return out.size();
	}


	inline int _set_log_values(vector<double> &out, vector<double> in, int steps){
//		DOUT dout("gPDF_Plot::_set_log_values");
		using NS_Utils::operator <<;
//		dout()	<<"in="<<in<<endl;
		for(int i=0; i<in.size(); ++i) in[i] = log10(in[i]);
		_set_values(out,in,steps);
		for(int i=0; i<out.size(); ++i) out[i] = exp10(out[i]);
//		dout()	<<"out="<<out<<endl;
		return out.size();
	}

	inline void _set(const vector<double> &xi_values, int xi_steps, const vector<double> &xT_or_qT_values, int xT_or_qT_steps){
//		DOUT dout("gPDF_Plot::set");
		xi_N	= _set_log_values(this->xi_values,xi_values,xi_steps);
		xT_or_qT_N	= _set_values(this->xT_or_qT_values,xT_or_qT_values,xT_or_qT_steps);


//		using NS_Utils::operator <<;
//		dout()	<<"xi_values:"<< xi_values	<<endl;
//		dout()	<<"xi_steps :"<< xi_steps	<<endl;
//		dout()	<<"xi       :"<< this->xi_values	<<endl;
//		dout()	<<"xi_N     :"<< xi_N	<<endl;

	}

	inline const string plot(double xi, double xT_or_qT){
//		stringstream ss;
//		ss.precision(ofprec);
//		ss.width(ofwidth);

		if(pol){
			tdpol(xi,xT_or_qT);
			return tdpol.to_String();
		}else{
			tdPDF(xi,xT_or_qT);
			return tdPDF.to_String();
		}
	}


	inline const string _FirstLine(){
		std::stringstream FL;

		time_t now = time(0);
		char* dt = ctime(&now);

		FL << "######################################################\n";
		FL << "##  ";											FL << "\n";
		FL << "##  ";					FL << PACKAGE_STRING;	FL << "\n";
		FL << "##  Date = ";					FL << dt;
		FL << "##  ";											FL << "\n";
		FL << "######################################################\n";
		FL << "##  ";											FL << "\n";
		FL << "##  Infile = ";			FL << OP->get_infilename().full;		FL << "\n";
		FL << "##  ";											FL << "\n";
		FL << "######################################################\n";

		OP->to_ss(FL,"##\t");
//		op->to_out_ss(FL);


		FL << "######################################################\n";
		FL << "###  Outfile: ";									FL << "\n";
		FL << "######################################################\n";

		if(pol){
			FL << tdpol.FL_exp();
			FL << tdpol.FL();
		}else{
			FL << tdPDF.FL_exp();
			FL << tdPDF.FL();
		}

		FL	<<endl;

		return FL.str();
	}

public:

	inline void init(const SOPS_TDPDF &OP){
		this->OP = &(OP);

		if(OP.TDPDF.TDPDF_Polynom.is_set){
			tdpol.init(OP);
			pol	= true;
		}else{
	//		DOUT dout("gPDF_Plot::init");
			tdPDF.init(OP);
			pol	= false;
		}

		outfile = NS_File_Path::File_Path(OP.get_infilename(),OP.get_infilename(),"out");

//		dout()	<< op.infilename.properties()	<<endl;
//		dout()	<< outfile.properties()	<<endl;


		ofprec	= max(static_cast<int>(log10(10./OP.CUBA.accuracy())),3);
		ofwidth	= max(ofprec+3,15);

		_set(OP.TDPDF.TDPDF_xi(),OP.TDPDF.TDPDF_xi_steps(),OP.TDPDF.TDPDF_xT_or_qT(),OP.TDPDF.TDPDF_xT_or_qT_steps());

	}

	inline void plot(){
		NS_Utils::Timer timer;
		timer.start();

		vector<double>::const_iterator xi,xi_beg(xi_values.begin()),xi_end(xi_values.end());
		vector<double>::const_iterator xT_or_qT,xT_or_qT_beg(xT_or_qT_values.begin()),xT_or_qT_end(xT_or_qT_values.end());

		std::ofstream of(outfile.full.c_str());
		string tmp;

		tmp	=_FirstLine();

		of	<< tmp;
		cout<< tmp;

		xi	= xi_beg;
		while(xi!=xi_end){
			xT_or_qT	= xT_or_qT_beg;
			while(xT_or_qT!=xT_or_qT_end){

				tmp = plot(*(xi),*(xT_or_qT));
//				cout	<<tmp;
				of		<<tmp;

				++xT_or_qT;
			}
			++xi;
		}
		timer.stop();
		cout	<<endl <<"Time: "<<time	<<endl;
		of		<<endl <<"##Time: "<<timer	<<endl;

		of.close();
	}

	TDPDF_Plot()
	:tdPDF(),tdpol(),pol(),OP(NULL)
	,outfile()
	,ofprec(),ofwidth()
	,xi_values(),xi_N()
	,xT_or_qT_values(),xT_or_qT_N(){

	}

};

}

//inline void exit_TDPDF(){
////	CuTe::DOUT dout("exit_TDPDF");
//	cout << "******************************** "<<PACKAGE_STRING<<"-TDPDF ***********************************" << endl;
//}
//
//inline void init_TDPDF(const std::string& pdfsetsPath){
//	cout << "******************************** "<<PACKAGE_STRING<<"-TDPDF ***********************************" << endl;
//	gsl_set_error_handler_off();
////	CuTe::NS_LHAFUNC::LHAPDF_initLHAPDF(pdfsetsPath);
//	cout << "PDF path = " << pdfsetsPath << endl;
//}
//
//inline void init_TDPDF(){
//	init_TDPDF(std::string(LHAPDF_PDF_DIR));
//}


int main (int argc, char *argv[]){
	CuTe::CuTe_INIT_EXIT _inex("-TDPDF");

	std::string filename;
	CuTe::SOPS_TDPDF OP;
	CuTe::TDPDF_Plot plot;

	if(argc>=2){
		for(int i=1; i<argc; ++i){
			filename = argv[i];
			cout << "argv[" << i << "] = " << argv[i] << endl;
			if(OP.open(filename)){
				plot.init(OP);
				plot.plot();
			}
		}
	}else{
		cout << " ! NO FILE ARGUMENT ! " << endl;
		OP.write_example();
		filename = "example.in";
		cout << filename << endl;
		if(OP.open(filename)){
			plot.init(OP);
			plot.plot();
		}
	}

	return 0;
}

#endif
