#ifndef APSF_CONVOLUTION_H
#define APSF_CONVOLUTION_H

#include "Convolutions.h"
#include "APSFunctions.h"
#include "Ibar_G_Functions.h"
#include "Ibar_Q_Functions.h"

namespace CuTe {
namespace NS_Convolutions{

using namespace NS_APSF;

using NS_PlusDists::PD_Params;

using NS_Ibar::Ibar_G_Parameter;
using NS_Ibar::Ibar_G;
using NS_Ibar::Ibar_Q_Parameter;
using NS_Ibar::Ibar_Q;

using std::ostream;
using std::endl;
using std::vector;


template< int n >
class Plus_Convolution:public _Convolution_Integrand{
protected:
	PD_Params p;

	virtual void _update_V(vector<double> &V, const double xi, const double z, const double Jac=1){
		p.set_x(xi);
		p.update(z);
		for(int i=(-nf); i<=+nf; ++i){
			V[NFMAX+i] =	 Jac*NS_PlusDists::PD<n>(f->get_fz(i)/z,f->get_fxi(i),p);
		}
	}

public:

	Plus_Convolution()
	:_Convolution_Integrand()
	,p(){

	}

	ostream& to_stream(ostream &os, const int ntabs=0)const{
		string tab("");
		for(int i=0; i<ntabs; ++i) tab += "\t";

		os	<<tab<<"Plus_Convolution<"<<n<<">:"<<endl;
		_Convolution_Integrand::to_stream(os,ntabs+1);
		os	<<tab<<"\t"<<"function f:"<<endl;
		f->to_stream(os,ntabs+1);

		return os;
	}

};

template< e_P e_i, e_P e_j, int n >
class APSF_single_Convolution:public _Convolution_Integrand{
protected:
	APSF_Parameter p;

	virtual void _update_V(vector<double> &V, const double xi, const double z, const double Jac=1){
		p.set_x(xi);
		p.update(z);

		for(int i=(-nf); i<=(+nf); ++i){
			V[NFMAX+i]	=	Jac*APSF<e_i,e_j,n>(f->get_fz(i)/z,f->get_fxi(i),p,nf);
			if( (V[NFMAX+i]!=V[NFMAX+i]) && (z==1) ){
				V[NFMAX+i]	= 0.;
			}
		}
	}

	virtual void _set_nf(){
		_Convolution_Integrand::_set_nf();
	}

public:

	APSF_single_Convolution()
	:_Convolution_Integrand()
	,p(){

	}

	ostream& to_stream(ostream &os, const int ntabs=0)const{
		string tab("");
		for(int i=0; i<ntabs; ++i) tab += "\t";

		os	<<tab<<"APSF_single_Convolution<"<<e_i<<","<<e_j<<","<<n<<">:"<<endl;
		_Convolution_Integrand::to_stream(os,ntabs+1);
		os	<<tab<<"\t"<<"function f:"<<endl;
		f->to_stream(os,ntabs+1);

		return os;
	}
};

template< int n >
class APSF_Convolution:public _Convolution_Integrand{
protected:
	APSF_Parameter p;

	virtual void _update_V(vector<double> &V, const double xi, const double z, const double Jac=1){
		p.set_x(xi);
		p.update(z);

		const double GG(APSF<e_g,e_g,n>(f->get_fz(0)/z,f->get_fxi(0),p,nf));
		const double GQ(APSF<e_g,e_q,n>(f->get_Sz( )/z,f->get_Sxi( ),p,nf));

		const double  QG(APSF<e_q,e_g,n>(f->get_fz(0)/z,f->get_fxi(0),p,nf));
		const double  QP (APSF<e_q,e_qp ,n>(f->get_Szq(+1)/z,f->get_Sxiq(+1),p,nf));
		const double  QPb(APSF<e_q,e_qbp,n>(f->get_Szq(-1)/z,f->get_Sxiq(-1),p,nf));

		const double bQG (QG);
		const double bQPb(APSF<e_q,e_qp ,n>(f->get_Szq(-1)/z,f->get_Sxiq(-1),p,nf));
		const double bQP (APSF<e_q,e_qbp,n>(f->get_Szq(+1)/z,f->get_Sxiq(+1),p,nf));

//		if(GG!=GG)	cerr<<"NAN APSF_Convolution<"<<n<<">[GG]"<<"\tz = "<<z<<"\t(z==1) = "<<(z==1)<<endl;
//		if(GQ!=GQ)	cerr<<"NAN APSF_Convolution<"<<n<<">[GQ]"<<"\tz = "<<z<<"\t(z==1) = "<<(z==1)<<endl;
//
//		if(QG !=QG )	cerr<<"NAN APSF_Convolution<"<<n<<">[QG]"<<"\tz = "<<z<<"\t(z==1) = "<<(z==1)<<endl;
//		if(QP !=QP )	cerr<<"NAN APSF_Convolution<"<<n<<">[QP]"<<"\tz = "<<z<<"\t(z==1) = "<<(z==1)<<endl;
//		if(QPb!=QPb)	cerr<<"NAN APSF_Convolution<"<<n<<">[QPb]"<<"\tz = "<<z<<"\t(z==1) = "<<(z==1)<<endl;
//
//		if(bQG !=bQG )	cerr<<"NAN APSF_Convolution<"<<n<<">[bQG]"<<"\tz = "<<z<<"\t(z==1) = "<<(z==1)<<endl;
//		if(bQPb!=bQPb)	cerr<<"NAN APSF_Convolution<"<<n<<">[bQPb]"<<"\tz = "<<z<<"\t(z==1) = "<<(z==1)<<endl;
//		if(bQP !=bQP )	cerr<<"NAN APSF_Convolution<"<<n<<">[bQP]"<<"\tz = "<<z<<"\t(z==1) = "<<(z==1)<<endl;

		for(int i=(-nf); i<0; ++i){
			V[NFMAX+i]	=	Jac*(
								 APSF<e_q,e_qmp ,n>(f->get_fz( +i)/z,f->get_fxi( +i),p,nf)
								+APSF<e_q,e_qbmp,n>(f->get_fz( -i)/z,f->get_fxi( -i),p,nf)
								+bQG+bQP+bQPb
							);
			if( (V[NFMAX+i]!=V[NFMAX+i])&&(z==1) ){
				V[NFMAX+i]	= 0.;
			}
		}
		V[NFMAX]	= Jac*( GG + GQ );
		if( (V[NFMAX]!=V[NFMAX])&&(z==1) ){
			V[NFMAX]	= 0.;
		}
		for(int i=1; i<=(+nf); ++i){
			V[NFMAX+i]	=	Jac*(
								 APSF<e_q,e_qmp ,n>(f->get_fz( +i)/z,f->get_fxi( +i),p,nf)
								+APSF<e_q,e_qbmp,n>(f->get_fz( -i)/z,f->get_fxi( -i),p,nf)
								+QG+QP+QPb
							);
			if( (V[NFMAX+i]!=V[NFMAX+i])&&(z==1) ){
				V[NFMAX+i]	= 0.;
			}
		}
	}

	virtual void _set_nf(){
		_Convolution_Integrand::_set_nf();
	}

public:

	APSF_Convolution()
	:_Convolution_Integrand()
	,p(){

	}

	ostream& to_stream(ostream &os, const int ntabs=0)const{
		string tab("");
		for(int i=0; i<ntabs; ++i) tab += "\t";

		os	<<tab<<"APSF_Convolution<"<<n<<">:"<<endl;
		_Convolution_Integrand::to_stream(os,ntabs+1);
		os	<<tab<<"\t"<<"function f:"<<endl;
		f->to_stream(os,ntabs+1);

		return os;
	}
};

template<int a, int l, int e >
class I_bar_Convolution:public _Convolution_Integrand{
protected:
	Ibar_G_Parameter pG;
	Ibar_Q_Parameter pQ;

	virtual void _update_V(vector<double> &V, const double xi, const double z, const double Jac=1){
		pG.set_x(xi);
		pG.update(z);
		pQ.set_x(xi);
		pQ.update(z);

		const double GG(Ibar_G<e_g,a,l,e>(f->get_fz(0)/z,f->get_fxi(0),pG,nf));
		const double GQ(Ibar_G<e_q,a,l,e>(f->get_Sz( )/z,f->get_Sxi( ),pG,nf));

		const double  QG(Ibar_Q<e_g,a,l,e>(f->get_fz(0)/z,f->get_fxi(0),pQ,nf));
		const double  QP (Ibar_Q<e_qp ,a,l,e>(f->get_Szq(+1)/z,f->get_Sxiq(+1),pQ,nf));
		const double  QPb(Ibar_Q<e_qbp,a,l,e>(f->get_Szq(-1)/z,f->get_Sxiq(-1),pQ,nf));

		const double bQG (QG);
		const double bQPb(Ibar_Q<e_qp ,a,l,e>(f->get_Szq(-1)/z,f->get_Sxiq(-1),pQ,nf));
		const double bQP (Ibar_Q<e_qbp,a,l,e>(f->get_Szq(+1)/z,f->get_Sxiq(+1),pQ,nf));

		if(GG!=GG)	cerr<<"NAN I_bar_Convolution<"<<a<<","<<l<<","<<e<<">[GG]"<<"\tz = "<<z<<"\t(z==1) = "<<(z==1)<<endl;
		if(GQ!=GQ)	cerr<<"NAN I_bar_Convolution<"<<a<<","<<l<<","<<e<<">[GQ]"<<"\tz = "<<z<<"\t(z==1) = "<<(z==1)<<endl;

		if(QG !=QG )	cerr<<"NAN I_bar_Convolution<"<<a<<","<<l<<","<<e<<">[QG]"<<"\tz = "<<z<<"\t(z==1) = "<<(z==1)<<endl;
		if(QP !=QP )	cerr<<"NAN I_bar_Convolution<"<<a<<","<<l<<","<<e<<">[QP]"<<"\tz = "<<z<<"\t(z==1) = "<<(z==1)<<endl;
		if(QPb!=QPb)	cerr<<"NAN I_bar_Convolution<"<<a<<","<<l<<","<<e<<">[QPb]"<<"\tz = "<<z<<"\t(z==1) = "<<(z==1)<<endl;

		if(bQG !=bQG )	cerr<<"NAN I_bar_Convolution<"<<a<<","<<l<<","<<e<<">[bQG]"<<"\tz = "<<z<<"\t(z==1) = "<<(z==1)<<endl;
		if(bQPb!=bQPb)	cerr<<"NAN I_bar_Convolution<"<<a<<","<<l<<","<<e<<">[bQPb]"<<"\tz = "<<z<<"\t(z==1) = "<<(z==1)<<endl;
		if(bQP !=bQP )	cerr<<"NAN I_bar_Convolution<"<<a<<","<<l<<","<<e<<">[bQP]"<<"\tz = "<<z<<"\t(z==1) = "<<(z==1)<<endl;

		for(int i=(-nf); i<0; ++i){
			V[NFMAX+i]	=	Jac*(
								 Ibar_Q<e_qmp ,a,l,e>(f->get_fz( +i)/z,f->get_fxi( +i),pQ,nf)
								+Ibar_Q<e_qbmp,a,l,e>(f->get_fz( -i)/z,f->get_fxi( -i),pQ,nf)
								+bQG+bQP+bQPb
							);
		}
		V[NFMAX]	= Jac*( GG + GQ );
		for(int i=1; i<=(+nf); ++i){
			V[NFMAX+i]	=	Jac*(
								 Ibar_Q<e_qmp ,a,l,e>(f->get_fz( +i)/z,f->get_fxi( +i),pQ,nf)
								+Ibar_Q<e_qbmp,a,l,e>(f->get_fz( -i)/z,f->get_fxi( -i),pQ,nf)
								+QG+QP+QPb
							);
		}
	}

	virtual void _set_nf(){
		_Convolution_Integrand::_set_nf();
		pG.init(e,true);
		pQ.init(e,true);
	}

public:

	I_bar_Convolution()
	:_Convolution_Integrand()
	,pG(),pQ(){
	}

	ostream& to_stream(ostream &os, const int ntabs=0)const{
		string tab("");
		for(int i=0; i<ntabs; ++i) tab += "\t";

		os	<<tab<<"I_bar_Convolution<"<<a<<","<<l<<","<<e<<">:"<<endl;
		_Convolution_Integrand::to_stream(os,ntabs+1);
		os	<<tab<<"\t"<<"function f:"<<endl;
		f->to_stream(os,ntabs+1);

		return os;
	}
};

}	//NS_Convolutions
}	//CuTe

#endif	//APSF_CONVOLUTION_H

