#ifndef APSFUNCTIONS_H_
#define APSFUNCTIONS_H_

#include "konst.h"
#include "APSF_Parameter.h"

namespace CuTe{
namespace NS_APSF{

using NS_PlusDists::PD;
using std::cerr;

template< e_P i, e_P j , int N >
inline double APSF(const double fz, const double f1, const APSF_Parameter &p, const int Nf){
	cerr	<<"APSF<"<<i<<","<<j<<","<<N<<"> called"<<endl;
	return 0.;
}

template<>
inline double APSF<e_g,e_g,0>(const double fz, const double f1, const APSF_Parameter &p, const int Nf){
	const double z(p.z);
	const HPLz &hpl(p.hpl);
	return	(
				fz*(8*C_A*(-2 + 1/z + z - pow_2(z)))
				+PD<-1>(fz,f1,p)*((22*C_A)/3. - (4*Nf)/3.)
				+PD<+0>(fz,f1,p)*(8*C_A)
	       	);
}

template<>
inline double APSF<e_g,e_q,0>(const double fz, const double f1, const APSF_Parameter &p, const int Nf){
	const double z(p.z);
	const HPLz &hpl(p.hpl);
	return	(
				fz*(C_F*(-8 + 8/z + 4*z))
	       	);
}

template<>
inline double APSF<e_q,e_g,0>(const double fz, const double f1, const APSF_Parameter &p, const int Nf){
	const double z(p.z);
	const HPLz &hpl(p.hpl);
	return	(
				fz*(2 - 4*z + 4*pow_2(z))
	       	);
}

template<>
inline double APSF<e_q,e_qmp,0>(const double fz, const double f1, const APSF_Parameter &p, const int Nf){
	const double z(p.z);
	const HPLz &hpl(p.hpl);
	return	(
				fz*(-4*C_F*(1 + z))
				+PD<-1>(fz,f1,p)*(6*C_F)
				+PD<+0>(fz,f1,p)*(8*C_F)
	       	);
}

template<>
inline double APSF<e_q,e_qbmp,0>(const double fz, const double f1, const APSF_Parameter &p, const int Nf){
	return 0;
}

template<>
inline double APSF<e_q,e_qp,0>(const double fz, const double f1, const APSF_Parameter &p, const int Nf){
	return 0;
}

template<>
inline double APSF<e_q,e_qbp,0>(const double fz, const double f1, const APSF_Parameter &p, const int Nf){
	return 0;
}

template<>
inline double APSF<e_q,e_q,0>(const double fz, const double f1, const APSF_Parameter &p, const int Nf){
return APSF<e_q,e_qmp,0>(fz,f1,p,Nf) + APSF<e_q,e_qp,0>(fz,f1,p,Nf);

}

template<>
inline double APSF<e_q,e_qb,0>(const double fz, const double f1, const APSF_Parameter &p, const int Nf){
return APSF<e_q,e_qbmp,0>(fz,f1,p,Nf) + APSF<e_q,e_qbp,0>(fz,f1,p,Nf);

}

template<>
inline double APSF<e_g,e_g,1>(const double fz, const double f1, const APSF_Parameter &p, const int Nf){
	const double z(p.z);
	const HPLz &hpl(p.hpl);
	return	(
				fz*((4*C_A2*(-25 - 109*z + PI2*(24 + 12*pow_2(z) - 6/(1 + z))))/9. - (8*C_A2*(25 - 11*z + 44*pow_2(z))*hpl(0))/3. + (32*C_A2*pow(1 + z + pow_2(z),2)*hpl(-1,0))/(z*(1 + z)) - (32*C_A2*pow(1 + z - pow_2(z),2)*hpl(0,0))/(-1 + pow_2(z)) + Nf*((16*C_F*(-12 + 1/z + 6*z + 5*pow_2(z)))/3. + (8*C_A*(-23 + 29*z - 19*pow_2(z) + 23*pow_3(z)))/(9.*z) + ((-16*C_A*(1 + z))/3. - 8*C_F*(3 + 5*z))*hpl(0) - 16*C_F*(1 + z)*hpl(0,0)) - (32*C_A2*pow(1 - z + pow_2(z),2)*hpl(0,1))/((-1 + z)*z) - (32*C_A2*pow(1 - z + pow_2(z),2)*hpl(1,0))/((-1 + z)*z))
				+PD<-1>(fz,f1,p)*(((-16*C_A)/3. - 4*C_F)*Nf + (8*C_A2*(8 + 9*zeta_3))/3.)
				+PD<+0>(fz,f1,p)*((-80*C_A*Nf)/9. - (8*C_A2*(-67 + 3*PI2))/9.)
	       	);
}

template<>
inline double APSF<e_g,e_q,1>(const double fz, const double f1, const APSF_Parameter &p, const int Nf){
	const double z(p.z);
	const HPLz &hpl(p.hpl);
	return	(
				fz*(-4*C_F2*(5 + 7*z) + (8*C_A*C_F*(9 + (19 + 6*PI2)*z + 37*pow_2(z) + 44*pow_3(z)))/(9.*z) + (4*C_F2*(4 + 7*z) - (8*C_A*C_F*(36 + 15*z + 8*pow_2(z)))/3.)*hpl(0) + (8*C_F2*(-6 + 6/z + 5*z) - (8*C_A*C_F*(22 - 22*z + 17*pow_2(z)))/(3.*z))*hpl(1) + Nf*((-32*C_F*(5 - 5*z + 4*pow_2(z)))/(9.*z) + (16*C_F*(2 - 2*z + pow_2(z))*hpl(1))/(3.*z)) + 16*C_A*C_F*(2 + 2/z + z)*hpl(-1,0) + (8*C_F2*(-2 + z) + 16*C_A*C_F*(2 + z))*hpl(0,0) + 16*C_A*C_F*(-2 + 2/z + z)*hpl(0,1) + 16*C_A*C_F*(-2 + 2/z + z)*hpl(1,0) + (C_F2*(32 - 32/z - 16*z) + 16*C_A*C_F*(-2 + 2/z + z))*hpl(1,1))
	       	);
}

template<>
inline double APSF<e_q,e_g,1>(const double fz, const double f1, const APSF_Parameter &p, const int Nf){
	const double z(p.z);
	const HPLz &hpl(p.hpl);
	return	(
				fz*((4*C_A*(-18 + 20/z + (225 - 6*PI2)*z - 218*pow_2(z)))/9. - (2*C_F*(-42 + 87*z - 60*pow_2(z) + PI2*(2 - 4*z + 4*pow_2(z))))/3. + (C_F*(6 - 8*z + 16*pow_2(z)) + C_A*(4 + 32*z + (176*pow_2(z))/3.))*hpl(0) + (-16*C_A*(-1 + z)*z + 16*C_F*(-1 + z)*z)*hpl(1) - 8*C_A*(1 + 2*z + 2*pow_2(z))*hpl(-1,0) + (-8*C_A*(1 + 2*z) + C_F*(4 - 8*z + 16*pow_2(z)))*hpl(0,0) + 8*C_F*(1 - 2*z + 2*pow_2(z))*hpl(0,1) + 8*C_F*(1 - 2*z + 2*pow_2(z))*hpl(1,0) + (-8*C_A*(1 - 2*z + 2*pow_2(z)) + 8*C_F*(1 - 2*z + 2*pow_2(z)))*hpl(1,1))
	       	);
}

template<>
inline double APSF<e_q,e_qmp,1>(const double fz, const double f1, const APSF_Parameter &p, const int Nf){
	const double z(p.z);
	const HPLz &hpl(p.hpl);
	return	(
				fz*(40*C_F2*(-1 + z) + (4*C_A*C_F*(53 - 187*z + 3*PI2*(1 + z)))/9. + ((8*C_F2*(3 + 2*z - 2*pow_2(z)))/(-1 + z) + (C_A*C_F*(68 + 20*pow_2(z)))/(3 - 3*z))*hpl(0) + Nf*((8*C_F*(-1 + 11*z))/9. + (8*C_F*(1 + pow_2(z))*hpl(0))/(3.*(-1 + z))) + (-8*C_F2*(1 + z) - (8*C_A*C_F*(1 + pow_2(z)))/(-1 + z))*hpl(0,0) - (16*C_F2*(1 + pow_2(z))*hpl(0,1))/(-1 + z) - (16*C_F2*(1 + pow_2(z))*hpl(1,0))/(-1 + z))
				+PD<-1>(fz,f1,p)*((-2*C_F*Nf*(3 + 4*PI2))/9. + C_A*C_F*(5.666666666666667 + (44*PI2)/9. - 24*zeta_3) + C_F2*(3 - 4*PI2 + 48*zeta_3))
				+PD<+0>(fz,f1,p)*((-80*C_F*Nf)/9. - (8*C_A*C_F*(-67 + 3*PI2))/9.)
	       	);
}

template<>
inline double APSF<e_q,e_qbmp,1>(const double fz, const double f1, const APSF_Parameter &p, const int Nf){
	const double z(p.z);
	const HPLz &hpl(p.hpl);
	return	(
				fz*((4*C_A*C_F*(12*(-1 + pow_2(z)) + PI2*(1 + pow_2(z))))/(3.*(1 + z)) - (8*C_F2*(12*(-1 + pow_2(z)) + PI2*(1 + pow_2(z))))/(3.*(1 + z)) + (-8*C_A*C_F*(1 + z) + 16*C_F2*(1 + z))*hpl(0) + ((16*C_A*C_F*(1 + pow_2(z)))/(1 + z) - (32*C_F2*(1 + pow_2(z)))/(1 + z))*hpl(-1,0) + ((-8*C_A*C_F*(1 + pow_2(z)))/(1 + z) + (16*C_F2*(1 + pow_2(z)))/(1 + z))*hpl(0,0))
	       	);
}

template<>
inline double APSF<e_q,e_qp,1>(const double fz, const double f1, const APSF_Parameter &p, const int Nf){
	const double z(p.z);
	const HPLz &hpl(p.hpl);
	return	(
				fz*((4*C_F*(-18 + 20/z + 54*z - 56*pow_2(z)))/9. + C_F*(4 + 20*z + (32*pow_2(z))/3.)*hpl(0) - 8*C_F*(1 + z)*hpl(0,0))
	       	);
}

template<>
inline double APSF<e_q,e_qbp,1>(const double fz, const double f1, const APSF_Parameter &p, const int Nf){
	const double z(p.z);
	const HPLz &hpl(p.hpl);
	return	(
				fz*((4*C_F*(-18 + 20/z + 54*z - 56*pow_2(z)))/9. + C_F*(4 + 20*z + (32*pow_2(z))/3.)*hpl(0) - 8*C_F*(1 + z)*hpl(0,0))
	       	);
}

template<>
inline double APSF<e_q,e_q,1>(const double fz, const double f1, const APSF_Parameter &p, const int Nf){
return APSF<e_q,e_qmp,1>(fz,f1,p,Nf) + APSF<e_q,e_qp,1>(fz,f1,p,Nf);

}

template<>
inline double APSF<e_q,e_qb,1>(const double fz, const double f1, const APSF_Parameter &p, const int Nf){
return APSF<e_q,e_qbmp,1>(fz,f1,p,Nf) + APSF<e_q,e_qbp,1>(fz,f1,p,Nf);

}

template<>
inline double APSF<e_g,e_g,2>(const double fz, const double f1, const APSF_Parameter &p, const int Nf){
	const double z(p.z);
	const HPLz &hpl(p.hpl);
	return	(
				fz*((256*C_A3*PI2*pow(1 + z + pow_2(z),2)*hpl(-1,-1))/(3.*z*(1 + z)) - (16*C_A3*(255 + 632*z + 486*pow_2(z) + 632*pow_3(z) + 255*pow_4(z) + 54*PI2*pow(1 + z + pow_2(z),2))*hpl(-1,0))/(9.*z*(1 + z)) - (32*C_A3*PI2*(-5 - 13*z - 3*pow_2(z) + 13*pow_3(z) + 3*pow_4(z) + 11*pow_5(z))*hpl(0,-1))/(3.*z*(-1 + pow_2(z))) + (8*C_A3*(2369 - 789*z + 129*pow_2(z) + 789*pow_3(z) - 3034*pow_4(z) + 12*PI2*(5 + 11*z - 5*pow_2(z) - 11*pow_3(z) + 8*pow_4(z)))*hpl(0,0))/(9.*(-1 + pow_2(z))) + (32*C_A3*(-67 + 49*z - 43*pow_2(z) - 116*pow_3(z) - 24*pow_4(z) - 67*pow_5(z) + 3*PI2*(3 + z + 5*pow_2(z) - pow_3(z) - 5*pow_4(z) + pow_5(z)))*hpl(0,1))/(9.*z*(-1 + pow_2(z))) + (16*C_A3*(-134 + 159*z - 318*pow_2(z) + 159*pow_3(z) - 134*pow_4(z) + 18*PI2*pow(1 - z + pow_2(z),2))*hpl(1,0))/(9.*(-1 + z)*z) + 32*C_A3*(-3 + 11/z - 3*z + 11*pow_2(z))*hpl(-1,-1,0) + (64*C_A3*(44 + 57*z + 57*pow_2(z) + 44*pow_3(z))*hpl(-1,0,0))/(3.*z) + 32*C_A3*(25 + 11/z + 25*z + 11*pow_2(z))*hpl(-1,0,1) + (32*C_A3*(11 - 62*z + 10*pow_2(z) - 36*pow_3(z) + 66*pow_4(z))*hpl(0,-1,0))/(3.*(-1 + z)*z) - (32*C_A3*(69 - 29*z + 132*pow_2(z))*hpl(0,0,0))/3. - (16*C_A3*(-174 + 153*z - 210*pow_2(z) + 242*pow_3(z))*hpl(0,0,1))/(3.*(-1 + z)) - (32*C_A3*(-22 + 43*z - 29*pow_2(z) + 66*pow_3(z))*hpl(0,1,0))/(3.*z) - (16*C_A3*(110 - 196*z + 183*pow_2(z) - 196*pow_3(z) + 110*pow_4(z))*hpl(1,0,0))/(3.*(-1 + z)*z) - (768*C_A3*pow(1 + z + pow_2(z),2)*hpl(-1,-1,0,0))/(z*(1 + z)) - (512*C_A3*pow(1 + z + pow_2(z),2)*hpl(-1,-1,0,1))/(z*(1 + z)) - (256*C_A3*pow(1 + z + pow_2(z),2)*hpl(-1,0,-1,0))/(z*(1 + z)) + (512*C_A3*pow(1 + z + pow_2(z),2)*hpl(-1,0,0,0))/(z*(1 + z)) + (512*C_A3*pow(1 + z + pow_2(z),2)*hpl(-1,0,0,1))/(z*(1 + z)) + (128*C_A3*pow(1 + z + pow_2(z),2)*hpl(-1,0,1,0))/(z*(1 + z)) - (128*C_A3*(-1 - 5*z + pow_2(z) + 5*pow_3(z) - pow_4(z) + 3*pow_5(z))*hpl(0,-1,-1,0))/(z*(-1 + pow_2(z))) + (64*C_A3*(-7 - 5*z - 13*pow_2(z) + 5*pow_3(z) + 13*pow_4(z) + 11*pow_5(z))*hpl(0,-1,0,0))/(z*(-1 + pow_2(z))) + (256*C_A3*(-1 - 2*z - pow_2(z) + 2*pow_3(z) + pow_4(z) + 2*pow_5(z))*hpl(0,-1,0,1))/(z*(-1 + pow_2(z))) + (128*C_A3*(3 - z - 3*pow_2(z) + pow_3(z) + 2*pow_4(z))*hpl(0,0,-1,0))/(-1 + pow_2(z)) - (128*C_A3*(-2 + 7*z + 2*pow_2(z) - 7*pow_3(z) + 2*pow_4(z))*hpl(0,0,0,0))/(-1 + pow_2(z)) - (64*C_A3*(5 + 13*z - 5*pow_2(z) - 13*pow_3(z) + 8*pow_4(z))*hpl(0,0,0,1))/(-1 + pow_2(z)) - (128*C_A3*(2 + z + 6*pow_2(z) - pow_3(z) - 6*pow_4(z) + 3*pow_5(z))*hpl(0,0,1,0))/(z*(-1 + pow_2(z))) - (256*C_A3*pow(1 - z + pow_2(z),2)*hpl(0,0,1,1))/((-1 + z)*z) - (320*C_A3*(1 - z + 3*pow_2(z) - 3*pow_3(z) + pow_4(z))*hpl(0,1,0,0))/((-1 + z)*z) - (256*C_A3*pow(1 - z + pow_2(z),2)*hpl(0,1,0,1))/((-1 + z)*z) - (256*C_A3*pow(1 - z + pow_2(z),2)*hpl(0,1,1,0))/((-1 + z)*z) + (128*C_A3*pow(1 - z + pow_2(z),2)*hpl(1,0,-1,0))/((-1 + z)*z) - (256*C_A3*pow(1 - z + pow_2(z),2)*hpl(1,0,0,0))/((-1 + z)*z) - (256*C_A3*pow(1 - z + pow_2(z),2)*hpl(1,0,0,1))/((-1 + z)*z) - (256*C_A3*pow(1 - z + pow_2(z),2)*hpl(1,0,1,0))/((-1 + z)*z) - (256*C_A3*pow(1 - z + pow_2(z),2)*hpl(1,1,0,0))/((-1 + z)*z) + pow_2(Nf)*((C_A*(944 + 12*(-55 + 8*PI2)*z + 12*(63 + 8*PI2)*pow_2(z) - 944*pow_3(z)))/(81.*z) + ((4*C_A*(76 + 43*z + 52*pow_2(z)))/27. + (32*C_F*(-22 + 68*z + 11*pow_2(z) + 3*PI2*(1 + z)))/27.)*hpl(0) + ((16*C_F*(-1 + z)*(-32 + 85*z + 22*pow_2(z)))/(27.*z) + (4*C_A*(-1 + z)*(52 + 19*z + 52*pow_2(z)))/(27.*z))*hpl(1) + ((-32*C_A*(1 + z))/9. + (16*C_F*(23 + 29*z + 8*pow_2(z)))/9.)*hpl(0,0) + ((-64*C_A*(1 + z))/9. + (32*C_F*(-1 - 4*z + 2*pow_2(z)))/9.)*hpl(0,1) + (32*C_F*(-4 - 3*z + 3*pow_2(z) + 4*pow_3(z))*hpl(1,0))/(9.*z) + (16*C_F*(-4 - 3*z + 3*pow_2(z) + 4*pow_3(z))*hpl(1,1))/(9.*z) - (32*C_F*(1 + z)*hpl(0,0,0))/3. - (64*C_F*(1 + z)*hpl(0,0,1))/3. - (64*C_F*(1 + z)*hpl(0,1,0))/3. - (32*C_F*(1 + z)*hpl(0,1,1))/3. - (16*C_F*(154 + 2*(85 + 3*PI2)*pow_3(z) - 3*z*(84 + PI2 - 18*zeta_3) - 6*pow_2(z)*(12 + 2*PI2 - 9*zeta_3)))/(81.*z)) - (8*C_A3*hpl(-1)*(PI2*pow(1 + z,2)*(11 + 42*z + 11*pow_2(z)) + 144*pow(1 + z + pow_2(z),2)*zeta_3))/(3.*z*(1 + z)) + (8*C_A3*hpl(1)*(413 + 9*PI2*pow(-1 + z,2)*(11 + 14*z + 11*pow_2(z)) + 648*zeta_3 + 6*pow_2(z)*(31 + 324*zeta_3) - 2*z*(253 + 648*zeta_3) - 2*pow_3(z)*(253 + 648*zeta_3) + pow_4(z)*(413 + 648*zeta_3)))/(27.*(-1 + z)*z) + (4*C_A3*hpl(0)*(6*PI2*(22 - 174*z - 125*pow_2(z) - 57*pow_3(z) + 92*pow_4(z) + 242*pow_5(z)) + pow_4(z)*(4687 - 2160*zeta_3) + 8*(-395 + 54*zeta_3) + 6*pow_5(z)*(2651 + 432*zeta_3) + 3*pow_2(z)*(-521 + 720*zeta_3) + z*(-15433 + 2592*zeta_3) - pow_3(z)*(509 + 2592*zeta_3)))/(27.*z*(-1 + pow_2(z))) - (4*C_A3*(18*PI4*(-29 - 70*z - 83*pow_2(z) + 59*pow_3(z) + 72*pow_4(z) + 26*pow_5(z)) - 30*PI2*(389 - 232*z - 194*pow_2(z) + 232*pow_3(z) - 329*pow_4(z) + 134*pow_5(z)) - 5*(1 + z)*(-73091 + z*(154223 - 30996*zeta_3) + pow_3(z)*(147608 - 9504*zeta_3) + 15444*zeta_3 + 3*pow_2(z)*(-51883 + 1620*zeta_3) + pow_4(z)*(-73091 + 19008*zeta_3))))/(405.*z*(-1 + pow_2(z))) + Nf*(((64*C_F2*PI2*(1 + z)*(1 - 4*z + pow_2(z)))/(9.*z) + (8*C_A2*PI2*(1 + z)*(2 - 35*z + 2*pow_2(z)))/(9.*z) - (16*C_A*C_F*PI2*(1 + z)*(4 - 43*z + 4*pow_2(z)))/(9.*z))*hpl(-1) + ((4*C_A2*(-1 + z)*(-410 - 2441*z - 410*pow_2(z) + 18*PI2*(2 + 9*z + 2*pow_2(z))))/(27.*z) - (8*C_A*C_F*(-1 + z)*(-521 - 2219*z + 352*pow_2(z) + 6*PI2*(8 + 35*z + 8*pow_2(z))))/(27.*z) + (8*C_F2*(-1 + z)*(8*PI2*(1 + 4*z + pow_2(z)) + 3*(-31 - 53*z + 54*pow_2(z))))/(9.*z))*hpl(1) + ((2144*C_F2*(1 + z))/3. - (32*C_A*C_F*(1 + z)*(-44 + 275*z + 10*pow_2(z)))/(9.*z) + (16*C_A2*(-114 + 77*z + 342*pow_2(z) + 77*pow_3(z) - 114*pow_4(z)))/(9.*z*(1 + z)))*hpl(-1,0) + (16*C_A2*PI2*(-1 + z) - (128*C_A*C_F*PI2*(-1 + z))/3. + (64*C_F2*PI2*(-1 + z))/3.)*hpl(0,-1) + ((8*C_F2*(-82 - 297*z + 56*pow_2(z) + 4*PI2*(1 + z)))/3. + (8*C_A*C_F*(1007 + 1208*z + 254*pow_2(z) + 36*PI2*(1 + z)))/9. + (8*C_A2*(132 + 605*z - 176*pow_2(z) - 605*pow_3(z) + 124*pow_4(z) - 3*PI2*(-3 - 5*z + 3*pow_2(z) + 5*pow_3(z))))/(9.*(-1 + pow_2(z))))*hpl(0,0) + ((8*C_A*C_F*(24 + (-674 + 48*PI2)*z + (-149 + 48*PI2)*pow_2(z) + 40*pow_3(z)))/(9.*z) + (8*C_A2*(132 + 9*(-53 + 2*PI2)*z + 312*pow_2(z) - (19 + 18*PI2)*pow_3(z) + 132*pow_4(z)))/(9.*(-1 + z)*z) + (8*C_F2*(48 + 15*z + 16*pow_2(z) - 8*PI2*(1 + z)))/3.)*hpl(0,1) + ((16*C_F2*(-1 + z)*(10 + 33*z + 28*pow_2(z)))/(3.*z) + (16*C_A*C_F*(-1 + z)*(-17 + 79*z + 37*pow_2(z)))/(9.*z) + (32*C_A2*(33 - 62*z + 78*pow_2(z) - 62*pow_3(z) + 33*pow_4(z)))/(9.*(-1 + z)*z))*hpl(1,0) + ((-8*C_A*C_F*(-1 + z)*(4 + 31*z + 4*pow_2(z)))/(9.*z) + (8*C_F2*(-1 + z)*(16 + 23*z + 16*pow_2(z)))/(3.*z))*hpl(1,1) + ((256*C_F2*(1 + z)*(1 - 4*z + pow_2(z)))/(3.*z) + (32*C_A2*(1 + z)*(2 - 9*z + 2*pow_2(z)))/z - (64*C_A*C_F*(1 + z)*(8 - 35*z + 8*pow_2(z)))/(3.*z))*hpl(-1,-1,0) + ((-128*C_F2*(1 + z)*(1 - 4*z + pow_2(z)))/(3.*z) - (32*C_A*C_F*(1 + z)*(2 + 25*z + 2*pow_2(z)))/(3.*z) - (16*C_A2*(1 + z)*(4 - 31*z + 4*pow_2(z)))/(3.*z))*hpl(-1,0,0) + ((64*C_A2*pow(1 + z,3))/(3.*z) - (128*C_A*C_F*pow(1 + z,3))/(3.*z))*hpl(-1,0,1) + ((-256*C_F2*(-3 + pow_2(z)))/3. + (32*C_A*C_F*(-39 + 4/z - 21*z + 12*pow_2(z)))/3. - (16*C_A2*(-4 + 55*z - 64*pow_2(z) + pow_3(z) + 8*pow_4(z)))/(3.*(-1 + z)*z))*hpl(0,-1,0) + (C_A2*(64 - (304*z)/3.) - (16*C_A*C_F*(-29 + 70*z))/3. + (16*C_F2*(-9 - 33*z + 4*pow_2(z)))/3.)*hpl(0,0,0) + (-32*C_F2*(4 + 5*z) + (16*C_A*C_F*(-8 + 19*z + 12*pow_2(z)))/3. - (8*C_A2*(-3 - 54*z + 45*pow_2(z) + 8*pow_3(z)))/(3.*(-1 + z)))*hpl(0,0,1) + ((-64*C_A2*(1 + z))/3. - (32*C_F2*(15 + 24*z + 4*pow_2(z)))/3. + (16*C_A*C_F*(-8 - 2*z + pow_2(z) + 4*pow_3(z)))/(3.*z))*hpl(0,1,0) + ((-32*C_F2*pow(3 + 2*z,2))/3. + (16*C_A*C_F*(-4 + 11*z + 17*pow_2(z) + 4*pow_3(z)))/(3.*z))*hpl(0,1,1) + ((-16*C_F2*(-1 + z)*(4 + 25*z + 4*pow_2(z)))/(3.*z) + (16*C_A*C_F*(-1 + z)*(16 + 55*z + 16*pow_2(z)))/(3.*z) - (8*C_A2*(25 - 54*z + 25*pow_2(z)))/(3.*(-1 + z)))*hpl(1,0,0) + ((32*C_A*C_F*(-4 - 3*z + 3*pow_2(z) + 4*pow_3(z)))/(3.*z) - (32*C_F2*(-4 - 3*z + 3*pow_2(z) + 4*pow_3(z)))/(3.*z))*hpl(1,1,0) + ((32*C_A*C_F*(-4 - 3*z + 3*pow_2(z) + 4*pow_3(z)))/(3.*z) - (32*C_F2*(-4 - 3*z + 3*pow_2(z) + 4*pow_3(z)))/(3.*z))*hpl(1,1,1) + (192*C_A2*(-1 + z) - 512*C_A*C_F*(-1 + z) + 256*C_F2*(-1 + z))*hpl(0,-1,-1,0) + (-96*C_A2*(-1 + z) + 128*C_A*C_F*(-1 + z) - 128*C_F2*(-1 + z))*hpl(0,-1,0,0) + (-256*C_A*C_F + 256*C_F2 - 32*C_A2*(-5 + z))*hpl(0,0,-1,0) + (128*C_A*C_F*(2 - 3*z) + 32*C_A2*z - 64*C_F2*(1 + z))*hpl(0,0,0,0) + (48*C_A2*(1 + z) - 192*C_A*C_F*(1 + z) - 64*C_F2*(1 + z))*hpl(0,0,0,1) - 128*C_A*C_F*(1 + z)*hpl(0,0,1,0) + (-32*C_A*C_F*(1 + z) + 32*C_F2*(1 + z))*hpl(0,0,1,1) + (48*C_A2*(1 + z) - 224*C_A*C_F*(1 + z) + 96*C_F2*(1 + z))*hpl(0,1,0,0) + (-64*C_A*C_F*(1 + z) + 64*C_F2*(1 + z))*hpl(0,1,1,0) + (-64*C_A*C_F*(1 + z) + 64*C_F2*(1 + z))*hpl(0,1,1,1) + (4*C_F2*(-66 + (-1275 - 48*PI2 + 13*PI4)*z + pow_2(z)*(297 + 253*PI2 + 9*PI4 - 144*zeta_3) + 96*zeta_3 - 4*pow_3(z)*(-261 + 4*PI2 + 96*zeta_3)))/(9.*z) + (2*C_A2*(27*PI4*(23 + 13*z) - (30*PI2*(96 + 481*z + 156*pow_2(z) - 137*pow_3(z) + 132*pow_4(z)))/(z*(1 + z)) + 10*(-9783 + (9632 - 2376*zeta_3)/z + 216*(31 + 1/(-1 + z))*zeta_3 + 32*pow_2(z)*(-301 + 54*zeta_3) + 3*z*(3679 + 504*zeta_3))))/405. - (4*C_A*C_F*(9*PI4*z*(83 + 43*z) + 15*PI2*(-152 - 674*z + 991*pow_2(z) + 40*pow_3(z)) - 5*(-15331 + z*(28806 - 6264*zeta_3) + 3024*zeta_3 + pow_3(z)*(-12263 + 432*zeta_3) - 3*pow_2(z)*(-91 + 5436*zeta_3))))/(405.*z) + hpl(0)*((4*C_F2*(-381 + 108*pow_2(z) + 4*PI2*(4 + 5*z) + 168*zeta_3 - z*(541 + 24*zeta_3)))/3. - (8*C_A*C_F*(344 - 756*pow_3(z) + 3*PI2*(-8 - 8*z + 61*pow_2(z) + 12*pow_3(z)) + z*(-805 + 108*zeta_3) - pow_2(z)*(3451 + 972*zeta_3)))/(27.*z) + (4*C_A2*(3*PI2*(8 - 3*z - 39*pow_2(z) - 9*pow_3(z) + 35*pow_4(z) + 8*pow_5(z)) + (1 + z)*(-568 - 3132*z + 3918*pow_4(z) + 9*pow_2(z)*(635 + 144*zeta_3) - pow_3(z)*(5897 + 1296*zeta_3))))/(27.*z*(-1 + pow_2(z))))))
				+PD<-1>(fz,f1,p)*(((29*C_A)/9. + (22*C_F)/9.)*pow_2(Nf) + Nf*((-241*C_A*C_F)/9. + 2*C_F2 + (C_A2*(-699 - 24*PI2 - 2*PI4 - 1440*zeta_3))/27.) + C_A3*(79 + (11*PI4)/27. + (1072*zeta_3)/3. - (8*PI2*(-1 + 6*zeta_3))/9. - 160*zeta_5))
				+PD<+0>(fz,f1,p)*((-32*C_A*pow_2(Nf))/27. + (4*C_A3*(-1340*PI2 + 66*PI4 + 45*(245 + 44*zeta_3)))/135. + Nf*((8*C_A2*(-209 + 20*PI2 - 252*zeta_3))/27. + C_A*C_F*(-73.33333333333333 + 64*zeta_3)))
	       	);
}

template<>
inline double APSF<e_g,e_q,2>(const double fz, const double f1, const APSF_Parameter &p, const int Nf){
	const double z(p.z);
	const HPLz &hpl(p.hpl);
	return	(
				fz*(((208*C_A2*C_F*PI2*(2 + 2*z + pow_2(z)))/(3.*z) - (80*C_A*C_F2*PI2*(2 + 2*z + pow_2(z)))/(3.*z))*hpl(-1,-1) + ((-8*C_A*C_F2*(24 + 456*z + 525*pow_2(z) + 2*PI2*(2 + 2*z + pow_2(z))))/(3.*z) - (8*C_A2*C_F*(218 + 230*z - 53*pow_2(z) + 120*pow_3(z) + 20*PI2*(2 + 2*z + pow_2(z))))/(3.*z) + (32*C_F3*(PI2*(2 + 2*z + pow_2(z)) + 3*(7 + 43*z + 38*pow_2(z))))/(3.*z))*hpl(-1,0) + (32*C_A*C_F2*PI2 - (64*C_F3*PI2)/3. - (8*C_A2*C_F*PI2*(20 + 54*z + 7*pow_2(z)))/(3.*z))*hpl(0,-1) + (-2*C_F3*(70 + 4*PI2*(-2 + z) + 555*z) - (2*C_A*C_F2*(2590 + (-4631 + 24*PI2)*z + 448*pow_2(z)))/9. - (4*C_A2*C_F*(PI2*(78 + 69*z) + 4*(635 - 49*z + 77*pow_2(z))))/9.)*hpl(0,0) + ((8*C_F3*(123 + 2*PI2*(-6 + z) + 66*z))/3. - (16*C_A*C_F2*(-144 + 307*z + 118*pow_2(z) + 108*pow_3(z) + 6*PI2*(1 - 5*z + pow_2(z))))/(9.*z) - (8*C_A2*C_F*(z*(142 - 17*z - 416*pow_2(z)) + 3*PI2*(8 + 10*z + 7*pow_2(z))))/(9.*z))*hpl(0,1) + ((8*C_A2*C_F*(-213 + 274*z + 100*pow_2(z) + 290*pow_3(z) - 12*PI2*(2 - 2*z + pow_2(z))))/(9.*z) - (16*C_F3*(21 + 6*z + 9*pow_2(z) + 2*PI2*(2 - 2*z + pow_2(z))))/(3.*z) + (8*C_A*C_F2*(191 - 168*z + 20*pow_2(z) + 2*PI2*(2 - 2*z + pow_2(z))))/(3.*z))*hpl(1,0) + ((4*C_A2*C_F*(390 - 50*z + 457*pow_2(z) + 296*pow_3(z) - 12*PI2*(2 - 2*z + pow_2(z))))/(9.*z) + (4*C_F3*(-46 + 48*z + 27*pow_2(z) + 4*PI2*(2 - 2*z + pow_2(z))))/z - (8*C_A*C_F2*(-16 + 221*z + 362*pow_2(z) + 104*pow_3(z) + 12*PI2*(2 - 2*z + pow_2(z))))/(9.*z))*hpl(1,1) + pow_2(Nf)*((32*C_F*(-1 + z + 2*pow_2(z)))/(9.*z) - (32*C_F*(5 - 5*z + 4*pow_2(z))*hpl(1))/(9.*z) + (16*C_F*(2 - 2*z + pow_2(z))*hpl(1,1))/(3.*z)) + (32*C_A*C_F2*(30 + 6/z + 19*z) - (128*C_F3*(3 + 7*z + 5*pow_2(z)))/z + (32*C_A2*C_F*(57 + 11*z - 11*pow_2(z) + 4*pow_3(z)))/(3.*z))*hpl(-1,-1,0) + (64*C_F3*(12 + 6/z + 7*z) - (96*C_A*C_F2*(1 + 5*z + 3*pow_2(z)))/z + (16*C_A2*C_F*(98 + 145*z + 29*pow_2(z) + 16*pow_3(z)))/(3.*z))*hpl(-1,0,0) + ((64*C_A*C_F2*(3 + 3*z + 2*pow_2(z)))/z + (32*C_A2*C_F*(9 + 38*z + 19*pow_2(z) + 4*pow_3(z)))/(3.*z))*hpl(-1,0,1) + (160*C_F3*(6 + z) - 32*C_A*C_F2*(36 + z) + (16*C_A2*C_F*(-22 + 145*z + 6*pow_2(z) + 24*pow_3(z)))/(3.*z))*hpl(0,-1,0) + (C_F3*(32 - 264*z) - (32*C_A2*C_F*(17 + 42*z + 14*pow_2(z)))/3. - (16*C_A*C_F2*(95 - 76*z + 16*pow_2(z)))/3.)*hpl(0,0,0) + ((16*C_A*C_F2*(94 + 49*z))/3. + C_F3*(-128 + 88*z) - (8*C_A2*C_F*(535 + 245*z + 72*pow_2(z)))/3.)*hpl(0,0,1) + (96*C_F3*z + (32*C_A*C_F2*(-1 + 18/z + 14*z))/3. - (32*C_A2*C_F*(2 + 49*z + 37*pow_2(z) + 14*pow_3(z)))/(3.*z))*hpl(0,1,0) + (-8*C_F3*(4 + 13*z) + (32*C_A*C_F2*(27 - 11*z + 28*pow_2(z) + 6*pow_3(z)))/(3.*z) - (8*C_A2*C_F*(136 - 78*z + 99*pow_2(z) + 32*pow_3(z)))/(3.*z))*hpl(0,1,1) + (24*C_F3*(16 - 4/z - 9*z) + C_A*C_F2*(-768 + 256/z + 464*z) - (8*C_A2*C_F*(-132 + 35*z + 83*pow_2(z) + 16*pow_3(z)))/(3.*z))*hpl(1,0,0) + ((8*C_A2*C_F*(110 - 110/z - 103*z))/3. + 16*C_F3*(4 + z) + (8*C_A*C_F2*(82 - 106*z + 83*pow_2(z)))/(3.*z))*hpl(1,0,1) + ((8*C_A2*C_F*(110 - 110/z - 103*z))/3. + (64*C_F3*(3 - 3*z + 2*pow_2(z)))/z + (8*C_A*C_F2*(2 - 2*z + 37*pow_2(z)))/(3.*z))*hpl(1,1,0) + (8*C_F3*(40 - 36/z - 31*z) + (16*C_A*C_F2*(140 - 139*z + 89*pow_2(z) - 4*pow_3(z)))/(3.*z) + (8*C_A2*C_F*(-172 + 158*z - 85*pow_2(z) + 8*pow_3(z)))/(3.*z))*hpl(1,1,1) + ((192*C_A2*C_F*(2 + 2*z + pow_2(z)))/z - (192*C_A*C_F2*(2 + 2*z + pow_2(z)))/z)*hpl(-1,-1,-1,0) + ((-384*C_A2*C_F*(2 + 2*z + pow_2(z)))/z + (128*C_A*C_F2*(2 + 2*z + pow_2(z)))/z - (128*C_F3*(2 + 2*z + pow_2(z)))/z)*hpl(-1,-1,0,0) + ((-320*C_A2*C_F*(2 + 2*z + pow_2(z)))/z + (64*C_A*C_F2*(2 + 2*z + pow_2(z)))/z)*hpl(-1,-1,0,1) + ((-160*C_A2*C_F*(2 + 2*z + pow_2(z)))/z + (96*C_A*C_F2*(2 + 2*z + pow_2(z)))/z - (64*C_F3*(2 + 2*z + pow_2(z)))/z)*hpl(-1,0,-1,0) + (48*C_A*C_F2*(2 + 2/z + z) + 32*C_F3*(2 + 2/z + z) + (176*C_A2*C_F*(2 + 2*z + pow_2(z)))/z)*hpl(-1,0,0,0) + (256*C_A2*C_F*(2 + 2*z + pow_2(z))*hpl(-1,0,0,1))/z + (64*C_A2*C_F*(2 + 2*z + pow_2(z))*hpl(-1,0,1,0))/z + ((64*C_A2*C_F*(2 + 2*z + pow_2(z)))/z - (64*C_A*C_F2*(2 + 2*z + pow_2(z)))/z)*hpl(-1,0,1,1) + (384*C_A*C_F2 - 256*C_F3 + (32*C_A2*C_F*(-4 - 22*z + pow_2(z)))/z)*hpl(0,-1,-1,0) + (-192*C_A*C_F2 + 64*C_F3*(4 + z) + C_A2*C_F*(352 + 448/z + 304*z))*hpl(0,-1,0,0) + (128*C_A2*C_F*(2 + 4*z + pow_2(z))*hpl(0,-1,0,1))/z + (512*C_F3 + 16*C_A2*C_F*(-18 + z) + 32*C_A*C_F2*(-14 + z))*hpl(0,0,-1,0) + (128*C_A*C_F2*(1 + z) - 64*C_F3*(1 + z) + C_A2*C_F*(-256 + 240*z))*hpl(0,0,0,0) + (48*C_F3*(-2 + z) + 64*C_A*C_F2*z + 8*C_A2*C_F*(26 + 25*z))*hpl(0,0,0,1) + (64*C_A*C_F2*(-2 + z) + (128*C_A2*C_F*(2 + z + 2*pow_2(z)))/z)*hpl(0,0,1,0) + (-48*C_F3*(-2 + z) - 16*C_A*C_F2*(14 + z) + (64*C_A2*C_F*(4 - 2*z + 3*pow_2(z)))/z)*hpl(0,0,1,1) + (-16*C_F3*(-10 + z) + 8*C_A2*C_F*(26 + 24/z + 21*z) + (64*C_A*C_F2*(2 - 7*z + 2*pow_2(z)))/z)*hpl(0,1,0,0) + (-32*C_F3*(-2 + z) + (64*C_A*C_F2*pow(-1 + z,2))/z + (96*C_A2*C_F*(2 - 2*z + pow_2(z)))/z)*hpl(0,1,0,1) + (32*C_A*C_F2*(-2 + 2/z + z) + (96*C_A2*C_F*(2 - 2*z + pow_2(z)))/z)*hpl(0,1,1,0) + (-16*C_F3*(-2 + z) - (16*C_A*C_F2*(8 - 14*z + pow_2(z)))/z + (32*C_A2*C_F*(4 - 8*z + pow_2(z)))/z)*hpl(0,1,1,1) + (C_A2*C_F*(64 - 64/z - 32*z) + (160*C_A*C_F2*(2 - 2*z + pow_2(z)))/z - (192*C_F3*(2 - 2*z + pow_2(z)))/z)*hpl(1,0,-1,0) + (48*C_A2*C_F*(-2 + 2/z + z) + (80*C_A*C_F2*(2 - 2*z + pow_2(z)))/z)*hpl(1,0,0,0) + (128*C_A2*C_F*(2 - 2*z + pow_2(z))*hpl(1,0,0,1))/z + ((192*C_A2*C_F*(2 - 2*z + pow_2(z)))/z - (64*C_F3*(2 - 2*z + pow_2(z)))/z)*hpl(1,0,1,0) + ((128*C_A2*C_F*(2 - 2*z + pow_2(z)))/z - (64*C_A*C_F2*(2 - 2*z + pow_2(z)))/z - (64*C_F3*(2 - 2*z + pow_2(z)))/z)*hpl(1,0,1,1) + ((64*C_A2*C_F*(2 - 2*z + pow_2(z)))/z + (160*C_A*C_F2*(2 - 2*z + pow_2(z)))/z - (96*C_F3*(2 - 2*z + pow_2(z)))/z)*hpl(1,1,0,0) + (C_A*C_F2*(64 - 64/z - 32*z) + (128*C_A2*C_F*(2 - 2*z + pow_2(z)))/z - (96*C_F3*(2 - 2*z + pow_2(z)))/z)*hpl(1,1,0,1) + (C_F3*(64 - 64/z - 32*z) + (128*C_A2*C_F*(2 - 2*z + pow_2(z)))/z - (96*C_A*C_F2*(2 - 2*z + pow_2(z)))/z)*hpl(1,1,1,0) + ((64*C_A2*C_F*(2 - 2*z + pow_2(z)))/z - (128*C_A*C_F2*(2 - 2*z + pow_2(z)))/z + (64*C_F3*(2 - 2*z + pow_2(z)))/z)*hpl(1,1,1,1) + hpl(-1)*((-32*C_F3*PI2*(3 + 7*z + 5*pow_2(z)))/(3.*z) + (8*C_A*C_F2*(PI2*(-6 + 18*z + 11*pow_2(z)) + 66*(2 + 2*z + pow_2(z))*zeta_3))/(3.*z) - (8*C_A2*C_F*(PI2*(-39 + 65*z + 49*pow_2(z) + 4*pow_3(z)) + 414*(2 + 2*z + pow_2(z))*zeta_3))/(9.*z)) + (C_F3*(PI4*(512 + 744*z + 60*pow_2(z)) + 120*PI2*(28 - 41*z + 130*pow_2(z)) + 45*(-8*(47 + 120*zeta_3) + pow_2(z)*(-363 + 832*zeta_3) + 2*z*(475 + 832*zeta_3))))/(90.*z) + hpl(0)*((2*C_F3*(273 + 2*PI2*(16 + 9*z) + 576*zeta_3 - 3*z*(245 + 48*zeta_3)))/3. + (4*C_A2*C_F*(3160 + 4672*pow_3(z) + 3*PI2*(-44 + 535*z + 257*pow_2(z) + 72*pow_3(z)) + pow_2(z)*(1601 - 1080*zeta_3) - 432*zeta_3 - 8*z*(-2389 + 378*zeta_3)))/(27.*z) + (2*C_A*C_F2*(-9925 + 2680*pow_2(z) - 12*PI2*(94 + 55*z) - 1728*zeta_3 + z*(7061 + 1728*zeta_3)))/27.) + (C_A2*C_F*(-9*PI4*(272 + 386*z + 183*pow_2(z)) - 120*PI2*(327 - 71*z - 71*pow_2(z) + 208*pow_3(z)) + 10*(138305 - 27216*zeta_3 + 16*pow_3(z)*(-2105 + 432*zeta_3) + 27*pow_2(z)*(1789 + 700*zeta_3) + 18*z*(-6355 + 2136*zeta_3))))/(405.*z) + (C_A*C_F2*(-24*PI4*(54 + 98*z + 23*pow_2(z)) + 40*PI2*(-360 + 614*z - 1339*pow_2(z) + 216*pow_3(z)) - 15*(288*pow_3(z)*(-7 + 4*zeta_3) - 36*(163 + 216*zeta_3) + 2*z*(11629 + 8832*zeta_3) + pow_2(z)*(-3589 + 10608*zeta_3))))/(270.*z) + hpl(1)*((4*C_A*C_F2*(6503 + 692*pow_3(z) + PI2*(-354 + 858*z - 591*pow_2(z)) + 648*zeta_3 + 4*pow_2(z)*(1367 + 81*zeta_3) - 2*z*(4037 + 324*zeta_3)))/(27.*z) + (4*C_A2*C_F*(-6758 - 872*pow_3(z) + 3*PI2*(-4 - 88*z + 125*pow_2(z) + 8*pow_3(z)) + z*(3130 - 1512*zeta_3) + 1512*zeta_3 + pow_2(z)*(-383 + 756*zeta_3)))/(27.*z) + (4*C_F3*(PI2*(24 - 64*z + 38*pow_2(z)) - 3*(47 + 128*zeta_3 - 2*z*(79 + 64*zeta_3) + pow_2(z)*(129 + 64*zeta_3))))/(3.*z)) + Nf*((-64*C_A*C_F*PI2*(2 + 2*z + pow_2(z))*hpl(-1))/(9.*z) + ((-4*C_F2*(760 + 2587*z + 1567*pow_2(z) + 176*pow_3(z) - 12*PI2*(4 + 2*z + 5*pow_2(z))))/(27.*z) + (16*C_A*C_F*(151 + 832*z + 211*pow_2(z) + 250*pow_3(z) - 3*PI2*(2 + 4*z + 5*pow_2(z))))/(27.*z))*hpl(0) + ((8*C_A*C_F*(502 - 296*z + 430*pow_2(z) + 100*pow_3(z) - 3*PI2*(2 - 2*z + pow_2(z))))/(27.*z) - (16*C_F2*(7 - z + 170*pow_2(z) + 3*PI2*(2 - 2*z + pow_2(z))))/(27.*z))*hpl(1) + ((32*C_F2*(31 - 27*pow_2(z) + 4*pow_3(z)))/(9.*z) - (16*C_A*C_F*(82 + 30*z - 9*pow_2(z) + 4*pow_3(z)))/(9.*z))*hpl(-1,0) + ((-8*C_A*C_F*(272 + 167*z + 8*pow_2(z)))/9. + (4*C_F2*(1118 + 653*z + 32*pow_2(z)))/9.)*hpl(0,0) + ((256*C_F2*(1 + z))/9. - (8*C_A*C_F*(80 - 64*z + 59*pow_2(z) + 24*pow_3(z)))/(9.*z))*hpl(0,1) + ((-16*C_F2*(4 + pow_2(z)))/(3.*z) - (80*C_A*C_F*(8 - 8*z + 7*pow_2(z)))/(9.*z))*hpl(1,0) + ((16*C_F2*(83 - 89*z + 64*pow_2(z)))/(9.*z) + (16*C_A*C_F*(-117 + 110*z - 61*pow_2(z) + 4*pow_3(z)))/(9.*z))*hpl(1,1) - (128*C_A*C_F*(2 + 2*z + pow_2(z))*hpl(-1,-1,0))/(3.*z) + (64*C_A*C_F*(2 + 2*z + pow_2(z))*hpl(-1,0,0))/(3.*z) + (64*C_A*C_F*(2 + 2*z + pow_2(z))*hpl(-1,0,1))/(3.*z) + ((32*C_A*C_F*(10 - 2/z - 3*z))/3. + (64*C_F2*(-6 + 2/z + 3*z))/3.)*hpl(0,-1,0) + ((64*C_A*C_F)/3. + (32*C_F2*(1 - 2*z))/3.)*hpl(0,0,0) + ((32*C_F2*(-2 + z))/3. + (64*C_A*C_F*(2 + z))/3.)*hpl(0,0,1) + ((32*C_F2*(-2 + z))/3. + (32*C_A*C_F*(2 - 2*z + pow_2(z)))/(3.*z))*hpl(0,1,0) + ((16*C_F2*(-2 + z))/3. + C_A*C_F*(-96 + 160/(3.*z) + 16*z))*hpl(0,1,1) + (16*C_F2*(-2 + 2/z + z) - (32*C_A*C_F*(2 - 2*z + pow_2(z)))/(3.*z))*hpl(1,0,0) + ((80*C_A*C_F*(2 - 2*z + pow_2(z)))/(3.*z) + (32*C_F2*(2 - 2*z + pow_2(z)))/(3.*z))*hpl(1,0,1) + ((80*C_A*C_F*(2 - 2*z + pow_2(z)))/(3.*z) + (64*C_F2*(2 - 2*z + pow_2(z)))/(3.*z))*hpl(1,1,0) + ((80*C_A*C_F*(2 - 2*z + pow_2(z)))/(3.*z) - (80*C_F2*(2 - 2*z + pow_2(z)))/(3.*z))*hpl(1,1,1) - 64*C_F2*(-2 + z)*hpl(0,0,0,0) + (C_F2*(-19500 - 1344*pow_3(z) - 16*PI2*(-31 + 8*z + 35*pow_2(z)) + z*(17414 - 8928*zeta_3) + 5184*zeta_3 + pow_2(z)*(3301 + 4464*zeta_3)))/(27.*z) + (4*C_A*C_F*(PI2*(-84 - 64*z + 77*pow_2(z) + 24*pow_3(z)) - 3*(-967 + 524*pow_3(z) + 360*zeta_3 + 6*pow_2(z)*(-37 + 46*zeta_3) - 8*z*(-145 + 84*zeta_3))))/(27.*z)))
	       	);
}

template<>
inline double APSF<e_q,e_g,2>(const double fz, const double f1, const APSF_Parameter &p, const int Nf){
	const double z(p.z);
	const HPLz &hpl(p.hpl);
	return	(
				fz*(((8*C_A2*PI2*(1 + 2*z + 2*pow_2(z)))/3. - 24*C_A*C_F*PI2*(1 + 2*z + 2*pow_2(z)))*hpl(-1,-1) + (-16*C_F2*(20 + 23*z + 5*pow_2(z)) + (8*C_A2*(104 - 88*z + 733*pow_2(z) + 831*pow_3(z)))/(9.*z) + 8*C_A*C_F*(37 + 19*z - 18*pow_2(z) + PI2*(3 + 6*z + 6*pow_2(z))))*hpl(-1,0) + ((16*C_F2*PI2)/3. + (4*C_A2*PI2*(7 - 2*z + 8*pow_2(z)))/3. + (8*C_A*C_F*PI2*(7 + 8*z + 18*pow_2(z)))/3.)*hpl(0,-1) + ((2*C_A2*(1198 + 900*z + 5122*pow_2(z) + PI2*(-9 + 42*z - 72*pow_2(z))))/9. + (C_A*C_F*(-1267 - 3190*z - 2204*pow_2(z) + 48*PI2*(3 + 3*z + 2*pow_2(z))))/9. + (C_F2*(-28*PI2*(1 - 2*z + 4*pow_2(z)) + 3*(89 + 126*z + 352*pow_2(z))))/3.)*hpl(0,0) + ((4*C_A2*(80 + (471 + 9*PI2)*z + 6*(157 + 5*PI2)*pow_2(z) + 155*pow_3(z)))/(9.*z) + (4*C_F2*(51 - 312*z + 183*pow_2(z) - 4*PI2*(1 - 4*z + 5*pow_2(z))))/3. + (4*C_A*C_F*(-172 + 275*z - 882*pow_2(z) + 6*PI2*(1 - 4*z + 8*pow_2(z))))/9.)*hpl(0,1) + ((4*C_A*C_F*(92 + (-29 + 54*PI2)*z + (472 - 108*PI2)*pow_2(z) + 9*(-53 + 12*PI2)*pow_3(z)))/(9.*z) + (C_A2*(72 + (92 - 40*PI2)*z + 16*(43 + 5*PI2)*pow_2(z) - 4*(207 + 20*PI2)*pow_3(z)))/(3.*z) - (8*C_F2*(7*PI2*(1 - 2*z + 2*pow_2(z)) - 6*(10 - 22*z + 17*pow_2(z))))/3.)*hpl(1,0) + ((-4*C_A2*(40 + (41 + 6*PI2)*z + (476 - 12*PI2)*pow_2(z) + 3*(-141 + 4*PI2)*pow_3(z)))/(9.*z) + (8*C_A*C_F*(46 + (-101 + 18*PI2)*z + (571 - 36*PI2)*pow_2(z) + 4*(-128 + 9*PI2)*pow_3(z)))/(9.*z) - (4*C_F2*(10*PI2*(1 - 2*z + 2*pow_2(z)) - 3*(27 - 74*z + 61*pow_2(z))))/3.)*hpl(1,1) + (128*C_F2*pow(1 + z,2) - 16*C_A*C_F*(1 + z)*(9 + 5*z) - (16*C_A2*(1 + z)*(4 - 25*z + 40*pow_2(z)))/(3.*z))*hpl(-1,-1,0) + (-32*C_F2*(4 + 9*z + 5*pow_2(z)) + 8*C_A*C_F*(5 + 8*z + 6*pow_2(z)) - (8*C_A2*(16 + 32*z + 127*pow_2(z) + 122*pow_3(z)))/(3.*z))*hpl(-1,0,0) + (-16*C_A*C_F*(1 + z)*(7 + 5*z) - (64*C_A2*(1 + z)*(1 + 2*z + 4*pow_2(z)))/(3.*z))*hpl(-1,0,1) + (16*C_A*C_F*(13 - 4*z + 5*pow_2(z)) - 16*C_F2*(11 + 2*z + 8*pow_2(z)) - 8*C_A2*(9 - 27*z + 50*pow_2(z)))*hpl(0,-1,0) + ((8*C_A2*(-7 + 7*z + 36*pow_2(z)))/3. + 4*C_F2*(3 + 16*z + 60*pow_2(z)) + (4*C_A*C_F*(11 + 86*z + 316*pow_2(z)))/3.)*hpl(0,0,0) + (4*C_F2*(9 - 40*z + 44*pow_2(z)) + (4*C_A2*(33 + 81*z + 62*pow_2(z)))/3. + C_A*C_F*(40 + 296*z + (1384*pow_2(z))/3.))*hpl(0,0,1) + (C_A2*(16 + 96*z - (40*pow_2(z))/3.) + 16*C_F2*(3 - 5*z + 5*pow_2(z)) + 8*C_A*C_F*(4 + 9*z + 49*pow_2(z)))*hpl(0,1,0) + (4*C_F2*(9 - 26*z + 24*pow_2(z)) + (4*C_A*C_F*(-11 + 124*z + 144*pow_2(z)))/3. - (16*C_A2*(-2 + 6*pow_2(z) + 53*pow_3(z)))/(3.*z))*hpl(0,1,1) + ((8*C_A*C_F*(4 + 59*z - 61*pow_2(z)))/(3.*z) + 16*C_F2*(-2 - 3*z + 5*pow_2(z)) + (4*C_A2*(-24 - 76*z - 37*pow_2(z) + 148*pow_3(z)))/(3.*z))*hpl(1,0,0) + (16*C_F2*(2 - 13*z + 11*pow_2(z)) - 4*C_A*C_F*(5 - 38*z + 30*pow_2(z)) - (4*C_A2*(11 - 46*z + 46*pow_2(z)))/3.)*hpl(1,0,1) + ((4*C_A2*(23 + 16/z + 98*z - 126*pow_2(z)))/3. + 16*C_F2*(-2 - 3*z + 5*pow_2(z)) + (4*C_A*C_F*(3 - 16/z - 66*z + 70*pow_2(z)))/3.)*hpl(1,1,0) + (C_F2*(-4 - 80*z + 96*pow_2(z)) + (4*C_A2*(8 + 17*z - 46*pow_2(z) + 32*pow_3(z)))/(3.*z) - (8*C_A*C_F*(4 + 7*z - 53*pow_2(z) + 52*pow_3(z)))/(3.*z))*hpl(1,1,1) + (96*C_A2*(1 + 2*z + 2*pow_2(z)) - 96*C_A*C_F*(1 + 2*z + 2*pow_2(z)))*hpl(-1,-1,-1,0) + (32*C_A2*(1 + 2*z + 2*pow_2(z)) + 96*C_A*C_F*(1 + 2*z + 2*pow_2(z)) + 64*C_F2*(1 + 2*z + 2*pow_2(z)))*hpl(-1,-1,0,0) + (32*C_A2*(1 + 2*z + 2*pow_2(z)) + 96*C_A*C_F*(1 + 2*z + 2*pow_2(z)))*hpl(-1,-1,0,1) + (-16*C_A2*(1 + 2*z + 2*pow_2(z)) + 48*C_A*C_F*(1 + 2*z + 2*pow_2(z)) + 32*C_F2*(1 + 2*z + 2*pow_2(z)))*hpl(-1,0,-1,0) + (-24*C_A2*(1 + 2*z + 2*pow_2(z)) - 88*C_A*C_F*(1 + 2*z + 2*pow_2(z)) - 16*C_F2*(1 + 2*z + 2*pow_2(z)))*hpl(-1,0,0,0) + (-32*C_A2*(1 + 2*z + 2*pow_2(z)) - 96*C_A*C_F*(1 + 2*z + 2*pow_2(z)))*hpl(-1,0,0,1) - 32*C_A*C_F*(1 + 2*z + 2*pow_2(z))*hpl(-1,0,1,0) + (32*C_A2*(1 + 2*z + 2*pow_2(z)) - 32*C_A*C_F*(1 + 2*z + 2*pow_2(z)))*hpl(-1,0,1,1) + (64*C_F2 + C_A2*(48 - 160*z) + 32*C_A*C_F*(1 + 4*z + 6*pow_2(z)))*hpl(0,-1,-1,0) + (-64*C_F2*(1 + z + 2*pow_2(z)) - 8*C_A2*(-3 + 26*z + 4*pow_2(z)) - 32*C_A*C_F*(2 + 3*z + 6*pow_2(z)))*hpl(0,-1,0,0) + (-32*C_A2*(1 + 2*z + 2*pow_2(z)) - 32*C_A*C_F*(3 + 2*z + 6*pow_2(z)))*hpl(0,-1,0,1) + (C_A*C_F*(48 - 32*z) + C_A2*(72 - 16*z) - 64*C_F2*(1 + 2*pow_2(z)))*hpl(0,0,-1,0) + (C_A2*(64 - 240*z) - 32*C_A*C_F*(1 + 2*z) + 16*C_F2*(1 + 8*pow_2(z)))*hpl(0,0,0,0) + (-32*C_A*C_F*(3 + 4*z + 2*pow_2(z)) + 56*C_F2*(1 - 2*z + 4*pow_2(z)) + 12*C_A2*(1 - 6*z + 8*pow_2(z)))*hpl(0,0,0,1) + (-64*C_A*C_F*(1 + 3*z) + 64*C_F2*(1 - 2*z + 3*pow_2(z)))*hpl(0,0,1,0) + (16*C_A2*(3 + 8*z) - 32*C_A*C_F*(2 + 3*z + 2*pow_2(z)) + 16*C_F2*(5 - 10*z + 12*pow_2(z)))*hpl(0,0,1,1) + (16*C_F2*(1 - 6*z + 8*pow_2(z)) - 4*C_A2*(21 + 50*z + 16*pow_2(z)) + C_A*C_F*(88 + 16*z + 96*pow_2(z)))*hpl(0,1,0,0) + (-32*C_A*C_F*pow_2(z) + 32*C_F2*(2 - 4*z + 5*pow_2(z)))*hpl(0,1,0,1) + (32*C_F2*pow(-1 + z,2) + 96*C_A*C_F*(-2 + z)*z + 32*C_A2*(1 + 4*z))*hpl(0,1,1,0) + (C_A2*(16 + 64*z) + 8*C_F2*(5 - 10*z + 8*pow_2(z)) - 8*C_A*C_F*(7 - 2*z + 8*pow_2(z)))*hpl(0,1,1,1) + (-48*C_A2*(1 - 2*z + 2*pow_2(z)) + 48*C_A*C_F*(1 - 2*z + 2*pow_2(z)) - 32*C_F2*(1 - 2*z + 2*pow_2(z)))*hpl(1,0,-1,0) + (40*C_A2*(1 - 2*z + 2*pow_2(z)) - 24*C_A*C_F*(1 - 2*z + 2*pow_2(z)) + 48*C_F2*(1 - 2*z + 2*pow_2(z)))*hpl(1,0,0,0) + (48*C_A2*(1 - 2*z + 2*pow_2(z)) - 96*C_A*C_F*(1 - 2*z + 2*pow_2(z)) + 112*C_F2*(1 - 2*z + 2*pow_2(z)))*hpl(1,0,0,1) + (-32*C_A2*(1 - 2*z + 2*pow_2(z)) + 96*C_F2*(1 - 2*z + 2*pow_2(z)))*hpl(1,0,1,0) + (-96*C_A*C_F*(1 - 2*z + 2*pow_2(z)) + 96*C_F2*(1 - 2*z + 2*pow_2(z)))*hpl(1,0,1,1) + (-16*C_A2*(1 - 2*z + 2*pow_2(z)) + 16*C_A*C_F*(1 - 2*z + 2*pow_2(z)) + 64*C_F2*(1 - 2*z + 2*pow_2(z)))*hpl(1,1,0,0) + (-32*C_A2*(1 - 2*z + 2*pow_2(z)) - 48*C_A*C_F*(1 - 2*z + 2*pow_2(z)) + 80*C_F2*(1 - 2*z + 2*pow_2(z)))*hpl(1,1,0,1) + (-64*C_A2*(1 - 2*z + 2*pow_2(z)) + 48*C_A*C_F*(1 - 2*z + 2*pow_2(z)) + 16*C_F2*(1 - 2*z + 2*pow_2(z)))*hpl(1,1,1,0) + (32*C_A2*(1 - 2*z + 2*pow_2(z)) - 64*C_A*C_F*(1 - 2*z + 2*pow_2(z)) + 32*C_F2*(1 - 2*z + 2*pow_2(z)))*hpl(1,1,1,1) + Nf*(((-8*C_A*(20 - 70*z + 229*pow_2(z)))/27. - (2*C_F*(2428 + 2635*z + 840*pow_2(z)))/27.)*hpl(0) + ((4*C_F*(56 - 37*z + 28*pow_2(z)))/27. - (4*C_A*(38 - 28*z + 28*pow_2(z) + PI2*(3 - 6*z + 6*pow_2(z))))/27.)*hpl(1) + (16*C_A*(5 + 7*z + 7*pow_2(z))*hpl(-1,0))/9. + ((8*C_A*(8 + 39*z + 8*pow_2(z)))/9. - (2*C_F*(685 + 442*z + 176*pow_2(z)))/9.)*hpl(0,0) + ((8*C_A*z)/3. - (16*C_F*(5 - 7*z + 7*pow_2(z)))/9.)*hpl(0,1) - (8*C_F*(5 - 16*z + 16*pow_2(z))*hpl(1,0))/9. + ((16*C_A*(5 - 7*z + 7*pow_2(z)))/9. - (16*C_F*(5 - 7*z + 7*pow_2(z)))/9.)*hpl(1,1) + (16*C_A*(1 + 2*z + 2*pow_2(z))*hpl(-1,0,0))/3. + ((8*C_A*(-2 + 8*z))/3. + (8*C_F*(-19 + 20*z + 12*pow_2(z)))/3.)*hpl(0,0,0) + (8*C_F*(1 - 2*z + 2*pow_2(z))*hpl(0,1,1))/3. + ((-8*C_A*(1 - 2*z + 2*pow_2(z)))/3. - (16*C_F*(1 - 2*z + 2*pow_2(z)))/3.)*hpl(1,0,0) + (8*C_A*(1 - 2*z + 2*pow_2(z))*hpl(1,0,1))/3. - (8*C_A*(1 - 2*z + 2*pow_2(z))*hpl(1,1,0))/3. + ((-8*C_A*(1 - 2*z + 2*pow_2(z)))/3. + (8*C_F*(1 - 2*z + 2*pow_2(z)))/3.)*hpl(1,1,1) + C_F*(-32 + 64*z)*hpl(0,0,0,0) + (2*C_A*(-212 + 339*z + 1400*pow_3(z) + 6*pow_2(z)*(-337 + 11*PI2 - 108*zeta_3)))/(81.*z) + (C_F*(2464 - 6*pow_2(z)*(89 + 56*PI2 - 144*zeta_3) + 3*z*(-13025 + 80*PI2 - 144*zeta_3) + 16*pow_3(z)*(2303 + 21*PI2 - 54*zeta_3)))/(162.*z)) + C_F2*(240.75 + (8*PI4*(5 - 6*z + 11*pow_2(z)))/45. - (2*PI2*(17 - 12*z + 61*pow_2(z)))/3. + pow_2(z)*(81 - 304*zeta_3) - 24*zeta_3 + z*(-320.5 + 136*zeta_3)) + hpl(-1)*((32*C_F2*PI2*pow(1 + z,2))/3. + (4*C_A*C_F*(5*PI2*pow(1 + z,2) + 102*(1 + 2*z + 2*pow_2(z))*zeta_3))/3. + (4*C_A2*(PI2*(4 + 45*z + 33*pow_2(z) - 8*pow_3(z)) - 90*z*(1 + 2*z + 2*pow_2(z))*zeta_3))/(9.*z)) + hpl(0)*((-2*C_A2*(448 + 6*(6840 + 31*PI2)*pow_3(z) + pow_2(z)*(15481 - 243*PI2 - 7776*zeta_3) + z*(4348 + 99*PI2 - 2592*zeta_3)))/(27.*z) + C_A*C_F*(99.85185185185185 - (4*PI2*(15 + 135*z + 173*pow_2(z)))/9. + z*(909.9629629629629 - 656*zeta_3) - 88*zeta_3 + pow_2(z)*(419.7037037037037 + 32*zeta_3)) + C_F2*(170 - (2*PI2*(9 - 32*z + 44*pow_2(z)))/3. - 56*zeta_3 - 4*pow_2(z)*(-87 + 56*zeta_3) + z*(57 + 208*zeta_3))) - (C_A*C_F*(60*pow_3(z)*(11991 - 588*PI2 + 8*PI4 - 6720*zeta_3) + 720*(-55 + 48*zeta_3) + z*(-64305 - 6880*PI2 + 876*PI4 + 18000*zeta_3) + pow_2(z)*(-2680*PI2 + 2424*PI4 + 630*(-987 + 512*zeta_3))))/(540.*z) + (C_A2*(3*PI4*z*(269 + 990*z + 112*pow_2(z)) - 20*PI2*(-128 + 471*z - 524*pow_2(z) + 155*pow_3(z)) - 20*(4702 + pow_2(z)*(51304 - 13500*zeta_3) - 1296*zeta_3 - z*(5227 + 648*zeta_3) + 8*pow_3(z)*(-6440 + 1557*zeta_3))))/(270.*z) + hpl(1)*((-2*C_F2*(4*PI2*(-2 - 5*z + 7*pow_2(z)) + z*(423 - 336*zeta_3) + 24*(2 + 7*zeta_3) + 6*pow_2(z)*(-87 + 56*zeta_3)))/3. - (2*C_A2*(-966 + 3*PI2*(-8 - 53*z + 16*pow_2(z) + 34*pow_3(z)) + pow_2(z)*(3875 - 6696*zeta_3) + 4*z*(-1036 + 837*zeta_3) + 8*pow_3(z)*(95 + 837*zeta_3)))/(27.*z) - (2*C_A*C_F*(742 + 13*z*(142 + 9*PI2 - 324*zeta_3) - 2*pow_3(z)*(-2405 + 90*PI2 + 4212*zeta_3) + pow_2(z)*(-6446 + 90*PI2 + 8424*zeta_3)))/(27.*z)))
	       	);
}

template<>
inline double APSF<e_q,e_qmp,2>(const double fz, const double f1, const APSF_Parameter &p, const int Nf){
	const double z(p.z);
	const HPLz &hpl(p.hpl);
	return	(
				fz*((-16*C_A*C_F2*PI2*(1 + z) + 32*C_F3*PI2*(1 + z))*hpl(-1) + (16*C_A2*C_F*(1 + z) + 32*C_A*C_F2*(1 + z) - 128*C_F3*(1 + z))*hpl(-1,0) + ((16*C_A*C_F2*PI2*(1 + pow_2(z)))/(-1 + z) - (32*C_F3*PI2*(1 + pow_2(z)))/(-1 + z))*hpl(0,-1) + ((8*C_F3*(14 - 72*z + 45*pow_2(z)))/(-1 + z) + (16*C_A*C_F2*(-2 + 456*z + (-337 + 24*PI2)*pow_2(z)))/(9.*(-1 + z)) - (4*C_A2*C_F*(425 + 240*z + 113*pow_2(z) + 12*PI2*(-1 + pow_2(z))))/(9.*(-1 + z)))*hpl(0,0) + pow_2(Nf)*((-32*C_F*(-7 + 6*z))/27. - (16*C_F*(11 - 12*z + 11*pow_2(z))*hpl(0))/(27.*(-1 + z)) - (16*C_F*(1 + pow_2(z))*hpl(0,0))/(9.*(-1 + z))) + ((-8*C_A2*C_F*(-14 + PI2)*(1 + z))/3. + C_F3*(8 + 328*z) + (16*C_A*C_F2*(-218 + 240*z - 290*pow_2(z) + PI2*(3 + 9*pow_2(z))))/(9.*(-1 + z)))*hpl(0,1) + (160*C_F3*(-1 + z) + (16*C_A2*C_F*PI2*(1 + pow_2(z)))/(3.*(-1 + z)) + (32*C_A*C_F2*(-127 + 120*z - 127*pow_2(z) + 3*PI2*(1 + pow_2(z))))/(9.*(-1 + z)))*hpl(1,0) + (128*C_A2*C_F*(1 + z) - 448*C_A*C_F2*(1 + z) + 384*C_F3*(1 + z))*hpl(-1,-1,0) + (-32*C_A2*C_F*(1 + z) + 160*C_A*C_F2*(1 + z) - 192*C_F3*(1 + z))*hpl(-1,0,0) + (64*C_A2*C_F*(1 + z) - 128*C_A*C_F2*(1 + z))*hpl(-1,0,1) + (16*C_A2*C_F*(5 + 6/(-1 + z) - 7*z) - 64*C_F3*(1 + 5*z) + (64*C_A*C_F2*(-1 - 8*z + 6*pow_2(z)))/(-1 + z))*hpl(0,-1,0) + (C_F3*(-48 + 80*z) - (16*C_A2*C_F*(5 + 18*z + 8*pow_2(z)))/(3.*(-1 + z)) - (8*C_A*C_F2*(13 - 132*z + 73*pow_2(z)))/(3.*(-1 + z)))*hpl(0,0,0) + (-32*C_F3*(1 + 9*z) + (C_A*C_F2*(64 + 768*z - 704*pow_2(z)))/(3 - 3*z) - (8*C_A2*C_F*(23 - 48*z + 47*pow_2(z)))/(3.*(-1 + z)))*hpl(0,0,1) + ((-16*C_F3*(-7 - 8*z + 3*pow_2(z)))/(-1 + z) - (64*C_A*C_F2*(7 + 4*pow_2(z)))/(3.*(-1 + z)))*hpl(0,1,0) + ((192*C_F3*(1 + pow_2(z)))/(-1 + z) + (8*C_A2*C_F*(19 - 16*z + 19*pow_2(z)))/(-1 + z) - (32*C_A*C_F2*(43 - 24*z + 43*pow_2(z)))/(3.*(-1 + z)))*hpl(1,0,0) + ((-128*C_A2*C_F*(1 + pow_2(z)))/(-1 + z) + (448*C_A*C_F2*(1 + pow_2(z)))/(-1 + z) - (384*C_F3*(1 + pow_2(z)))/(-1 + z))*hpl(0,-1,-1,0) + ((32*C_A2*C_F*(1 + pow_2(z)))/(-1 + z) - (160*C_A*C_F2*(1 + pow_2(z)))/(-1 + z) + (192*C_F3*(1 + pow_2(z)))/(-1 + z))*hpl(0,-1,0,0) + ((-64*C_A2*C_F*(1 + pow_2(z)))/(-1 + z) + (128*C_A*C_F2*(1 + pow_2(z)))/(-1 + z))*hpl(0,-1,0,1) + ((64*C_A*C_F2)/(-1 + z) + (128*C_F3*pow_2(z))/(-1 + z) - (32*C_A2*C_F*(1 + pow_2(z)))/(-1 + z))*hpl(0,0,-1,0) + ((64*C_F3*(1 + pow_2(z)))/(-1 + z) + (8*C_A2*C_F*(3 + 5*pow_2(z)))/(-1 + z) - (32*C_A*C_F2*(3 + 7*pow_2(z)))/(-1 + z))*hpl(0,0,0,0) + ((-8*C_A2*C_F*(7 + pow_2(z)))/(-1 + z) - (64*C_A*C_F2*(-1 + 2*pow_2(z)))/(-1 + z) - (32*C_F3*(1 + 3*pow_2(z)))/(-1 + z))*hpl(0,0,0,1) + ((-32*C_A*C_F2*(1 + pow_2(z)))/(-1 + z) - (64*C_F3*(1 + 3*pow_2(z)))/(-1 + z))*hpl(0,0,1,0) - (128*C_F3*(1 + pow_2(z))*hpl(0,0,1,1))/(-1 + z) + ((32*C_F3*(1 - 5*pow_2(z)))/(-1 + z) - (32*C_A*C_F2*(7 + 3*pow_2(z)))/(-1 + z) + (8*C_A2*C_F*(9 + 7*pow_2(z)))/(-1 + z))*hpl(0,1,0,0) - (128*C_F3*(1 + pow_2(z))*hpl(0,1,0,1))/(-1 + z) - (128*C_F3*(1 + pow_2(z))*hpl(0,1,1,0))/(-1 + z) + ((-192*C_A2*C_F*(1 + pow_2(z)))/(-1 + z) + (512*C_A*C_F2*(1 + pow_2(z)))/(-1 + z) - (256*C_F3*(1 + pow_2(z)))/(-1 + z))*hpl(1,0,-1,0) + ((96*C_A2*C_F*(1 + pow_2(z)))/(-1 + z) - (352*C_A*C_F2*(1 + pow_2(z)))/(-1 + z) + (128*C_F3*(1 + pow_2(z)))/(-1 + z))*hpl(1,0,0,0) + ((-128*C_A2*C_F*(1 + pow_2(z)))/(-1 + z) + (256*C_A*C_F2*(1 + pow_2(z)))/(-1 + z) - (256*C_F3*(1 + pow_2(z)))/(-1 + z))*hpl(1,0,0,1) - (128*C_F3*(1 + pow_2(z))*hpl(1,0,1,0))/(-1 + z) + ((128*C_A2*C_F*(1 + pow_2(z)))/(-1 + z) - (256*C_A*C_F2*(1 + pow_2(z)))/(-1 + z))*hpl(1,1,0,0) + hpl(1)*((-32*C_F3*(5 - 10*z + 12*zeta_3 + pow_2(z)*(5 + 12*zeta_3)))/(-1 + z) + (16*C_A*C_F2*(-7 + 14*z + 48*zeta_3 + pow_2(z)*(-7 + 48*zeta_3)))/(-1 + z) - (16*C_A2*C_F*(-4 + 8*z + 54*zeta_3 + pow_2(z)*(-4 + 54*zeta_3)))/(3.*(-1 + z))) + (4*C_F3*(PI2*(195 + 480*z - 675*pow_2(z)) + PI4*(-46 + 28*pow_2(z)) - 45*(-1 + z)*(91 + z*(-91 + 96*zeta_3))))/(45.*(-1 + z)) + Nf*(((-8*C_F2*(60 - 8*z - 37*pow_2(z) + 2*PI2*(1 + pow_2(z))))/(9.*(-1 + z)) - (8*C_A*C_F*(-440 + 408*z - 302*pow_2(z) + 6*PI2*(1 + pow_2(z))))/(27.*(-1 + z)))*hpl(0) + (32*C_A*C_F*(-1 + z)*hpl(1))/3. + ((C_F2*(352 + 384*z - 448*pow_2(z)))/(9 - 9*z) + (16*C_A*C_F*(10 - z + 5*pow_2(z)))/(3.*(-1 + z)))*hpl(0,0) + ((-16*C_A*C_F*(1 + z))/3. + (128*C_F2*(4 - 3*z + 4*pow_2(z)))/(9.*(-1 + z)))*hpl(0,1) + (128*C_F2*(4 - 3*z + 4*pow_2(z))*hpl(1,0))/(9.*(-1 + z)) + ((C_F2*(16 - 80*pow_2(z)))/(3 - 3*z) + (32*C_A*C_F*(1 + pow_2(z)))/(3.*(-1 + z)))*hpl(0,0,0) + ((16*C_A*C_F*(1 + pow_2(z)))/(3.*(-1 + z)) + (64*C_F2*(1 + pow_2(z)))/(3.*(-1 + z)))*hpl(0,0,1) + (64*C_F2*(1 + pow_2(z))*hpl(0,1,0))/(3.*(-1 + z)) + ((-16*C_A*C_F*(1 + pow_2(z)))/(-1 + z) + (128*C_F2*(1 + pow_2(z)))/(3.*(-1 + z)))*hpl(1,0,0) - (2*C_F2*(493 - 656*z - 48*zeta_3 + pow_2(z)*(163 + 240*zeta_3)))/(9.*(-1 + z)) + (4*C_A*C_F*(1185 - 2788*z + PI2*(2 + 24*z - 26*pow_2(z)) - 180*zeta_3 + pow_2(z)*(1603 + 324*zeta_3)))/(27.*(-1 + z))) + hpl(0)*((4*C_F3*(79 - 52*z - 32*zeta_3 + 4*pow_2(z)*(-6 + PI2 + 4*zeta_3)))/(-1 + z) - (4*C_A*C_F2*(555 - 656*z + PI2*(-4 + 24*z + 62*pow_2(z)) - 648*zeta_3 - 32*pow_2(z)*(11 + 18*zeta_3)))/(9.*(-1 + z)) + (8*C_A2*C_F*(-922 + 1248*z + 3*PI2*(11 - 6*z + 26*pow_2(z)) - 378*zeta_3 - pow_2(z)*(1369 + 486*zeta_3)))/(27.*(-1 + z))) + (4*C_A*C_F2*(PI4*(67 + 2*pow_2(z)) + 30*PI2*(3 - 16*z + 13*pow_2(z)) + 10*(-635 + z*(1270 - 864*zeta_3) + 258*zeta_3 + pow_2(z)*(-635 + 546*zeta_3))))/(45.*(-1 + z)) - (2*C_A2*C_F*(6*PI4*(7 + 19*pow_2(z)) - 20*PI2*(-31 - 39*z + 70*pow_2(z)) + 5*(3125 + 1152*zeta_3 + 11*pow_2(z)*(685 + 216*zeta_3) - 4*z*(2665 + 972*zeta_3))))/(135.*(-1 + z)))
				+PD<-1>(fz,f1,p)*((2*C_F*pow_2(Nf)*(40*PI2 - 9*(17 + 16*zeta_3)))/81. + Nf*(C_F2*(-46 + (20*PI2)/9. + (116*PI4)/135. - (272*zeta_3)/3.) + C_A*C_F*(40 - (1336*PI2)/81. + (2*PI4)/45. + (400*zeta_3)/9.)) + C_F3*(29 + (16*PI4)/5. + PI2*(6 - (32*zeta_3)/3.) + 136*zeta_3 - 480*zeta_5) + C_A2*C_F*(-92.05555555555556 + (4496*PI2)/81. - PI4/9. - (3104*zeta_3)/9. + 80*zeta_5) + (C_A*C_F2*(-988*PI4 + 60*PI2*(-205 + 24*zeta_3) + 45*(453 + 3376*zeta_3 + 1440*zeta_5)))/270.)
				+PD<+0>(fz,f1,p)*((-32*C_F*pow_2(Nf))/27. + (4*C_A2*C_F*(-1340*PI2 + 66*PI4 + 45*(245 + 44*zeta_3)))/135. + Nf*((8*C_A*C_F*(-209 + 20*PI2 - 252*zeta_3))/27. + C_F2*(-73.33333333333333 + 64*zeta_3)))
	       	);
}

template<>
inline double APSF<e_q,e_qbmp,2>(const double fz, const double f1, const APSF_Parameter &p, const int Nf){
	const double z(p.z);
	const HPLz &hpl(p.hpl);
	return	(
				fz*((-32*C_F3*(30 + PI2)*(-1 + z) - (32*C_A2*C_F*(35 + PI2)*(-1 + z))/3. + (16*C_A*C_F2*(230 + 7*PI2)*(-1 + z))/3.)*hpl(1) + ((-256*C_A2*C_F*PI2*(1 + pow_2(z)))/(3.*(1 + z)) + (896*C_A*C_F2*PI2*(1 + pow_2(z)))/(3.*(1 + z)) - (256*C_F3*PI2*(1 + pow_2(z)))/(1 + z))*hpl(-1,-1) + ((64*C_F3*(3*pow(1 + z,2) + 10*PI2*(1 + pow_2(z))))/(3.*(1 + z)) + (16*C_A2*C_F*(173 + 78*z + 173*pow_2(z) + 33*PI2*(1 + pow_2(z))))/(9.*(1 + z)) - (32*C_A*C_F2*(63*PI2*(1 + pow_2(z)) + 2*(91 + 48*z + 91*pow_2(z))))/(9.*(1 + z)))*hpl(-1,0) + ((512*C_F3*PI2*(1 + pow_2(z)))/(3.*(1 + z)) + (8*C_A2*C_F*PI2*(15 + 17*pow_2(z)))/(3.*(1 + z)) - (16*C_A*C_F2*PI2*(31 + 33*pow_2(z)))/(3.*(1 + z)))*hpl(0,-1) + ((16*C_A*C_F2*(116 - 135*z + 17*pow_2(z) + 6*PI2*(6 + 7*pow_2(z))))/(9.*(1 + z)) - (16*C_F3*(-6*(4 + 15*z + 11*pow_2(z)) + PI2*(13 + 15*pow_2(z))))/(3.*(1 + z)) - (4*C_A2*C_F*(8*(38 + 29*pow_2(z)) + PI2*(33 + 39*pow_2(z))))/(9.*(1 + z)))*hpl(0,0) + ((32*C_F3*(19 + 30*z + 11*pow_2(z) + PI2*(1 + pow_2(z))))/(1 + z) + (16*C_A2*C_F*(35*pow(1 + z,2) + 2*PI2*(1 + pow_2(z))))/(3.*(1 + z)) - (16*C_A*C_F2*(127 + 230*z + 103*pow_2(z) + 7*PI2*(1 + pow_2(z))))/(3.*(1 + z)))*hpl(0,1) + (64*C_A*C_F2*(-1 + z) - 128*C_F3*(-1 + z))*hpl(1,0) + ((-16*C_A2*C_F*(13 + 48*z + 13*pow_2(z)))/(3.*(1 + z)) - (32*C_F3*(17 + 40*z + 17*pow_2(z)))/(1 + z) + (16*C_A*C_F2*(77 + 216*z + 77*pow_2(z)))/(3.*(1 + z)))*hpl(-1,0,0) + ((-192*C_F3*(5 + 8*z + 5*pow_2(z)))/(1 + z) - (32*C_A2*C_F*(35 + 48*z + 35*pow_2(z)))/(3.*(1 + z)) + (32*C_A*C_F2*(115 + 168*z + 115*pow_2(z)))/(3.*(1 + z)))*hpl(-1,0,1) + ((-64*C_F3*z*(4 + z))/(1 + z) + (32*C_A*C_F2*(4 + 12*z - 23*pow_2(z)))/(3.*(1 + z)) + (32*C_A2*C_F*(-2 + 13*pow_2(z)))/(3.*(1 + z)))*hpl(0,-1,0) + ((32*C_F3*z*(22 + 13*z))/(1 + z) - (16*C_A2*C_F*(5 - 18*z + 8*pow_2(z)))/(3.*(1 + z)) - (16*C_A*C_F2*(-10 + 102*z + 23*pow_2(z)))/(3.*(1 + z)))*hpl(0,0,0) + ((32*C_F3*(13 + 32*z + 25*pow_2(z)))/(1 + z) + (16*C_A2*C_F*(23 + 48*z + 47*pow_2(z)))/(3.*(1 + z)) - (16*C_A*C_F2*(85 + 192*z + 169*pow_2(z)))/(3.*(1 + z)))*hpl(0,0,1) + (-32*C_A*C_F2*(1 + z) + 64*C_F3*(1 + z))*hpl(0,1,0) + ((256*C_A2*C_F*(1 + pow_2(z)))/(1 + z) - (1152*C_A*C_F2*(1 + pow_2(z)))/(1 + z) + (1280*C_F3*(1 + pow_2(z)))/(1 + z))*hpl(-1,-1,0,0) + ((512*C_A2*C_F*(1 + pow_2(z)))/(1 + z) - (1792*C_A*C_F2*(1 + pow_2(z)))/(1 + z) + (1536*C_F3*(1 + pow_2(z)))/(1 + z))*hpl(-1,-1,0,1) + ((-128*C_A*C_F2*(1 + pow_2(z)))/(1 + z) + (256*C_F3*(1 + pow_2(z)))/(1 + z))*hpl(-1,0,-1,0) + ((-96*C_A2*C_F*(1 + pow_2(z)))/(1 + z) + (544*C_A*C_F2*(1 + pow_2(z)))/(1 + z) - (704*C_F3*(1 + pow_2(z)))/(1 + z))*hpl(-1,0,0,0) + ((-256*C_A2*C_F*(1 + pow_2(z)))/(1 + z) + (1024*C_A*C_F2*(1 + pow_2(z)))/(1 + z) - (1024*C_F3*(1 + pow_2(z)))/(1 + z))*hpl(-1,0,0,1) + ((64*C_A*C_F2*(1 + pow_2(z)))/(1 + z) - (128*C_F3*(1 + pow_2(z)))/(1 + z))*hpl(-1,0,1,0) + (32*C_A2*C_F*(-1 + z) + (256*C_F3*(1 + pow_2(z)))/(1 + z) - (64*C_A*C_F2*(1 + 3*pow_2(z)))/(1 + z))*hpl(0,-1,-1,0) + ((192*C_A*C_F2*(3 + 4*pow_2(z)))/(1 + z) - (16*C_A2*C_F*(7 + 9*pow_2(z)))/(1 + z) - (64*C_F3*(11 + 15*pow_2(z)))/(1 + z))*hpl(0,-1,0,0) + ((-256*C_A2*C_F*(1 + pow_2(z)))/(1 + z) + (960*C_A*C_F2*(1 + pow_2(z)))/(1 + z) - (896*C_F3*(1 + pow_2(z)))/(1 + z))*hpl(0,-1,0,1) + ((-16*C_A2*C_F*(1 + 3*pow_2(z)))/(1 + z) + (64*C_A*C_F2*(1 + 4*pow_2(z)))/(1 + z) - (64*C_F3*(1 + 5*pow_2(z)))/(1 + z))*hpl(0,0,-1,0) + ((96*C_F3*(1 + 3*pow_2(z)))/(1 + z) + (8*C_A2*C_F*(3 + 5*pow_2(z)))/(1 + z) - (32*C_A*C_F2*(3 + 7*pow_2(z)))/(1 + z))*hpl(0,0,0,0) + ((64*C_A2*C_F*(1 + pow_2(z)))/(1 + z) - (320*C_A*C_F2*(1 + pow_2(z)))/(1 + z) + (384*C_F3*(1 + pow_2(z)))/(1 + z))*hpl(0,0,0,1) + ((-32*C_A*C_F2*(1 + pow_2(z)))/(1 + z) + (64*C_F3*(1 + pow_2(z)))/(1 + z))*hpl(0,0,1,0) + hpl(-1)*((32*C_F3*(PI2*(5 + 8*z + 5*pow_2(z)) + 36*(1 + pow_2(z))*zeta_3))/(1 + z) + (16*C_A2*C_F*(PI2*(35 + 48*z + 35*pow_2(z)) + 216*(1 + pow_2(z))*zeta_3))/(9.*(1 + z)) - (16*C_A*C_F2*(PI2*(115 + 168*z + 115*pow_2(z)) + 756*(1 + pow_2(z))*zeta_3))/(9.*(1 + z))) + (4*C_F3*(7*PI4*(1 + pow_2(z)) - 48*PI2*(4 + 7*z + 3*pow_2(z)) - 108*(-5 + 8*zeta_3 + 24*z*zeta_3 + pow_2(z)*(5 + 22*zeta_3))))/(9.*(1 + z)) + (4*C_A*C_F2*(3*PI4*pow_2(z) + 4*PI2*(95 + 297*z + 68*pow_2(z)) + 12*(-251 + 432*zeta_3 + 756*z*zeta_3 + pow_2(z)*(251 + 603*zeta_3))))/(27.*(1 + z)) - (C_A2*C_F*(3*PI4*(7 + 9*pow_2(z)) + 8*PI2*(23 + 171*z + 14*pow_2(z)) + 12*(-367 + 648*zeta_3 + 864*z*zeta_3 + pow_2(z)*(367 + 612*zeta_3))))/(27.*(1 + z)) + hpl(0)*((-4*C_A2*C_F*(-41 + 386*z + PI2*(47 + 132*z + 89*pow_2(z)) + 252*zeta_3 + pow_2(z)*(535 + 324*zeta_3)))/(9.*(1 + z)) + (8*C_A*C_F2*(-185 + 764*z + PI2*(83 + 252*z + 182*pow_2(z)) + 432*zeta_3 + pow_2(z)*(1111 + 612*zeta_3)))/(9.*(1 + z)) - (16*C_F3*(PI2*(12 + 40*z + 31*pow_2(z)) + 6*(-8 + 21*z + 10*zeta_3 + 16*pow_2(z)*(2 + zeta_3))))/(3.*(1 + z))) + Nf*(((-32*C_A*C_F*PI2*(1 + pow_2(z)))/(9.*(1 + z)) + (64*C_F2*PI2*(1 + pow_2(z)))/(9.*(1 + z)))*hpl(-1) + ((8*C_A*C_F*(41*pow(1 + z,2) + PI2*(1 + pow_2(z))))/(9.*(1 + z)) - (16*C_F2*(41*pow(1 + z,2) + PI2*(1 + pow_2(z))))/(9.*(1 + z)))*hpl(0) + ((64*C_A*C_F*(-1 + z))/3. - (128*C_F2*(-1 + z))/3.)*hpl(1) + ((-128*C_A*C_F*(4 + 3*z + 4*pow_2(z)))/(9.*(1 + z)) + (256*C_F2*(4 + 3*z + 4*pow_2(z)))/(9.*(1 + z)))*hpl(-1,0) + ((16*C_A*C_F*(19 + 18*z + 19*pow_2(z)))/(9.*(1 + z)) - (32*C_F2*(19 + 18*z + 19*pow_2(z)))/(9.*(1 + z)))*hpl(0,0) + ((-32*C_A*C_F*(1 + z))/3. + (64*C_F2*(1 + z))/3.)*hpl(0,1) + ((-32*C_A*C_F*(1 + pow_2(z)))/(3.*(1 + z)) + (64*C_F2*(1 + pow_2(z)))/(3.*(1 + z)))*hpl(-1,0,0) + ((64*C_A*C_F*(1 + pow_2(z)))/(3.*(1 + z)) - (128*C_F2*(1 + pow_2(z)))/(3.*(1 + z)))*hpl(-1,0,1) + ((-32*C_A*C_F*(1 + pow_2(z)))/(3.*(1 + z)) + (64*C_F2*(1 + pow_2(z)))/(3.*(1 + z)))*hpl(0,-1,0) + ((32*C_A*C_F*(1 + pow_2(z)))/(3.*(1 + z)) - (64*C_F2*(1 + pow_2(z)))/(3.*(1 + z)))*hpl(0,0,0) + ((-32*C_A*C_F*(1 + pow_2(z)))/(3.*(1 + z)) + (64*C_F2*(1 + pow_2(z)))/(3.*(1 + z)))*hpl(0,0,1) - (8*C_A*C_F*(10*PI2*(1 + pow_2(z)) - 3*(61 + 18*zeta_3 + pow_2(z)*(-61 + 18*zeta_3))))/(27.*(1 + z)) + (16*C_F2*(10*PI2*(1 + pow_2(z)) - 3*(61 + 18*zeta_3 + pow_2(z)*(-61 + 18*zeta_3))))/(27.*(1 + z))))
	       	);
}

template<>
inline double APSF<e_q,e_qp,2>(const double fz, const double f1, const APSF_Parameter &p, const int Nf){
	const double z(p.z);
	const HPLz &hpl(p.hpl);
	return	(
				fz*((4*C_A*C_F*PI2*(1 + z)*(4 + 11*z + 4*pow_2(z))*hpl(-1))/(9.*z) + ((-4*C_F2*(-1 + z)*(-523 - 2017*z + 242*pow_2(z)))/(27.*z) - (4*C_A*C_F*(-1 + z)*(571 + 1882*z - 302*pow_2(z) + 3*PI2*(4 + 7*z + 4*pow_2(z))))/(27.*z))*hpl(1) + (32*C_A*C_F*(1 + z)*(26 - 2*z + 53*pow_2(z))*hpl(-1,0))/(9.*z) - (8*C_A*C_F*PI2*(-1 + z)*hpl(0,-1))/3. + ((4*C_A*C_F*(269 - 3*PI2*(-1 + z) - 397*z + 498*pow_2(z)))/9. + (4*C_F2*(-128 + 53*z - 88*pow_2(z) + 12*PI2*(1 + z)))/3.)*hpl(0,0) + ((-4*C_F2*(612 + 45*z + 112*pow_2(z)))/9. + (4*C_A*C_F*(80 + (479 + 6*PI2)*z + 2*(91 + 3*PI2)*pow_2(z) - 112*pow_3(z)))/(9.*z))*hpl(0,1) + ((-8*C_A*C_F*(-1 + z)*(9 + 22*z + 27*pow_2(z)))/(3.*z) - (16*C_F2*(-1 + z)*(23 - z + 50*pow_2(z)))/(9.*z))*hpl(1,0) + ((4*C_A*C_F*(-1 + z)*(4 + 31*z + 4*pow_2(z)))/(9.*z) - (4*C_F2*(-1 + z)*(16 + 23*z + 16*pow_2(z)))/(3.*z))*hpl(1,1) + (16*C_A*C_F*(3 - 4/z + 3*z - 4*pow_2(z))*hpl(-1,-1,0))/3. - (8*C_A*C_F*(1 + z)*(16 - 19*z + 16*pow_2(z))*hpl(-1,0,0))/(3.*z) - (32*C_A*C_F*(2 + 3*z + 3*pow_2(z) + 2*pow_3(z))*hpl(-1,0,1))/(3.*z) - 8*C_A*C_F*(3 - 13*z + 8*pow_2(z))*hpl(0,-1,0) + ((8*C_A*C_F*(-29 + 31*z))/3. + (8*C_F2*(9 + 15*z + 44*pow_2(z)))/3.)*hpl(0,0,0) + ((4*C_A*C_F*(59 + 71*z + 8*pow_2(z)))/3. + (16*C_F2*(-6 + 9*z + 16*pow_2(z)))/3.)*hpl(0,0,1) + (16*C_F2*(2 + 7*z + 4*pow_2(z)) + (8*C_A*C_F*(6 + 15*z + 4*pow_2(z)))/3.)*hpl(0,1,0) + ((16*C_F2*pow(3 + 2*z,2))/3. - (8*C_A*C_F*(-4 + 11*z + 17*pow_2(z) + 4*pow_3(z)))/(3.*z))*hpl(0,1,1) + ((-8*C_F2*(-1 + z)*(4 + 7*z + 4*pow_2(z)))/(3.*z) + (4*C_A*C_F*(-1 + z)*(8 + 17*z + 8*pow_2(z)))/z)*hpl(1,0,0) + ((16*C_A*C_F*(3 + 4/z - 3*z - 4*pow_2(z)))/3. + (16*C_F2*(-4 - 3*z + 3*pow_2(z) + 4*pow_3(z)))/(3.*z))*hpl(1,1,0) + ((16*C_A*C_F*(3 + 4/z - 3*z - 4*pow_2(z)))/3. + (16*C_F2*(-4 - 3*z + 3*pow_2(z) + 4*pow_3(z)))/(3.*z))*hpl(1,1,1) - 32*C_A*C_F*(-1 + z)*hpl(0,-1,-1,0) - 48*C_A*C_F*(-1 + z)*hpl(0,-1,0,0) - 16*C_A*C_F*(-5 + z)*hpl(0,0,-1,0) + (C_A*C_F*(64 - 48*z) - 64*C_F2*(1 + z))*hpl(0,0,0,0) + (-8*C_A*C_F*(1 + z) - 96*C_F2*(1 + z))*hpl(0,0,0,1) - 64*C_F2*(1 + z)*hpl(0,0,1,0) + (16*C_A*C_F*(1 + z) - 16*C_F2*(1 + z))*hpl(0,0,1,1) + (-56*C_A*C_F*(1 + z) + 16*C_F2*(1 + z))*hpl(0,1,0,0) + (32*C_A*C_F*(1 + z) - 32*C_F2*(1 + z))*hpl(0,1,1,0) + (32*C_A*C_F*(1 + z) - 32*C_F2*(1 + z))*hpl(0,1,1,1) + (2*C_F2*((6325 + 3060*PI2 - 111*PI4)*z + 90*(55 - 48*zeta_3) + 20*pow_3(z)*(-981 + 28*PI2 + 288*zeta_3) + pow_2(z)*(8345 + 225*PI2 - 111*PI4 + 10800*zeta_3)))/(135.*z) + hpl(0)*((-2*C_A*C_F*(448 + 4*(2299 + 6*PI2)*pow_3(z) + z*(7130 + 177*PI2 - 2808*zeta_3) + pow_2(z)*(6608 - 21*PI2 - 1944*zeta_3)))/(27.*z) - (2*C_F2*(-2265 - 8229*z + 52*pow_2(z) + 12*PI2*(-6 + 9*z + 16*pow_2(z)) + 1512*zeta_3 + 1512*z*zeta_3))/27.) + Nf*((16*C_F*(37 - 35*z - 57*pow_2(z) + 3*PI2*(1 + z))*hpl(0))/27. - (8*C_F*(-20 + 63*z - 117*pow_2(z) + 74*pow_3(z))*hpl(1))/(27.*z) + (232*C_F*(1 + z)*hpl(0,0))/9. + (16*C_F*(-5 + 4*z + 6*pow_2(z))*hpl(0,1))/9. - (8*C_F*(-4 - 3*z + 3*pow_2(z) + 4*pow_3(z))*hpl(1,1))/(9.*z) + (16*C_F*(1 + z)*hpl(0,0,0))/3. - (32*C_F*(1 + z)*hpl(0,0,1))/3. + (16*C_F*(1 + z)*hpl(0,1,1))/3. - (8*C_F*(-8 + 2*(-50 + 3*PI2)*pow_3(z) + pow_2(z)*(185 + 4*PI2 - 18*zeta_3) - z*(77 + 5*PI2 + 18*zeta_3)))/(27.*z)) + (C_A*C_F*(9*PI4*z*(107 + 117*z) + 30*PI2*(128 - 479*z + 226*pow_2(z) + 112*pow_3(z)) - 20*(6761 - 1944*zeta_3 + 27*z*(-121 + 4*zeta_3) - 54*pow_2(z)*(-316 + 31*zeta_3) + pow_3(z)*(-20558 + 2808*zeta_3))))/(405.*z) + dABCdABCoNc*((-2*PI2*(1 + z)*(4 - z + 4*pow_2(z))*hpl(-1))/(3.*z) + (2*(-1 + z)*(-273*z + PI2*(4 + 7*z + 4*pow_2(z)))*hpl(1))/(9.*z) - (52*(1 + z)*hpl(-1,0))/3. - (4*PI2*(-1 + z)*hpl(0,-1))/3. + ((76 + 58*z - 2*PI2*(5 + 3*z))*hpl(0,0))/3. - (2*(-41 + 2*PI2)*(1 + z)*hpl(0,1))/3. + (8*(3 - 4/z + 3*z - 4*pow_2(z))*hpl(-1,-1,0))/3. + (4*(1 + z)*(8 - 5*z + 8*pow_2(z))*hpl(-1,0,0))/(3.*z) + (16*(2 + 3*z + 3*pow_2(z) + 2*pow_3(z))*hpl(-1,0,1))/(3.*z) + (4*(3 - 9*z + 8*pow_2(z))*hpl(0,-1,0))/3. - (4*(3 + 8*pow_2(z))*hpl(0,0,0))/3. + (2 - 18*z - (32*pow_2(z))/3.)*hpl(0,0,1) + 6*(-1 + z)*hpl(1,0,0) - 16*(-1 + z)*hpl(0,-1,-1,0) + 8*(-1 + z)*hpl(0,-1,0,0) + 8*(-1 + z)*hpl(0,0,-1,0) + 8*hpl(0,0,0,0) + 12*(1 + z)*hpl(0,0,0,1) - 4*(1 + z)*hpl(0,1,0,0) + (hpl(0)*(PI2*(3 + 27*z + 32*pow_2(z)) + 6*(100 + 9*z - 24*zeta_3)))/9. + (PI4*(3 + 5*z) - 2*PI2*(67 + 41*z) + 96*(25 - 25*z + 3*zeta_3 + 5*pow_2(z)*zeta_3))/18.))
	       	);
}

template<>
inline double APSF<e_q,e_qbp,2>(const double fz, const double f1, const APSF_Parameter &p, const int Nf){
	const double z(p.z);
	const HPLz &hpl(p.hpl);
	return	(
				fz*((4*C_A*C_F*PI2*(1 + z)*(4 + 11*z + 4*pow_2(z))*hpl(-1))/(9.*z) + ((-4*C_F2*(-1 + z)*(-523 - 2017*z + 242*pow_2(z)))/(27.*z) - (4*C_A*C_F*(-1 + z)*(571 + 1882*z - 302*pow_2(z) + 3*PI2*(4 + 7*z + 4*pow_2(z))))/(27.*z))*hpl(1) + (32*C_A*C_F*(1 + z)*(26 - 2*z + 53*pow_2(z))*hpl(-1,0))/(9.*z) - (8*C_A*C_F*PI2*(-1 + z)*hpl(0,-1))/3. + ((4*C_A*C_F*(269 - 3*PI2*(-1 + z) - 397*z + 498*pow_2(z)))/9. + (4*C_F2*(-128 + 53*z - 88*pow_2(z) + 12*PI2*(1 + z)))/3.)*hpl(0,0) + ((-4*C_F2*(612 + 45*z + 112*pow_2(z)))/9. + (4*C_A*C_F*(80 + (479 + 6*PI2)*z + 2*(91 + 3*PI2)*pow_2(z) - 112*pow_3(z)))/(9.*z))*hpl(0,1) + ((-8*C_A*C_F*(-1 + z)*(9 + 22*z + 27*pow_2(z)))/(3.*z) - (16*C_F2*(-1 + z)*(23 - z + 50*pow_2(z)))/(9.*z))*hpl(1,0) + ((4*C_A*C_F*(-1 + z)*(4 + 31*z + 4*pow_2(z)))/(9.*z) - (4*C_F2*(-1 + z)*(16 + 23*z + 16*pow_2(z)))/(3.*z))*hpl(1,1) + (16*C_A*C_F*(3 - 4/z + 3*z - 4*pow_2(z))*hpl(-1,-1,0))/3. - (8*C_A*C_F*(1 + z)*(16 - 19*z + 16*pow_2(z))*hpl(-1,0,0))/(3.*z) - (32*C_A*C_F*(2 + 3*z + 3*pow_2(z) + 2*pow_3(z))*hpl(-1,0,1))/(3.*z) - 8*C_A*C_F*(3 - 13*z + 8*pow_2(z))*hpl(0,-1,0) + ((8*C_A*C_F*(-29 + 31*z))/3. + (8*C_F2*(9 + 15*z + 44*pow_2(z)))/3.)*hpl(0,0,0) + ((4*C_A*C_F*(59 + 71*z + 8*pow_2(z)))/3. + (16*C_F2*(-6 + 9*z + 16*pow_2(z)))/3.)*hpl(0,0,1) + (16*C_F2*(2 + 7*z + 4*pow_2(z)) + (8*C_A*C_F*(6 + 15*z + 4*pow_2(z)))/3.)*hpl(0,1,0) + ((16*C_F2*pow(3 + 2*z,2))/3. - (8*C_A*C_F*(-4 + 11*z + 17*pow_2(z) + 4*pow_3(z)))/(3.*z))*hpl(0,1,1) + ((-8*C_F2*(-1 + z)*(4 + 7*z + 4*pow_2(z)))/(3.*z) + (4*C_A*C_F*(-1 + z)*(8 + 17*z + 8*pow_2(z)))/z)*hpl(1,0,0) + ((16*C_A*C_F*(3 + 4/z - 3*z - 4*pow_2(z)))/3. + (16*C_F2*(-4 - 3*z + 3*pow_2(z) + 4*pow_3(z)))/(3.*z))*hpl(1,1,0) + ((16*C_A*C_F*(3 + 4/z - 3*z - 4*pow_2(z)))/3. + (16*C_F2*(-4 - 3*z + 3*pow_2(z) + 4*pow_3(z)))/(3.*z))*hpl(1,1,1) - 32*C_A*C_F*(-1 + z)*hpl(0,-1,-1,0) - 48*C_A*C_F*(-1 + z)*hpl(0,-1,0,0) - 16*C_A*C_F*(-5 + z)*hpl(0,0,-1,0) + (C_A*C_F*(64 - 48*z) - 64*C_F2*(1 + z))*hpl(0,0,0,0) + (-8*C_A*C_F*(1 + z) - 96*C_F2*(1 + z))*hpl(0,0,0,1) - 64*C_F2*(1 + z)*hpl(0,0,1,0) + (16*C_A*C_F*(1 + z) - 16*C_F2*(1 + z))*hpl(0,0,1,1) + (-56*C_A*C_F*(1 + z) + 16*C_F2*(1 + z))*hpl(0,1,0,0) + (32*C_A*C_F*(1 + z) - 32*C_F2*(1 + z))*hpl(0,1,1,0) + (32*C_A*C_F*(1 + z) - 32*C_F2*(1 + z))*hpl(0,1,1,1) + (2*C_F2*((6325 + 3060*PI2 - 111*PI4)*z + 90*(55 - 48*zeta_3) + 20*pow_3(z)*(-981 + 28*PI2 + 288*zeta_3) + pow_2(z)*(8345 + 225*PI2 - 111*PI4 + 10800*zeta_3)))/(135.*z) + hpl(0)*((-2*C_A*C_F*(448 + 4*(2299 + 6*PI2)*pow_3(z) + z*(7130 + 177*PI2 - 2808*zeta_3) + pow_2(z)*(6608 - 21*PI2 - 1944*zeta_3)))/(27.*z) - (2*C_F2*(-2265 - 8229*z + 52*pow_2(z) + 12*PI2*(-6 + 9*z + 16*pow_2(z)) + 1512*zeta_3 + 1512*z*zeta_3))/27.) + Nf*((16*C_F*(37 - 35*z - 57*pow_2(z) + 3*PI2*(1 + z))*hpl(0))/27. - (8*C_F*(-20 + 63*z - 117*pow_2(z) + 74*pow_3(z))*hpl(1))/(27.*z) + (232*C_F*(1 + z)*hpl(0,0))/9. + (16*C_F*(-5 + 4*z + 6*pow_2(z))*hpl(0,1))/9. - (8*C_F*(-4 - 3*z + 3*pow_2(z) + 4*pow_3(z))*hpl(1,1))/(9.*z) + (16*C_F*(1 + z)*hpl(0,0,0))/3. - (32*C_F*(1 + z)*hpl(0,0,1))/3. + (16*C_F*(1 + z)*hpl(0,1,1))/3. - (8*C_F*(-8 + 2*(-50 + 3*PI2)*pow_3(z) + pow_2(z)*(185 + 4*PI2 - 18*zeta_3) - z*(77 + 5*PI2 + 18*zeta_3)))/(27.*z)) + (C_A*C_F*(9*PI4*z*(107 + 117*z) + 30*PI2*(128 - 479*z + 226*pow_2(z) + 112*pow_3(z)) - 20*(6761 - 1944*zeta_3 + 27*z*(-121 + 4*zeta_3) - 54*pow_2(z)*(-316 + 31*zeta_3) + pow_3(z)*(-20558 + 2808*zeta_3))))/(405.*z) + dABCdABCoNc*((2*PI2*(1 + z)*(4 - z + 4*pow_2(z))*hpl(-1))/(3.*z) - (2*(-1 + z)*(-273*z + PI2*(4 + 7*z + 4*pow_2(z)))*hpl(1))/(9.*z) + (52*(1 + z)*hpl(-1,0))/3. + (4*PI2*(-1 + z)*hpl(0,-1))/3. + (2*(-38 - 29*z + PI2*(5 + 3*z))*hpl(0,0))/3. + (2*(-41 + 2*PI2)*(1 + z)*hpl(0,1))/3. + (8*(-3 + 4/z - 3*z + 4*pow_2(z))*hpl(-1,-1,0))/3. - (4*(1 + z)*(8 - 5*z + 8*pow_2(z))*hpl(-1,0,0))/(3.*z) - (16*(2 + 3*z + 3*pow_2(z) + 2*pow_3(z))*hpl(-1,0,1))/(3.*z) - (4*(3 - 9*z + 8*pow_2(z))*hpl(0,-1,0))/3. + (4 + (32*pow_2(z))/3.)*hpl(0,0,0) + (-2 + 18*z + (32*pow_2(z))/3.)*hpl(0,0,1) + (6 - 6*z)*hpl(1,0,0) + 16*(-1 + z)*hpl(0,-1,-1,0) + (8 - 8*z)*hpl(0,-1,0,0) + (8 - 8*z)*hpl(0,0,-1,0) - 8*hpl(0,0,0,0) - 12*(1 + z)*hpl(0,0,0,1) + 4*(1 + z)*hpl(0,1,0,0) + (hpl(0)*(-(PI2*(3 + 27*z + 32*pow_2(z))) - 6*(100 + 9*z - 24*zeta_3)))/9. + (-(PI4*(3 + 5*z)) + 2*PI2*(67 + 41*z) - 96*(25 - 25*z + 3*zeta_3 + 5*pow_2(z)*zeta_3))/18.))
	       	);
}

template<>
inline double APSF<e_q,e_q,2>(const double fz, const double f1, const APSF_Parameter &p, const int Nf){
return APSF<e_q,e_qmp,2>(fz,f1,p,Nf) + APSF<e_q,e_qp,2>(fz,f1,p,Nf);

}

template<>
inline double APSF<e_q,e_qb,2>(const double fz, const double f1, const APSF_Parameter &p, const int Nf){
return APSF<e_q,e_qbmp,2>(fz,f1,p,Nf) + APSF<e_q,e_qbp,2>(fz,f1,p,Nf);

}


} // namespace NS_APSF
} // namespace CuTe

#endif //APSFUNCTIONS_H_
