#include "CuTe.h"

#ifdef __cplusplus
extern "C" {
#endif
	#include "gsl/gsl_errno.h"
#ifdef __cplusplus
}
#endif

using CuTe::CuTe_INIT_EXIT;
using CuTe::Crosssection;
using std::vector;

CuTe_INIT_EXIT::CuTe_INIT_EXIT(){
	add	= "";
	std::cout << "******************************** "<<PACKAGE_STRING<<add<<" ***********************************" << std::endl;
	gsl_set_error_handler_off();
}
CuTe_INIT_EXIT::CuTe_INIT_EXIT(const string& add){
	this->add	= add;
	std::cout << "******************************** "<<PACKAGE_STRING<<add<<" ***********************************" << std::endl;
	gsl_set_error_handler_off();
}
CuTe_INIT_EXIT::~CuTe_INIT_EXIT(){
	std::cout << "******************************** "<<PACKAGE_STRING<<add<<" ***********************************" << std::endl;
}

Crosssection::~Crosssection(){
	if(RPS!=NULL)	delete RPS;
	if(FPS!=NULL)	delete FPS;
}

vector<double>& Crosssection::_unset_results(){
	results[ER_erg_result]		= 0.;
	results[ER_erg_RES]			= 0.;
	results[ER_erg_FO]			= 0.;
	results[ER_erg_SQT]			= 0.;
	results[ER_erg_MC]			= 0.;
	results[ER_erg_RC]			= 0.;
	results[ER_erg_Delta]		= 0.;
	results[ER_erg_PDF_Delta]	= 0.;
	results[ER_erg_PDF_DeltaM]	= 0.;
	results[ER_erg_PDF_DeltaP]	= 0.;
	results[ER_erg_PDF_Scale]	= 0.;
	results[ER_erg_mu_M]		= 0.;
	results[ER_erg_mu_P]		= 0.;
	results[ER_erg_exp]			= 0.;
	results[ER_erg_DCP]			= 0.;
	results[ER_erg_DCM]			= 0.;

	return results;
}

inline void Crosssection::_init_params(const SOPS_CS& OP){
//	NS_Utils::DOUT dout("Crosssection::_init_params","OP");
	this-> res	= OP.Scheme.resummation();
//	dout()	<<"Delete"<<endl;
//	dout()	<<"RPS = "<<RPS<<endl;
//	dout()	<<"FPS = "<<FPS<<endl;
	delete	RPS;	RPS=NULL;
	delete	FPS;	FPS=NULL;
//	dout()	<<"RPS = "<<RPS<<endl;
//	dout()	<<"FPS = "<<FPS<<endl;

	int order_a		= OP.Scheme.order();
	int order		= order_a;
	bool improved	= false;

	exp_errors		= OP.Scheme.exponent_errors();

	partial_results	= false;
	N_Centre		= 1;

//	dout()	<<"Switch(res = "<<res<<")"<<endl;
	switch (res) {
	case RES_LL:
		improved	= (OP.Scheme.Power_Counting()==PC_IMP);
		RPS	= new Res_Kinematic_Parameters;
		without_combined_result	= false;
		break;
	case RES_LLO:
		RPS	= new Res_Kinematic_Parameters;
		exp_errors	= false;
		without_combined_result	= false;
		break;
	case RES_LO:
		FPS	= new FO_Kinematic_Paramete_Set;
		exp_errors	= false;
		without_combined_result	= false;
		break;
	case RES_MATCHED:
		improved	= (OP.Scheme.Power_Counting()==PC_IMP);
		RPS	= new Res_Kinematic_Parameters;
		FPS	= new FO_Kinematic_Paramete_Set;
		partial_results	= OP.Scheme.partial_results();
		if(partial_results){
			N_Centre	= 6-without_combined_result;
		}else{
			without_combined_result	= false;
		}
		break;
	case RES_MATCHING_CORR:
		RPS	= new Res_Kinematic_Parameters;
		FPS	= new FO_Kinematic_Paramete_Set;
		exp_errors	= false;
		partial_results	= OP.Scheme.partial_results();
		if(partial_results){
			N_Centre	= 3-without_combined_result;
		}else{
			without_combined_result	= false;
		}
		break;
	case RES_RES_CORR:
		improved	= (OP.Scheme.Power_Counting()==PC_IMP);
		RPS	= new Res_Kinematic_Parameters;
		FPS	= new FO_Kinematic_Paramete_Set;
		partial_results	= OP.Scheme.partial_results();
		if(partial_results){
			N_Centre	= 3-without_combined_result;
		}else{
			without_combined_result	= false;
		}
		break;
	default:
		cerr	<<"Crosssection::_init_params WRONG res ="<<res<<endl;
		exit(1);
		break;
	}

//	dout()	<<"Set order"<<endl;
	if(RPS!=NULL){
//		NS_Utils::DOUT dout("Crosssection::_init_params","RPS!=NULL");
		if(improved){
			if(OP.Scheme.order_epsilon.is_set){
				order	= OP.Scheme.order_epsilon();
			}else{
				order	= 2*order_a + 1;
			}
		}

		RPS->init(&Kin,OP.Process.prod(),order,improved,order_a);
	}

	if(FPS!=NULL){
//		NS_Utils::DOUT dout("Crosssection::_init_params","FPS!=NULL");
//		dout()<<"ord = "<<order_a<<endl;
		FPS->init(&Kin,order_a);
	}

//	dout()	<<"exp_errors = "<<exp_errors<<endl;

}

vector<double>& Crosssection::_update(const int dim, NS_CUBA_integration::cuba_integrand_t integrand){
//		NS_Utils::DOUT dout("Crosssection::_update","dim,integrand");

//	NS_Utils::Timer timer;

	cuba.set(dim,N_Comp);
	cuba.cube(integrand,this);
//	cerr	<<"******************************************"<<endl
//			<<"**** Crosssection::_update::cuba.test ****"<<endl
//			<<"******************************************"<<endl;
//	cuba.test(10,integrand,this);//FIXME debug
	_set_results();

//	using NS_OPTIONS::operator <<;
//	cout	<<"("<<timer.stop()<<")"<<"\t"<<results<<endl;

	return results;
}

void Crosssection::_set_qT_mu(const double mu){
//	NS_Utils::DOUT dout("Crosssection::_set_qT_mu","mu");

	const double& qT(Kin.qT);

	if(RPS!=NULL)	RPS->set_qT(&Kin);
	if(FPS!=NULL)	FPS->set_qT(&Kin);

	results.clear();
	results.resize(ER_Last);

	results[ER_qT]	= qT;
	results[ER_mu]	= mu;

	const double res_reduction = transition_function.f(qT);
//	if(CO_reduction!=NULL){
//		CO_reduction->update(res_reduction);
//	}

	QDCS_central.set_qT_mu(qT,RPS,FPS,mu,mu,res_reduction);//,CO_reduction);

	for(int i=0; i<N_mu; ++i){
//		dout()	<<"n_mu="<<i<<"/"<<N_mu	<<"\t"<<"mu = "<<mu_star.mus[i]<<endl;
		QDCS_mu[i].set_qT_mu(qT,RPS,FPS,mu_star.mus[i],mu,res_reduction);//,CO_reduction);
	}

	for(int i=0; i<N_pdf; ++i){
		//			dout()	<<"n_pdf="<<i<<endl;
		QDCS_pdf[i].set_qT_mu(qT,RPS,FPS,mu,mu,res_reduction);//,CO_reduction);
	}

	if(exp_errors)	QDCS_exponent.set_qT_mu(qT,RPS,FPS,mu,mu,res_reduction);//,CO_reduction);
}

void Crosssection::_set_qT_y_mu(const double mu){
//	NS_Utils::DOUT dout("Crosssection::_set_qT_y_mu","mu");
	_set_qT_mu(mu);
	results[ER_y]	= Kin.y;
}

void Crosssection::_set_results(){
//	NS_Utils::DOUT dout("Crosssection::_set_results");
	vector<double>::const_iterator it_results(cuba.integral.begin());

	double &centre(results[ER_erg_result]);

	centre	= (*it_results);
	results[ER_erg_Delta]=cuba.error[0];

	if(partial_results){
		switch (res) {
			case RES_MATCHED:
				results[ER_erg_RES]	= (*(++it_results));
				results[ER_erg_FO]	= (*(++it_results));
				results[ER_erg_SQT]	= (*(++it_results));
				results[ER_erg_MC]	= (*(++it_results));
				results[ER_erg_RC]	= (*(++it_results));
				break;
			case RES_MATCHING_CORR:
				results[ER_erg_FO]	= (*(++it_results));
				results[ER_erg_SQT]	= (*(++it_results));
				break;
			case RES_RES_CORR:
				results[ER_erg_RES]	= (*(++it_results));
				results[ER_erg_SQT]	= (*(++it_results));
				break;
			default:
				cerr	<<"*************************************************"<<endl;
				cerr	<<"*****  Wrong res = "<<res<<" with N_Centre = "<<N_Centre<<" in Crosssection::_set_results() -> exit(1)"<<endl;
				cerr	<<"*************************************************"<<endl;
				exit(1);
				break;
		}
	}

	if(N_pdf){
		_results_pdf.assign(it_results,it_results+N_pdf+1);
		_results_pdf[0] = centre;

//		using NS_Utils::operator<<;
//		dout()	<<"_results_pdf:"<<_results_pdf<<endl;

		pdfset.calc_errors(results,_results_pdf);
		it_results += N_pdf;
	}

	if(N_mu){
		vector<double>::iterator it_mu(_results_mu_c_0_1.begin());

		(*it_mu)	= centre;
		for(int i=0; i<N_mu; ++i) (*(++it_mu))	= (*(++it_results));

//		using NS_Utils::operator<<;
//		dout()	<<"_results_mu_c_0_1:"<<_results_mu_c_0_1<<endl;

		mu_star.calc_error(_results_mu_c_0_1,results);
	}

	if(exp_errors){
		results[ER_erg_exp]	= (*(++it_results))-centre;
	}


	results[ER_erg_DCP]=results[ER_erg_DCM]=pow_2(results[ER_erg_Delta]);

	if(N_pdf){
		results[ER_erg_DCP]	+= 	pow_2(results[ER_erg_PDF_DeltaP]);
		results[ER_erg_DCM]	+=	pow_2(results[ER_erg_PDF_DeltaM]);
	}

	if(N_mu){
		results[ER_erg_DCP]	+= 	pow_2(results[ER_erg_mu_P]);
		results[ER_erg_DCM]	+=	pow_2(results[ER_erg_mu_M]);
	}

	if(exp_errors){
		const double &err(results[ER_erg_exp]);
		const double err2(pow_2(err));

		if(0<err){
			results[ER_erg_DCP]	+= 	err2;
		}else{
			results[ER_erg_DCM]	+=	err2;
		}

	}

	results[ER_erg_DCP]	=	+sqrt(results[ER_erg_DCP]);
	results[ER_erg_DCM]	=	-sqrt(results[ER_erg_DCM]);
}

void Crosssection::init(const SOPS_CS& OP){
//	NS_Utils::DOUT dout("Crosssection::init","op");
//	dout()	<<endl
//			<<OP.Process.prod		<<endl
//			<<OP.Scheme.resummation	<<endl
//			<<OP.Scheme.order	<<endl
//			<<OP.Scheme.Power_Counting<<endl;
//	if(OP.Scheme.order_epsilon.is_set && OP.Scheme.Power_Counting()==PC_IMP){
//		cout	<<OP.Scheme.order_epsilon<<endl;
//	}
//	cout	<<OP.Scheme.MScheme	<<endl;
//	dout()	<<endl;

	{
//		NS_Utils::DOUT dout("init","Kinematics");
		Kin.init(OP.Process.sqrts(),OP.Process.M(),OP.Cuts);
		cuba.init(OP.CUBA,3,1);

		_init_params(OP);
	}

	{
//		NS_Utils::DOUT dout("init","pdfset");

		pdfset.init(OP);
		N_pdf	= pdfset.N_error();
		_results_pdf.resize(N_pdf+1);
	}

	{
//		NS_Utils::DOUT dout("init","mu_star");
//		mu_star.init(OP,pdfset.as_central());
//		NS_CS_Params::Q_star QS(pdfset.as_central(),OP);
//		q_star	= QS();

		change_mustar(OP);

		N_mu	= mu_star.N_errors();
		_results_mu_c_0_1.resize(N_mu+1);

	}

	{
//		NS_Utils::DOUT dout("init","_QDCSs");

		N_QDCS	= 1 + N_pdf + N_mu + exp_errors;
		_QDCSs.resize(N_QDCS);
	}


	vector<Quad_Differential_Crosssection*>::iterator	IT_CS(_QDCSs.begin());

	{
//		NS_Utils::DOUT dout("init","QDCS_central");

		QDCS_central.init(OP,pdfset[0],RPS,FPS,false,without_combined_result);
		*(IT_CS)	= &QDCS_central;
	}

	if(N_pdf){
//		NS_Utils::DOUT dout("init","QDCS_pdf");

		NS_CS_Params::PDFset::iterator it_pdf(pdfset.begin());

//		QDCS_pdf.resize(N_pdf);
//		for(int i=0; i<N_pdf; ++i){
//			QDCS_pdf[i].init(OP,*(++it_pdf),RPS,FPS,false,without_combined_result);
//			*(++IT_CS)	= &(QDCS_pdf[i]);
//		}

		QDCS_pdf.assign(N_pdf,QDCS_central);
		for(int i=0; i<N_pdf; ++i){
			QDCS_pdf[i].change_PDF(*(++it_pdf));
			*(++IT_CS)	= &(QDCS_pdf[i]);
		}
	}

	if(N_mu){
//		NS_Utils::DOUT dout("init","QDCS_mu");

//		QDCS_mu.resize(N_mu);
//		for(int i=0; i<N_mu; ++i){
//			QDCS_mu[i].init(OP,pdfset[0],RPS,FPS,false,without_combined_result);//Quad_Differential_Crosssection(QDCS)
//			*(++IT_CS)	= &(QDCS_mu[i]);
//		}
//		QDCS_mu.resize(N_mu,QDCS_central);//Quad_Differential_Crosssection(QDCS)	// WRONG if QDCS_mu is yet filled

		QDCS_mu.assign(N_mu,QDCS_central);
		for(int i=0; i<N_mu; ++i){
			*(++IT_CS)	= &(QDCS_mu[i]);
		}
	}

	if(exp_errors){
//		NS_Utils::DOUT dout("init","QDCS_exponent");

		QDCS_exponent.init(OP,pdfset[0],RPS,FPS,exp_errors,without_combined_result);
		*(++IT_CS)	= &QDCS_exponent;
	}

	N_Comp	= N_Centre + (N_QDCS-1);

	transition_function.init(&OP);

//	delete CO_reduction; CO_reduction = NULL;
//	if(transition_to_FO.is_active()){
//		CO_reduction = new Continious_Order_Reduction;
//		CO_reduction->init(OP.Scheme.order());
//	}

}

vector<double>& Crosssection::update_qT_mu(const double qT, const double mu){
//	NS_Utils::DOUT dout("Crosssection::update_qT_mu","qT,mu");


	int dim(3);

	NS_CUBA_integration::cuba_integrand_t integrand;

	const bool OK(Kin.set_qT_get_ok(qT));
	_set_qT_mu(mu);

	if(OK){
		switch (res) {
		case RES_LL:
			integrand = _cuba_int_yxx<RES_LL>;
			break;
		case RES_LLO:
			integrand = _cuba_int_yxx<RES_LLO>;
			break;
		case RES_LO:
			integrand = _cuba_int_yxx<RES_LO>;
			break;
		case RES_MATCHING_CORR:
			integrand = _cuba_int_yxx<RES_MATCHING_CORR>;
			break;
		case RES_MATCHED:
			integrand = _cuba_int_yxx<RES_MATCHED>;
			break;
		case RES_RES_CORR:
			integrand = _cuba_int_yxx<RES_RES_CORR>;
			break;
		default:
			break;
		}

		return _update(dim,integrand);
	}else{
		return _unset_results();
	}
}

vector<double>& Crosssection::update_qT_mu_y(const double qT, const double mu, const double y){
//	NS_Utils::DOUT dout("Crosssection::update_qT_mu_y","qT,mu,y");
	const int dim(2);
	bool OK;
	NS_CUBA_integration::cuba_integrand_t integrand;

	switch (res) {
	case RES_LL:
		OK	= _set_qT_mu_y_get_OK<RES_LL>(qT,mu,y);
		integrand = _cuba_int_xx<RES_LL>;
		break;
	case RES_LLO:
		OK	= _set_qT_mu_y_get_OK<RES_LLO>(qT,mu,y);
		integrand = _cuba_int_xx<RES_LLO>;
		break;
	case RES_LO:
		OK	= _set_qT_mu_y_get_OK<RES_LO>(qT,mu,y);
		integrand = _cuba_int_xx<RES_LO>;
		break;
	case RES_MATCHING_CORR:
		OK	= _set_qT_mu_y_get_OK<RES_MATCHING_CORR>(qT,mu,y);
		integrand = _cuba_int_xx<RES_MATCHING_CORR>;
		break;
	case RES_MATCHED:
		OK	= _set_qT_mu_y_get_OK<RES_MATCHED>(qT,mu,y);
		integrand = _cuba_int_xx<RES_MATCHED>;
		break;
	case RES_RES_CORR:
		OK	= _set_qT_mu_y_get_OK<RES_RES_CORR>(qT,mu,y);
		integrand = _cuba_int_xx<RES_RES_CORR>;
		break;
	default:
		OK	= false;
		break;
	}
	if(OK){
		return _update(dim,integrand);
	}else{
		return _unset_results();
	}
}

std::vector<bool>& CuTe::Crosssection::to_plot(std::vector<bool>& to_plot, const bool with_y)const{
	to_plot.assign(ER_Last,false);

	to_plot[ER_qT]	= true;
	to_plot[ER_mu]	= true;
	if(with_y)	to_plot[ER_y]	= true;
	to_plot[ER_erg_result]	=	true;
	to_plot[ER_erg_Delta]	=	true;

//	 Set which results will be displayed

	if(partial_results){
	switch (res) {
		case RES_MATCHED:
			to_plot[ER_erg_RES]	= true;
			to_plot[ER_erg_FO]	= true;
			to_plot[ER_erg_SQT]	= true;
			to_plot[ER_erg_MC]	= true;
			to_plot[ER_erg_RC]	= true;
			break;
		case RES_MATCHING_CORR:
			to_plot[ER_erg_FO]	= true;
			to_plot[ER_erg_SQT]	= true;
			break;
		case RES_RES_CORR:
			to_plot[ER_erg_RES]	= true;
			to_plot[ER_erg_SQT]	= true;
			break;
		default:
			break;
		}
	}

	//	 Set which errors will be displayed
	if(N_mu){
		to_plot[ER_erg_mu_P]	= true;
		to_plot[ER_erg_mu_M]	= true;
	}
	if(N_pdf){
		to_plot[ER_erg_PDF_Delta]	= true;
		to_plot[ER_erg_PDF_DeltaM]	= true;
		to_plot[ER_erg_PDF_DeltaP]	= true;
		to_plot[ER_erg_PDF_Scale]	= false;
	}
	if(exp_errors){
		to_plot[ER_erg_exp]	= true;
	}
	to_plot[ER_erg_DCM]	= true;
	to_plot[ER_erg_DCP]	= true;

	return to_plot;
}
const std::vector<bool> CuTe::Crosssection::to_plot(const bool with_y)const{
	vector<bool> toplot;
	return to_plot(toplot,with_y);
}
std::ostream& CuTe::Crosssection::FL_into_stream(std::ostream& os, const std::vector<bool>& to_plot, const int width)const{
	os	<< "#";
	os.width(width);
	os	<< ER_qT;
	for(size_t i=1; i<to_plot.size(); ++i){
		if(to_plot[i]){
			os.width(width+1);
			os	<< static_cast<E_RESULTS>(i);
		}
	}
	return os;

}
std::ostream& CuTe::Crosssection::into_stream(std::ostream& os, const std::vector<bool>& to_plot, const int width, const int prec)const{
	const int old=os.precision(prec);
	for(size_t i=0; i<to_plot.size(); ++i){
		if(to_plot[i]){
			os	<< " ";
			os.width(width);
			os	<< results[i];
		}
	}
	os.precision(old);
	return os;
}
