#ifndef FO_HIGGS_H
#define FO_HIGGS_H

//#include "konst.h"
#include "utils.h"
#include "FO_Parameter.h"

namespace CuTe {

namespace NS_FO{

namespace NS_Higgs {

using namespace NS_Utils::NS_pow;

/*
 * LO
 */

inline double qg_Hq_LO(double s, double t, double u){		return (s*s+t*t)/(-u);}
inline double gq_Hq_LO(double s, double t, double u){		return qg_Hq_LO(s,u,t);}
inline double qqbar_Hg_LO(double s, double t, double u){	return -qg_Hq_LO(u,t,s);}
inline double qbarq_Hg_LO(double s, double t, double u){	return qqbar_Hg_LO(s,u,t);}

inline double qg_Hq_LO(const FO_NLO_Kinematic_Parameter& p){		return -(p.s2+p.t2)*p.iu;}
inline double qqbar_Hg_LO(const FO_NLO_Kinematic_Parameter& p){	return +(p.u2+p.t2)*p.is;}

//inline double gg_Hg_LO(double s, double t, double u, double M){
//	return (s*s*s*s+t*t*t*t+u*u*u*u+M*M*M*M*M*M*M*M)/(s*t*u);
//}
inline double gg_Hg_LO(double s, double t, double u, double M2){
	return (pow_4(s)+pow_4(t)+pow_4(u)+pow_4(M2))/(s*t*u);
}


inline double gg_Hg_LO(const FO_NLO_Kinematic_Parameter& p){
	return (pow_2(p.s2)+p.t4+p.u4+pow_4(p.M2))*(p.is*p.it*p.iu);
}



/*
 * NLO Coefficients
 */

inline double A0(const FO_NNLO_Kinematic_Parameter& p){

	double   s2(p.s2);
	double   t4(p.t4);
	double   u4(p.u4);
	double   x2(p.x2);
	double  qT2(p.qT2);
	double iza4(p.iza4);
	double izb4(p.izb4);
	double ixT2(p.ix2T);
	double  ftu(p.ftu);

//cout	<<"\t"<<"s="<<sqrt(s2)	<<"\t"<<"t="<<sqrt(sqrt(t4))	<<"\t"<<"u="<<sqrt(sqrt(u4))	<<endl
//		<<"\t"<<"s="<<p.s	<<"\t"<<"t="<<p.t	<<"\t"<<"u="<<p.u	<<endl
//		<<"\t"<<"mx2="<<x2	<<"\t"<<"pt2="<<QT2	<<"\t"<<"mxt2="<<1./ixT2	<<endl
//		<<"\t"<<"mx2="<<x2	<<"\t"<<"pt2="<<QT2	<<"\t"<<"mxt2="<<p.xT2	<<endl
//		<<"\t"<<"za="<<sqrt(sqrt(1./iza4))	<<"\t"<<"zb="<<sqrt(sqrt(1./izb4))	<<endl
//		<<"\t"<<"za="<<p.za	<<"\t"<<"zb="<<p.zb	<<endl
//		<<"\t"<<"ftu="<<ftu	<<endl
//		;
	return	x2*qT2*(
				 (17./3. + 4*ftu)*s2
				+(5. +x2*ixT2*(-7. +20./3.*x2*ixT2))*(t4*iza4 + u4*izb4)*(ixT2*ixT2)
	        );
}


inline double A1234(const FO_NNLO_Kinematic_Parameter& p)
{
	double M2(p.M2);
	double M4(M2*M2);
	double M8(M4*M4);
	double qT2(p.qT2);
	double qT4(qT2*qT2);
	double qT8(qT4*qT4);
	double x2(p.x2);
	double x4(p.x4);
	double x8(x4*x4);
	double s(p.s);
	double s2(p.s2);
	double s4(s2*s2);
	double t(p.t);
	double t2(p.t2);
	double it(p.it);
	double it2(p.it2);
	double it4(it2*it2);
	double u(p.u);
	double u2(p.u2);
	double u3(u*u2);
	double u4(p.u4);
	double ds(p.ds);	//=iA
	double ids(p.ids);	//=A
	double B2(p.GS_B2);
	double B4(B2*B2);
	double iB2(p.GS_iB2);
	double iB4(p.GS_iB4);
	double iB5(p.GS_iB*iB4);
	double fL1a(-p.fst);
	double fL1b(-p.fsu);
	double fL2a(p.fL2a);
	double fL2b(p.fL2b);
	double fL3(p.fL3);
	double iM2t(p.iM2t);
	double iM2u(p.iM2u);
	return (
				2.*M4*(x4)*(qT4)*(s2)*(-6.*iB4 + 8.*iB2*it2 - 4.*it4)
				-(fL1a*(M8*x8 + qT8*s4))*it4
				+(-fL1a + fL2a)*x2*qT2*s*(M8 + (M2 - u)*(M2 - u)*(M2 - u)*(M2 - u))*(ds*it*(-iM2u))
				+(M2*x2*qT2*s*u2*(t2 + u2))*iB2*it2
				+((fL1b - fL2b)*x2*qT2*s*u3)*(ds*iM2t)
				+(fL1a + fL1b)*x2*s*M4*u3*(iM2t*it*iM2u)
				-fL3*qT2*s*(
						M2*x2*t*(ids*ids)*(
								-6.*M2*x2*qT2*s
								+B2*(-6.*M2*x2 + 3.*t*u + t2 + 3.*u2)
						)
						+M2*x2*t*(B2)*(
								2.*M2*x2*qT2*s
								+B2*(-2.*M2*x2 + t*u + t2 + 7.*u2)
						)
						-2.*ids*(B4)*(
								2.*M4*x4 - M2*x2*(t*u + t2 + 4*u2)
								+(t + u)*u3
						)
					)*ds*it*iB5
				-fL1b*u4
			)/2.;
}


inline double A3412(const FO_NNLO_Kinematic_Parameter& p)
{
	double M2(p.M2);
	double M4(M2*M2);
	double M8(M4*M4);
	double QT2(p.qT2);
	double QT4(QT2*QT2);
	double QT6(QT2*QT4);
	double x2(p.x2);
	double x4(p.x4);
	double x8(x4*x4);
	double xT2(p.x2T);
	double xT4(xT2*xT2);
	double ixT2(p.ix2T);
	double ixT4(ixT2*ixT2);
	double ixT6(ixT4*ixT2);
	double s(p.s);
	double s2(p.s2);
	double s4(s2*s2);
	double t(p.t);
	double t2(p.t2);
	double it(p.it);
	double it2(p.it2);
	double u(p.u);
	double u2(p.u2);
	double iu(p.iu);
	double iu2(p.iu2);
	double iu4(iu2*iu2);
	double A(p.ids);	//=A
	double A2(A*A);
	double A4(A2*A2);
	double dt(p.dt);
	double idt(p.idt);
	double idt2(idt*idt);
	double du(p.du);
	double idu(p.idu);
	double idu2(idu*idu);
	double B2(p.GS_B2);
	double B4(B2*B2);
	double iB(p.GS_iB);
	double L1b(-p.fsu);
	double L2a(p.fL2a);
	double L3(p.fL3);
	double M2t(p.M2t);
	double M2u(p.M2u);
	double iM2u(p.iM2u);

	return (
				QT2*s*(
					+(24.*(L1b+L2a)*x2*(A2*A))*(it*(-iM2u))
					+(3.*L3*(t+u)*(A4+6.*(A2)*(B2)+B4))*(iB*it*iu)
					+(48.*s*(M4))*iu
					-(8.*s*(15.*(M4*M2)+27.*M2*(x4)+4.*(x4*x2)))*(it*iu)
					+24*(1.-(2.*QT2)*ixT2)*idu*(
							x2*u*(M2*(-2.*x2+t)+x2*u)
							+3.*x2*M2t*(M2*idt-x2*u+M4)
							+3.*(-(s*t)+x2*(s+u))*(M2t*M2t)
					)*(dt*it*iu)
					+(12*(2*xT2*QT2+xT4-6*(QT4))*(
							-3*(-M2u)*(x2*(s+t)-s*u)
							+3*x2*(x2*(-M2t)+M2*(-M2u))
							+4*u*(s2)
					)*(idt2))*(it*du*iu*ixT4)
					+(4*(x2*(s+t)-s*u)*(
							3*QT2*(xT4)
							+2*(xT4*xT2)
							+6*xT2*(QT4)
							-22*(QT6)
						)*(idt2*idt)
					)*(it*du*iu*ixT6)
					+(12*(
						-3*(-M2t)*(-(s*t)+x2*(s+u))
						+x2*(M2*idt-x2*u+M4)
						)*(2*xT2*QT2+xT4-6*(QT4))*(idu2)
					)*(dt*it*iu*ixT4)
					+(24*(1-(2*QT2)*ixT2)*idt*(
							M2*x2*(-x2+t)*u
							+x2*u*(M2*u+t*(x2-2*(t+2*u))+4*(M4))
							+3*M2*(-M2t)*(x4)
							+8*t*u*(s2)
							+2*u*(s2*s)
							-3*M2*x2*(M2t*M2t)
							-3*(M2-x2)*x2*(t2)
							+3*(x2*(s+t)-s*u)*(M2u*M2u)
							-2*x2*u*(idu2))
					)*(it*du*iu)
					+(4*(-(s*t)+x2*(s+u))*(
							3*QT2*(xT4)
							+2*(xT4*xT2)
							+6*xT2*(QT4)-22*(QT6)
						)*(idu2*idu)
					)*(dt*it*iu*ixT6)
					+8*s*(M4)*((x2+s)*(it*iu)+x2*(it2)+x2*iu2)
					+(24*M2*x2*(B2))*iu2
					+(96*x2*s*(M4))*iu2
					+(4*(
							-22*x2*(M4*M2)
							+17*(M8)
							+4*(x8)
							+11*(s4)
							+M4*(
									-52*t*u
									+2*x2*(67*t+55*u)
									+20*(x4)
									-35*(t2)
									-23*(u2)
								)
							-x4*(52*t*u+33*(t2)+21*(u2))
							+x2*(73*u*(t2)+29*(t2*t)+61*t*(u2)+17*(u2*u))
							+M2*(
									52*(x4*x2)
									-x2*(170*t*u+109*(t2)+73*(u2))
									+6*(6*u*(t2)+3*(t2*t)+4*t*(u2)+u2*u)))
					)*(it*iu)
					-
					(192*QT2*s*(M4)*(x4))*(iu4)
					-(24*L1b*(
							-4*M2*x2*t*(A2)*(u2)
							+2*x2*s*t*(M4)*(x2*QT2+2*(u2))
							-2*x2*(M4*M2)*(u2*u)
							+M8*(u2*u)
							+(x2-s)*(x2-s)*(2*A*x2+(x2-s)*(x2-s))*(u2*u)
						))*(it*iu4)
				)
			)/48.;
}


inline double A1324(const FO_NNLO_Kinematic_Parameter& p)
{
	double M2(p.M2);
	double M4(M2*M2);
	double M8(M4*M4);
	double QT2(p.qT2);
	double QT4(QT2*QT2);
	double QT8(QT4*QT4);
	double x2(p.x2);
	double x4(p.x4);
	double x8(x4*x4);
	double s(p.s);
	double s2(p.s2);
	double s4(s2*s2);
	double t(p.t);
	double it(p.it);
	double it2(p.it2);
	double it4(it2*it2);
	double u(p.u);
	double u2(p.u2);
	double u4(p.u4);
	double iA(p.ds);
	double za(p.za);
	double zb(p.zb);
	double iB(p.GS_iB);
	double L1a(-p.fst);
	double L1b(-p.fsu);
	double L2a(p.fL2a);
	double L2b(p.fL2b);
	double L3(p.fL3);
	double iM2t(p.iM2t);
	double M2u(p.M2u);
	double iM2u(p.iM2u);
	return (
				(2*L3*x2*QT2*(M4)*(s2))*(iA*iB)
				+(2*L1a*x2*QT2*(M4)*(s2))*(it2)
				+(M2*x2*QT2*s*(-6*M2*x2*QT2*s+M4*(x4)+QT4*(s2)))*(it4)
				-(L1a*(M8*(x8)+QT8*(s4)))*(it4)
				+((-L1a+L2a)*QT2*(1-za)*(s2)*(M8+(M2u*M2u*M2u*M2u)))*(-iA*it*iM2u)
				+((L1a+L1b)*s*(-(QT2*t*u)+M2*QT2*(t+u)-(x2+QT2)*(M4))*(u2*u))*(-it*iM2t*iM2u)
				-((L1b-L2b)*QT2*(-1+zb)*(s2)*(u2*u))*(iA*iM2t)
				-L1b*(u4)
			)/2.;
}


/**
*@noteThisiswithoutthet<->upart
*/
inline double A3241(const FO_NNLO_Kinematic_Parameter& p)
{
	double M2(p.M2);
	double M4(M2*M2);
	double QT2(p.qT2);
	double x2(p.x2);
	double x4(p.x4);
	double xT2(p.x2T);
	double xT4(xT2*xT2);
	double ixT2(p.ix2T);
	double ixT4(ixT2*ixT2);
	double s(p.s);
	double t(p.t);
	double t2(p.t2);
	double it2(p.it2);
	double it4(it2*it2);
	double u(p.u);
	double iu(p.iu);
	double iu2(p.iu2);
	double A(p.ids);	//=A
	double A2(A*A);
	double dt(p.dt);
	double dt2(dt*dt);
	double idt(p.idt);
	double idt2(idt*idt);
	double idu(p.idu);
	double idu2(idu*idu);
	double za(p.za);
	double B2(p.GS_B2);
	double L1a(-p.fst);
	double L2a(p.fL2a);
	double iM2t(p.iM2t);
	double iM2u(p.iM2u);
	return (
				QT2*s*(
						-16*QT2*s*(M4)*(x4)
						+2*L1a*M2*x2*(
								A2*(4+(t*(4-(A2)*(iM2t*iM2u)))*iu)*(t2)
								-(2*M2*(s*(t+u)*(t2)+x2*(QT2*s*u-t2*t)))*iu
							)
						-(2*(L1a-L2a)*s*(-1+za)*(A2*A)*(t2*t))*iM2u
						+(4*x2*(-1+(2*x2)*ixT2)*s*idu*(-(M2*idt*idu)+t*idu*u-M2*(idt2)-idt2*idt)*(t2*t))*(iu*dt2)
						+(2*x2*s*(-2*x2*t+t*(t+u)+x4)*(-10*x2*xT2+6*(x4)+3*(xT4))*(t2*t)*(idu2))*(iu*ixT4*dt2)
						+
						(x2*s*(M4)*(t2)*((t+u)*(t+u)))*iu2+
						(x2*s*(t2*t)*(8*M2*x2-2*(2*M2+3*x2)*(t+u)+6*(x4)+
						(t+u)*(t+u)))*iu+
						2*M2*x2*(t2)*(-4*M2*x2+B2+(t*((t+u)*(t+u)))*iu)
				)
			)*(0.25*it4);
}


inline double Aeps(const FO_NLO_Kinematic_Parameter& p)
{
	double M2(p.M2);
	double M4(M2*M2);
	double QT2(p.qT2);
	double QT4(QT2*QT2);
	double x4(p.x4);
	double s2(p.s2);
	double it2(p.it2);
	double it4(it2*it2);
	double iu2(p.iu2);
	double iu4(iu2*iu2);
	return 4*(M4)*(x4)*(QT4)*(s2)*(it4+iu4);
}


/**
*@noteThisiswithoutthet<->upart
*/
inline double B1pm(const FO_NNLO_Kinematic_Parameter& p)
{
	double QT2(p.qT2);
	double QT4(QT2*QT2);
	double x2(p.x2);
	double s2(p.s2);
	double t(p.t);
	double t2(p.t2);
	double it(p.it);
	double it2(p.it2);
	double za(p.za);
	double ftu(p.ftu);
	return -((
				QT2*(s2)*(
						(1+ftu)*x2*(t2*t)
						+(-1+za)*za*(s2)*(
								-4*QT2*t*(-1+za)*za
								+t2*((-1+za)*(-1+za))
								+QT4*(za*za)
							)
					)
			)*(it2*it));
}


/**
*@noteThisiswithoutthet<->upart
*/
inline double B1pp(const FO_NNLO_Kinematic_Parameter& p)
{
	double QT2(p.qT2);
	double QT4(QT2*QT2);
	double x2(p.x2);
	double x4(p.x4);
	double xT2(p.x2T);
	double ixT2(p.ix2T);
	double ixT4(ixT2*ixT2);
	double s(p.s);
	double s2(p.s2);
	double t(p.t);
	double t2(p.t2);
	double it(p.it);
	double it2(p.it2);
	double u(p.u);
	double iu(p.iu);
	double za(p.za);
	double fx2(p.fx2);
	double fstu(p.fstu);
	return (
			QT2*(s2)*(
					-(((fx2+fstu)*x2*(x4+s2)*(t2))*iu)
					-(x2*(t2)*(2*x2*xT2*QT2*s+(x2*QT2+t*u)*(x2*QT2+t*u)))*(iu*ixT4)
					-za*(QT4)*(s2)*((-1+za)*(-1+za)*(-1+za))
					+4*QT2*t*(s2)*((-1+za)*(-1+za))*(za*za)
					-(-1+za)*(s2)*(t2)*(za*za*za)
				)
			)*(it2*it);
}


/**
*@noteThisiswithoutthet<->upart
*/
inline double B2pm(const FO_NNLO_Kinematic_Parameter& p)
{
	double QT2(p.qT2);
	double QT4(QT2*QT2);
	double QT6(QT2*QT4);
	double x2(p.x2);
	double x4(p.x4);
	double xT2(p.x2T);
	double xT4(xT2*xT2);
	double ixT2(p.ix2T);
	double ixT4(ixT2*ixT2);
	double ixT8(ixT4*ixT4);
	double s2(p.s2);
	double t4(p.t4);
	double iza4(p.iza4);
	return (QT2*(-(x2*(s2)) - ((x4*x2+xT4*xT2-QT6)*(t4))*(ixT8*iza4)))/3.;
}


/**
*@noteThisiswithoutthet<->upart
*/
inline double B2pp(const FO_NNLO_Kinematic_Parameter& p)
{
	double M2(p.M2);
	double M4(M2*M2);
	double QT2(p.qT2);
	double x2(p.x2);
	double x4(p.x4);
	double ixT2(p.ix2T);
	double ixT4(ixT2*ixT2);
	double s(p.s);
	double s2(p.s2);
	double s4(s2*s2);
	double t(p.t);
	double t2(p.t2);
	double it2(p.it2);
	double u(p.u);
	double u2(p.u2);
	double iu(p.iu);
	double dt(p.dt);
	double dt2(dt*dt);
	double idt(p.idt);
	double idt2(idt*idt);
	double idu(p.idu);
	double idu2(idu*idu);
	double fx2(p.fx2);
	double fstu(p.fstu);
	double M2t(p.M2t);
	return (QT2*s*(
			-4*x2*s*(M4)
			+(22*t*(x4)*(s2))*iu
			+(6*(fx2+fstu)*x2*s*t*(x4+s2))*iu
			+(6*(1-(2*x2)*ixT2)*t*idu*(
					3*x2*s*(x2+s)*idt
					+x2*u*((M2-x2)*(M2-x2))
					+M2*x2*(s2)
					-t*(s2*s))
				)*(dt*iu)
			+(t*(
					6*M2*s*t*(-M2t)
					-4*M2*x2*t*u
					-2*x2*s*t*u
					-x2*(M4*M2)
					+2*s*(M4*M2)
					-2*M2*s*(x4)
					+3*M2*(x4*x2)
					+8*x2*s*((x2-s)*(x2-s))
					+7*M2*x2*(s2)
					-2*(s4)
				))*iu
			+(6*t*(-(s*t)+x2*(s+u))*(2./3.+x2*ixT2-(10*(x4*x2))*(ixT4*ixT2)/(3.))*(idu2*idu))*(dt*iu)
			-(6*t*(1-(2*(x4))*ixT4)*(idu2)*(
					x2*idt*(2*t*u+M4)
					+(4*M2*x2-(M2+x2)*u+s2)*(idt2)
					+3*x2*(idt2*idt)+M2*u*(t2)-x4*(u2)
				))*(iu*dt2)
		))/12.*it2;
}


inline double C1pm(const FO_NNLO_Kinematic_Parameter& p)
{
	double QT2(p.qT2);
	double x2(p.x2);
	double s2(p.s2);
	double ftu(p.ftu);
	return -2*ftu*x2*QT2*(s2);
}


inline double C1mp(const FO_NNLO_Kinematic_Parameter& p)
{
	double QT2(p.qT2);
	double x2(p.x2);
	double x4(p.x4);
	double ixT2(p.ix2T);
	double ixT4(ixT2*ixT2);
	double s(p.s);
	double s2(p.s2);
	double t2(p.t2);
	double iu(p.iu);
	double idt(p.idt);
	double idt2(idt*idt);
	double idu(p.idu);
	double idu2(idu*idu);
	return (QT2*(
					2*x2*(s2)-(3*(s2)*(t2))*iu
					+((-3+(10*x2)*ixT2-(6*(x4))*ixT4)*(s*(idt2*idt)+x2*(idu2*idu)))*ixT2
				)
			)/2.;
}


inline double C1pp(const FO_NNLO_Kinematic_Parameter& p)
{
	double M2(p.M2);
	double M4(M2*M2);
	double QT2(p.qT2);
	double x2(p.x2);
	double x4(p.x4);
	double xT2(p.x2T);
	double ixT2(p.ix2T);
	double ixT4(ixT2*ixT2);
	double s(p.s);
	double s2(p.s2);
	double t(p.t);
	double t2(p.t2);
	double it(p.it);
	double it2(p.it2);
	double u(p.u);
	double iu(p.iu);
	double iu2(p.iu2);
	double A(p.ids);	//=A
	double A2(A*A);
	double dt(p.dt);
	double idt(p.idt);
	double idt2(idt*idt);
	double du(p.du);
	double idu(p.idu);
	double idu2(idu*idu);
	double iB(p.GS_iB);
	double L1a(-p.fst);
	double L2b(p.fL2b);
	double L3(p.fL3);
	return (QT2*s*(
				(2*M2*x2*(t+u))*it
				-(2*L2b*x2*(A2))*iu
				+(2*L3*(-4*A*(A-M2)*x2*s+(-M2+x2+s)*(x2*(A2)+s*((A-M2)*(A-M2)))))*(iB*iu)
				+(2*(2*x2-xT2)*idt*(2*x2*t*u+2*s*(s+t)*u+M2*x2*(-4*x2+u)+t*(x4)))*(ixT2*du*iu)
				+(5*x2*s*(-x2+s)+M2*x2*(3*M2-2*x2+2*s-u)-2*(M2+x2)*s*u+s*(M4)+2*M2*(s2))*iu
				+(2*(-1+(2*x2)*ixT2)*idu*(2*x2*idt*u-2*s*t*u+t*(s2)))*(dt*iu)
				-(3*(s2*s))*iu
				-(4*(M4)*(x4))*(it2)
				+(2*L1a*(-2*A*M2*x2*t*u-x2*idt*u*(M4)+s*((A-M2)*(A-M2))*(t2)))*(iu*it2)
				+(-3+(10*x2)*ixT2-(6*(x4))*ixT4)*((s*du-x2*iu)*(idt2)+(x2*dt+s*iu)*(idu2))
				+(x2*s*(M4))*iu2
			))/2.;
}


inline double C1mm(const FO_NNLO_Kinematic_Parameter& p)
{
	double M2(p.M2);
	double M4(M2*M2);
	double QT2(p.qT2);
	double x2(p.x2);
	double s(p.s);
	double t(p.t);
	double t2(p.t2);
	double u(p.u);
	double iu(p.iu);
	double B2(p.GS_B2);
	double B4(B2*B2);
	double B(p.GS_B);
	double iB(p.GS_iB);
	double iB2(p.GS_iB2);
	double L1a(-p.fst);
	double L2b(p.fL2b);
	double L3(p.fL3);
	double M2t(p.M2t);
	return (QT2*s*(
					B*M2*x2*u*(-2*M2*x2+t*(t+u))
					+L3*x2*u*(2*M2*x2+x2*(t-u)-t*(t+u))*(M4)
					+L3*(B4)*(t2)
					+B2*B*(-(L2b*x2*(M2t*M2t))+L1a*s*(t2))
					+L3*M2*(B2)*(M2*x2*(2*x2+3*t-u)-(t+u)*(t2))
			))*(iu*iB2*iB);
}


inline double C2pm(const FO_NLO_Kinematic_Parameter& p)
{
	return 0;
}


inline double C2mp(const FO_NNLO_Kinematic_Parameter& p)
{
	double QT2(p.qT2);
	double x2(p.x2);
	double ixT2(p.ix2T);
	double ixT4(ixT2*ixT2);
	double s(p.s);
	double s2(p.s2);
	double idt(p.idt);
	double idt2(idt*idt);
	double idu(p.idu);
	double idu2(idu*idu);
	double ftu(p.ftu);
	return x2*QT2*(
				2*(s2)
				+4*ftu*(s2)
				+((-2+(3*x2)*ixT2)*(s*(idt2*idt)+x2*(idu2*idu)))*(ixT4)
			);
}


inline double C2pp(const FO_NNLO_Kinematic_Parameter& p)
{
	double M2(p.M2);
	double M4(M2*M2);
	double QT2(p.qT2);
	double x2(p.x2);
	double x4(p.x4);
	double ixT2(p.ix2T);
	double ixT4(ixT2*ixT2);
	double s(p.s);
	double t(p.t);
	double u(p.u);
	double u2(p.u2);
	double iu(p.iu);
	double iu2(p.iu2);
	double iu4(iu2*iu2);
	double A(p.ids);	//=A
	double A2(A*A);
	double dt(p.dt);
	double idt(p.idt);
	double idt2(idt*idt);
	double du(p.du);
	double idu(p.idu);
	double idu2(idu*idu);
	double za(p.za);
	double zb(p.zb);
	double L1a(-p.fst);
	double L1b(-p.fsu);
	double L2a(p.fL2a);
	double L2b(p.fL2b);
	double M2t(p.M2t);
	double iM2u(p.iM2u);
	return (QT2*s*(
					((M2+x2)*s*(-t+u))*iu
					+((-1+(2*x2)*ixT2)*idu*(3*QT2*s*(x2+s)+3*M2*x2*idu-x2*idt*u+s*t*(s+u)))*(dt*iu)
					+(2*(L1b-L2a)*x2*(A2))*iM2u
					+(L1a-L2a)*(-1+za)*(A2)
					-((L1b-L2b)*M2t*(-1+zb)*(A2))*iu
					+((L1a-L1b)*((x2-s)*(x2-s)*(x2-s)))*iu
					-(-3+(10*x2)*ixT2-(6*(x4))*ixT4)*((s*du-x2*iu)*(idt2)+(x2*dt+s*iu)*(idu2))
					+(M2*x2*(-4*M2*x2+2*M2*(M2-s)+u*(-x2+u)+2*((x2-s)*(x2-s))))*iu2
					+(2*L1b*x2*(4*A*M2*M2t-2*idt*(M4)-M4*M2+2*u*((x2-s)*(x2-s))))*iu2
					+((-1+(2*x2)*ixT2)*idt*(-(M2*s*u)+2*idu*(x4)+(x2+s)*(-3*QT2*s-x4+u2)))*(du*iu)
					-(4*(4+L1b)*QT2*s*(M4)*(x4))*(iu4)
			))/2.;
}


inline double C2mm(const FO_NNLO_Kinematic_Parameter& p)
{
	double M2(p.M2);
	double M4(M2*M2);
	double QT2(p.qT2);
	double x2(p.x2);
	double x4(p.x4);
	double s(p.s);
	double t(p.t);
	double t2(p.t2);
	double iu(p.iu);
	double iu2(p.iu2);
	double iu4(iu2*iu2);
	double za(p.za);
	double zb(p.zb);
	double L1a(-p.fst);
	double L1b(-p.fsu);
	double L2a(p.fL2a);
	double L2b(p.fL2b);
	double M2t(p.M2t);
	double iM2u(p.iM2u);
	return (QT2*s*(
				(2*(L1b+L2a)*x2*(t2))*iM2u
				+((L1a+3*L1b)*x2*(t2))*iu
				+((-L1a+L1b)*s*(t2))*iu
				+(L1a-L2a)*(-1+za)*(t2)
				+(2*L1b*M2*x2*((M2-2*t)*(M2-2*t)))*iu2
				+((-L1b+L2b)*x2*zb*(M2t*M2t*M2t))*iu2
				+(2*M2*x2*(t2))*iu2
				-(4*(4+L1b)*QT2*s*(M4)*(x4))*(iu4)
			))/2.;
}


inline double C2eps(const FO_NLO_Kinematic_Parameter& p)
{
	double M2(p.M2);
	double M4(M2*M2);
	double QT2(p.qT2);
	double QT4(QT2*QT2);
	double x4(p.x4);
	double s2(p.s2);
	double iu2(p.iu2);
	double iu4(iu2*iu2);
	return (4*(M4)*(x4)*(QT4)*(s2))*(iu4);
}


/**
*@noteThisiswithoutthet<->upart
*/
inline double D1pm(const FO_NNLO_Kinematic_Parameter& p)
{
	double QT2(p.qT2);
	double x2(p.x2);
	double s(p.s);
	double s2(p.s2);
	double t(p.t);
	double it(p.it);
	double za(p.za);
	double ftu(p.ftu);
	return (QT2*(-((1+ftu)*x2*t)-s*(-1+za)*(t*(-1+za)-QT2*za))*(s2))*it;
}


/**
*@noteThisiswithoutthet<->upart
*/
inline double D1pp(const FO_NNLO_Kinematic_Parameter& p)
{
	double M2(p.M2);
	double M4(M2*M2);
	double QT2(p.qT2);
	double x2(p.x2);
	double s(p.s);
	double t(p.t);
	double it2(p.it2);
	double u(p.u);
	double u2(p.u2);
	double iA(p.ds);
	double za(p.za);
	double zb(p.zb);
	double iB(p.GS_iB);
	double L1a(-p.fst);
	double L1b(-p.fsu);
	double L2a(p.fL2a);
	double L2b(p.fL2b);
	double L3(p.fL3);
	double M2u(p.M2u);
	return QT2*s*(
				(-2*L3*x2*s*(M4))*(iA*iB)
				+(M2*x2*(2*QT2*s-t*u))*(it2)
				+(L1a*M2*x2*(QT2*s+t*u))*(it2)
				+((L1a-L2a)*s*(-1+za)*((M2u*M2u)))*iA
				+((L1b-L2b)*s*(-1+zb)*(u2))*iA
			);
}


/**
*@noteThisiswithoutthet<->upart
*/
inline double D2pm(const FO_NNLO_Kinematic_Parameter& p)
{
	double QT2(p.qT2);
	double x2(p.x2);
	double x4(p.x4);
	double ixT2(p.ix2T);
	double ixT4(ixT2*ixT2);
	double s(p.s);
	double t2(p.t2);
	double iza2(p.iza2);
	return (QT2*s*(
				-2*x2*s+11*(t2)
				-((11-(12*x2)*ixT2+(3*(x4))*ixT4)*(t2))*iza2
			))/6.;
}


/**
*@noteThisiswithoutthet<->upart
*/
inline double D2pp(const FO_NNLO_Kinematic_Parameter& p)
{
	double M2(p.M2);
	double M4(M2*M2);
	double QT2(p.qT2);
	double x2(p.x2);
	double x4(p.x4);
	double s(p.s);
	double t(p.t);
	double t2(p.t2);
	double u(p.u);
	double u2(p.u2);
	double iA(p.ds);
	double A(p.ids);	//=A
	double A2(A*A);
	double za(p.za);
	double zb(p.zb);
	double B2(p.GS_B2);
	double B4(B2*B2);
	double iB(p.GS_iB);
	double iB2(p.GS_iB2);
	double iB4(p.GS_iB4);
	double L1a(-p.fst);
	double L1b(-p.fsu);
	double L2a(p.fL2a);
	double L2b(p.fL2b);
	double L3(p.fL3);
	double M2t(p.M2t);
	double M2u(p.M2u);
	return (QT2*s*(
					(24*QT2*s*(M4)*(x4))*iB4
					-(2*(L1a-L2a)*(-1+za)*((M2u*M2u)*M2u))*iA
					-2*(L1a+L1b)*(u2)
					-(2*(L1b-L2b)*M2t*(-1+zb)*(u2))*iA
					-(4*M2*x2*(u2))*iB2
					+(L3*(
							2*A*(B4)*(M2*x2*(t-3*u)+2*(t+u)*(u2))
							+M2*x2*(A2)*(-12*M2*x2*QT2*s+B4+2*(B2)*(u2))
							+M2*x2*(B2)*(4*M2*x2*QT2*s+B2*(-6*t*u+t2+7*(u2)))
						))*(iA*iB4*iB)
			))/4.;
}


inline double E1(const FO_NLO_Kinematic_Parameter& p)
{
	double M2(p.M2);
	double QT2(p.qT2);
	double x2(p.x2);
	double s(p.s);
	return (-4*x2*QT2*(M2-2*s)*s)/3.;
}


inline double E2(const FO_NNLO_Kinematic_Parameter& p)
{
	double QT2(p.qT2);
	double x2(p.x2);
	double ixT2(p.ix2T);
	double ixT4(ixT2*ixT2);
	double s(p.s);
	double idt(p.idt);
	double idt2(idt*idt);
	double idu(p.idu);
	double idu2(idu*idu);
	return x2*QT2*s*(2*s+((x2-QT2)*(idt2+idu2))*ixT4);
}


inline double E3(const FO_NNLO_Kinematic_Parameter& p)
{
	double QT2(p.qT2);
	double x2(p.x2);
	double xT2(p.x2T);
	double ixT2(p.ix2T);
	double ixT4(ixT2*ixT2);
	double s(p.s);
	double t(p.t);
	double u(p.u);
	double idt(p.idt);
	double idt2(idt*idt);
	double idu(p.idu);
	double idu2(idu*idu);
	double ftu(p.ftu);
	return QT2*s*(
				-6*x2*s-(x2*(x2+2*xT2)*(idt2+idu2))*(ixT4)
				-2*ftu*(-2*QT2*s+(-2*x2+t+u)*(-2*x2+t+u))
			);
}


inline double E4(const FO_NNLO_Kinematic_Parameter& p)
{
	double QT2(p.qT2);
	double x2(p.x2);
	double x4(p.x4);
	double ixT2(p.ix2T);
	double s(p.s);
	double s2(p.s2);
	double fx2(p.fx2);
	double fstu(p.fstu);
	double ftu(p.ftu);
	return 2*x2*QT2*s*(2*ftu*s-((fx2+fstu-ftu)*(x4+s2))*ixT2);
}



inline double GG_2R_ns(const FO_NNLO_Kinematic_Parameter& p,const FO_NNLO_Kinematic_Parameter& p_tu, const FO_Scale_Parameter & ps){
	double is2(p.is2);
	double iQT2(p.iqT2);
	double ix2(p.ix2);

	int nf(ps.nf);

	double a0	= A0(p);
	double a1234= A1234(p);
	double a2341= A1234(p_tu);
	double a3412= A3412(p);
	double a4123= A3412(p_tu);
	double a1324= A1324(p);
	double a2413= A1324(p_tu);
	double a3241= A3241(p)+A3241(p_tu);
	double aeps	= Aeps(p);
	double b1pm	= B1pm(p)+B1pm(p_tu);
	double b1pp	= B1pp(p)+B1pp(p_tu);
	double b2pm	= B2pm(p)+B2pm(p_tu);
	double b2pp	= B2pp(p)+B2pp(p_tu);
//	cout	<<"\t"<<"a0 ="<<a0	<<endl;
//	cout	<<"\t"<<"a1234 "<<a1234	<<endl;
//	cout	<<"\t"<<"a2341 "<<a2341	<<endl;
//	cout	<<"\t"<<"a3412 "<<a3412	<<endl;
//	cout	<<"\t"<<"a4123 "<<a4123	<<endl;
//	cout	<<"\t"<<"a1324 "<<a1324	<<endl;
//	cout	<<"\t"<<"a2413 "<<a2413	<<endl;
//	cout	<<"\t"<<"a3241 "<<a3241	<<endl;
//	cout	<<"\t"<<"aeps "<<aeps	<<endl;
//	cout	<<"\t"<<"b1pm "<<b1pm	<<endl;
//	cout	<<"\t"<<"b1pp "<<b1pp	<<endl;
//	cout	<<"\t"<<"b2pm "<<b2pm	<<endl;
//	cout	<<"\t"<<"b2pp "<<b2pp 	<<endl;
	return	(is2*iQT2*ix2)*(
					C_A*C_A*(a0+a1234+a2341+a3412+a4123+a1324+a2413+a3241+aeps)
					+nf*C_F*(b1pm+b1pp)
					+nf*C_A*(b2pm+b2pp)
			);
}



inline double qG_2R_ns(const FO_NNLO_Kinematic_Parameter& p, const FO_NNLO_Kinematic_Parameter& p_tu){
	double is2(p.is2);
	double iQT2(p.iqT2);
	double ix2(p.ix2);

	double c1pm	= C1pm(p);
	double c1mp	= C1mp(p_tu);
	double c1pp	= C1pp(p);
	double c1mm	= C1mm(p_tu);
	double c2pm	= C2pm(p);
	double c2mp	= C2mp(p_tu);
	double c2pp	= C2pp(p);
	double c2mm	= C2mm(p_tu);
	double c2eps= C2eps(p);

	// FIXME p <-> p_tu ??

	return (is2*iQT2*ix2)*(
					C_F*C_F*(c1pm+c1mp+c1pp+c1mm)
					+C_A*C_F*(c2pm+c2mp+c2pp+c2mm+c2eps)
				);
}

/*
 * qbq_2R_ns - qp_2R_ns
 */
inline double qbq_2R_ns(const FO_NNLO_Kinematic_Parameter& p, const FO_NNLO_Kinematic_Parameter& p_tu,const FO_Scale_Parameter &ps){
	double is2(p.is2);
	double iQT2(p.iqT2);
	double ix2(p.ix2);

	int nf(ps.nf);

	double d1pm	= D1pm(p)+D1pm(p_tu);
	double d1pp	= D1pp(p)+D1pp(p_tu);
	double d2pm	= D2pm(p)+D2pm(p_tu);
	double d2pp	= D2pp(p)+D2pp(p_tu);
	double e1	= E1(p);
//	double e2	= E2(p);
	double e3	= E3(p);

	return	(is2*iQT2*ix2)*(
					2*C_F*C_F*C_F*(d1pm+d1pp)
					+2*C_A*C_F*C_F*(d2pm+d2pp)
					+nf*C_F*C_F*e1
//					+C_F*C_F*e2
					+C_F*C_F/C_A*e3
				);
}

inline double qp_2R_ns(const FO_NNLO_Kinematic_Parameter& p){
	double iQT2(p.iqT2);
	double ix2(p.ix2);
	double is2(p.is2);

	double e2		= E2(p);

	return (is2*iQT2*ix2)*(C_F*C_F*e2);
}



//some functions, in particular for the real parts=============================
inline double gg_help1(const FO_NNLO_Kinematic_Parameter& p)
{
	double M2(p.M2);
	double M4(M2*M2);
	double M8(M4*M4);
	double x4(p.x4);
	double x8(x4*x4);
	double s2(p.s2);
	double s4(s2*s2);
	double is(p.is);
	double t(p.t);
	double t4(p.t4);
	double it(p.it);
	double u(p.u);
	double u4(p.u4);
	double iu(p.iu);
	double dt(p.dt);
	double idt(p.idt);
	double idt2(idt*idt);
	double idt4(idt2*idt2);
	double du(p.du);
	double idu(p.idu);
	double idu2(idu*idu);
	double idu4(idu2*idu2);

	return -((-M8 - x8 - s4 - t4 - dt*du*(idt4 + idu4 + M8 + x8 + s4)*t*u - u4)*(is*it*iu));
}


inline double gg_help2(const FO_NNLO_Kinematic_Parameter& p){
	double M2(p.M2);
	double M4(M2*M2);
	double M8(M4*M4);
	double x2(p.x2);
	double s(p.s);
	double s2(p.s2);
	double s4(s2*s2);
	double t(p.t);
	double t4(p.t4);
	double u(p.u);
	double dt(p.dt);
	double dt2(dt*dt);
	double dt4(dt2*dt2);

	return -((-M8 - t4 - dt4*s4*t4 - dt4*((M2*x2 - t*u)*(M2*x2 - t*u)*(M2*x2 - t*u)*
	          (M2*x2 - t*u)))/(s*(M2*x2 - t*u)));
}


inline double ggh1(const FO_NNLO_Kinematic_Parameter& p){
	double x8(pow_2(p.x4));
	double s4(pow_2(p.s2));
	double is(p.is);
	double it(p.it);
	double iu(p.iu);
	double za(p.za);
	double zb(p.zb);
	double M8(pow_4(p.M2));
	double idt4(pow_4(p.idt));
	double idu4(pow_4(p.idu));


	return	(
				x8
				+
				za*zb*(
					 x8
					+M8
					+s4
					+idt4
					+idu4
				)
			)*(is*it*iu);
}

inline double gghbeta(const FO_NNLO_Kinematic_Parameter& p){
	double s4(pow_2(p.s2));
	double is(p.is);
	double it(p.it);
	double iu(p.iu);
	double za(p.za);
	double zb(p.zb);
	double M8(pow_4(p.M2));
	double idt4(pow_4(p.idt));
	double idu4(pow_4(p.idu));


	return	(
				 M8
				+s4
				+za*zb*(
					 idt4
					+idu4
				)
			)*(is*it*iu);
}

inline double gghzab(const FO_NNLO_Kinematic_Parameter& p){
	double M2(p.M2);
	double qT2(p.qT2);
	double s(p.s);
	double t(p.t);
	double dt(p.dt);
	double za(p.za);
	double iza(p.iza);


	return	pow_2( (1.-za*(1.-za))*iza )*gg_Hg_LO(s*za,t,-dt*qT2*s,M2);
}


//matrixelements======================================================

/*
 * 1/2 * PeTeR gg_delta = gg_2s(delta(x2))
 * muOK
 */
inline double GG_2s_delta(const FO_NNLO_Kinematic_Parameter& p, const FO_Scale_Parameter &ps){

	double M2(p.M2);
	double is(p.is);
	double it(p.it);
	double iu(p.iu);
	double fs(p.fs);
	double ft(p.ft);
	double f1t(p.f1t);
	double f2t(p.f2t);
	double fu(p.fu);
	double f1u(p.f1u);
	double f2u(p.f2u);
	double fA(p.fA);
	double fmuF(ps.fmuF);
	double fmuR(ps.fmuR);
	int 	nf(ps.nf);

#ifndef PLUS_X2_X2
	double Ggg_LO = C_A*gg_Hg_LO(p);

	double Delta	= 0.;//(5*C_A-3*C_F) = C_t^{(1)} -> Hardfunc_FO;
	double beta0	= 11/6.*C_A-1/3.*nf;
	double delta	= 1.5*beta0*(2*fmuR-ft-fu)+(67/18.*C_A-5/9.*nf);
	double U		= 2*f1t+2*f1u-f2t-f2u-2*ft*fu+4*zeta_2+(ft*ft)/2.+(fu*fu)/2.;
	double NCminusnf= C_A-nf;

//	return (C_A*M2*NCminusnf*(1+M2*(is+it+iu)))/3.		//from gg_2s(\delta(x2))
//			+Ggg_LO*(
//					delta								//from gg_2s(\delta(x2))
//					+Delta								//from gg_2s(\delta(x2))
//					+0.5*beta0*(-2*fA-4*fmuF+3*(ft+fu)) //from gg_2s(P_gg(delta))
//					+0.5*C_A*(
//							+2*U						//from gg_2s(\delta(x2))
//							-(ft*(2*fs+ft))				//from gg_2s((za/zb)_+)
//							-2*(fs-2*ft)*fu				//	""
//							+4*fmuF*(ft+fu)				//	""
//							-4*fA*(2*fmuF-fs+ft+fu)		//	""
//							+6*(fA*fA)					//	""
//							-fu*fu						//	""
//						)
//			);

	double M4		= (M2*M2);
	double iM2		= (p.iM2);

	double CAmNF	= (C_A-nf);

	double Ggg0 	= C_A*gg_Hg_LO(p);
	double GggCAmNF0= C_A*M4*(iM2 + is + it + iu)/3.;

	double fx2p0	= ft+fu-fs;

	return(
			 (delta + C_A*U + Delta)*Ggg0 + CAmNF*GggCAmNF0
			+beta0*(-2.*(fmuF-fA) + 1.5*( ft + fu -2.*fA ))*Ggg0
			+C_A*(
					 (2.*fmuF+fx2p0-3.*fA)*( ft + fu -2.*fA )
					-(3.)*(0.5*pow_2(ft-fA) + 0.5*pow_2(fu-fA))
				)*Ggg0
			);
#else

	double M4		= (M2*M2);
	double iM2		= (p.iM2);

	double CAmNF	= (C_A-nf);
	double beta0	= (11/6.*C_A-1/3.*nf);
	double Delta	= (5*C_A-3*C_F);
	double delta	= (1.5*beta0*(2*fmuR-ft-fu)+(67/18.*C_A-5/9.*nf));
	double U		= (2*f1t+2*f1u-f2t-f2u-2*ft*fu+4*zeta_2+(ft*ft)/2.+(fu*fu)/2.);

	double Ggg0 	= C_A*gg_Hg_LO(p);
	double GggCAmNF0= C_A*M4*(iM2 + is + it + iu)/3.;

	double fmuFX2	= (p.LmuX2);
	double   LtX2	= (p.LtX2);
	double   LuX2	= (p.LuX2);
	double fx2pX2	= (p.Lx2pX2);

	double Ltumu	= (p.Ltumu);

	return(
			 (delta + C_A*U + Delta)*Ggg0 + CAmNF*GggCAmNF0
			+0.5*beta0*(2.*Ltumu + LtX2 + LuX2)*Ggg0
			+2.*C_A*(
					fmuFX2*(LtX2 + LuX2)
					-0.5*pow_2(LtX2)
					-0.5*pow_2(LuX2)
				)*Ggg0
			+C_A*(
					fx2pX2*(LtX2 + LuX2)
					-0.5*pow_2(LtX2)
					-0.5*pow_2(LuX2)
				)*Ggg0
			);
#endif
}

/*
 * 1/2 * PeTeR gg_plus0 = gg_2s((x2)_+)
 * muOK
 */
inline double GG_2s_plus0(const FO_NNLO_Kinematic_Parameter& p,const FO_NNLO_Kinematic_Parameter& p_tu, const FO_Scale_Parameter &ps)
{
#ifndef PLUS_X2_X2
	double M2(p.M2);
	double M4(M2*M2);
	double M8(M4*M4);
	double s2(p.s2);
	double s4(s2*s2);
	double is(p.is);
	double t(p.t);
	double it(p.it);
	double it2(p.it2);
	double it4(it2*it2);
	double u(p.u);
	double iu(p.iu);
	double iu2(p.iu2);
	double iu4(iu2*iu2);
	double dt(p.dt);
	double idt(p.idt);
	double idt2(idt*idt);
	double idt4(idt2*idt2);
	double du(p.du);
	double idu(p.idu);
	double idu2(idu*idu);
	double idu4(idu2*idu2);
	double fst(p.fst);
	double fsu(p.fsu);
	double ft(p.ft);
	double fu(p.fu);

	double fmuF(ps.fmuF);
	int 	nf(ps.nf);


	double beta0=11/6.*C_A-1/3.*nf;
	double gghelp1=gg_help1(p);
	double gghelpt=gg_help2(p);
	double gghelpu=gg_help2(p_tu);
//	cout	//<<"\t"<<"s="<<s	<<"\t"<<"t="<<t	<<"\t"<<"u="<<u	<<endl
//			<<"\t"<<"M2="<<M2	<<"\t"<<"mx2="<<mx2	<<endl
//			<<"\t"<<"dt="<<dt	<<"\t"<<"du="<<du	<<endl
//			<<"\t"<<"fmuF="<<fmuF	<<endl
//			<<"\t"<<"ft="<<ft	<<"\t"<<"fu="<<fu	<<endl
//			<<"\t"<<"fst="<<fst	<<"\t"<<"fsu="<<fsu	<<endl
//			<<"\t"<<"s4="<<s4	<<"\t"<<"t4="<<t4	<<"\t"<<"u4="<<u4	<<endl
//			<<"\t"<<"dt4="<<dt4		<<"\t"<<"du4="<<du4	<<endl
//			<<"\t"<<"M8="<<M8	<<"\t"<<"mx8="<<mx8	<<endl
//			<<"\t"<<"beta0="<<beta0	<<endl
//			<<"\t"<<"gghelp1="<<gghelp1	<<endl
//			<<"\t"<<"gghelpt="<<gghelpt	<<endl
//			<<"\t"<<"gghelpu="<<gghelpu	<<endl
//			;
	double qT2		= p.qT2;
	double s		= p.s;
	double za		= p.za;
	double zb		= p.zb;

	double Ggg		= C_A*gg_Hg_LO(p);
	double Gggh1	= C_A*ggh1(p);
	double Ggghbeta = C_A*gghbeta(p);
	double Ggghza	= C_A*gghzab(p);
	double Ggghzb	= C_A*gghzab(p_tu);

//	cout	<<"s="<<s<<"\tza="<<za<<"\tt="<<t<<"\tqT2="<<qT2<<"\tM2="<<M2	<<endl
//			<<gg_Hg_LO(-s*za,t,qT2*s*dt,M2)<<"\t"<<Ggghza-C_A*gghelpt*((1+dt*t*(1+dt*t))*(1+dt*t*(1+dt*t)))*(idt4*it4)	<<endl;

	double fx2p		= 0.5*(-fst-fsu+2*(ft+fu));

	return (
//				(-beta0*C_A*(M8+s4+dt*du*(idt4+idu4)*t*u))*(is*it*iu)
				-beta0*Ggghbeta
				-C_A*fx2p*(Ggg + Gggh1)
				-2.*C_A*fmuF*(Ggghza+Ggghzb)
//				+0.5*C_A*C_A*(
////						(fst+fsu-2*(ft+fu))*gghelp1
//						+(4*fmuF*gghelpt*((1+dt*t*(1+dt*t))*(1+dt*t*(1+dt*t))))*(idt4*it4)
//						+(4*fmuF*gghelpu*((1+du*u*(1+du*u))*(1+du*u*(1+du*u))))*(idu4*iu4)
//				)
			);
//	double qT2		= p.qT2;
//	double s		= p.s;
//	double za		= p.za;
//	double zb		= p.zb;
//	double fx2p		= 0.5*(-fst-fsu+2*(ft+fu));
//
//	double Ggg		= C_A*gg_Hg_LO(p);
//	double Gggh1	= C_A*ggh1(p);
//	double Ggghbeta = C_A*gghbeta(p);
//	double Ggghza	= 2.*C_A*pow_2( (1.+za*(1.+za))/za )*gg_Hg_LO(-s*za,t,qT2*s*dt,M2);
//	double Ggghzb	= 2.*C_A*pow_2( (1.+zb*(1.+zb))/zb )*gg_Hg_LO(-s*zb,u,qT2*s*du,M2);
//
////	double beta0	= (11/6.*C_A-1/3.*nf);
//
//	return	-beta0*Ggghbeta
//			-C_A*fx2p*(Ggg + Gggh1)
//			-C_A*fmuF*(Ggghza + Ggghzb);

#else
	double Ggg		= C_A*gg_Hg_LO(p);
	double Gggh1	= C_A*ggh1(p);
	double Ggghbeta = C_A*gghbeta(p);
	double Ggghza	= C_A*gghzab(p);
	double Ggghzb	= C_A*gghzab(p_tu);

	double beta0	= (11/6.*C_A-1/3.*nf);

	double fmuFX2	= (p.LmuX2);
	double fx2pX2	= (p.Lx2pX2);

	return	-beta0*Ggghbeta
			-C_A*fx2pX2*(Ggg + Gggh1)
			-2.*C_A*fmuFX2*(Ggghza + Ggghzb);
#endif
}


/*
 * 1/2 * PeTeR gg_plus1 = gg_2s(log(x2)*(x2)_+)
 */
inline double GG_2s_plus1(const FO_NNLO_Kinematic_Parameter& p,const FO_NNLO_Kinematic_Parameter& p_tu)
{
#ifndef PLUS_X2_X2
	double t(p.t);
	double t2(p.t2);
	double it2(p.it2);
	double it4(it2*it2);
	double u(p.u);
	double u2(p.u2);
	double iu2(p.iu2);
	double iu4(iu2*iu2);
	double dt(p.dt);
	double dt2(dt*dt);
	double idt(p.idt);
	double idt2(idt*idt);
	double idt4(idt2*idt2);
	double du(p.du);
	double du2(du*du);
	double idu(p.idu);
	double idu2(idu*idu);
	double idu4(idu2*idu2);

	double gghelp1=gg_help1(p);
	double gghelpt=gg_help2(p);
	double gghelpu=gg_help2(p_tu);

	double M2		= p.M2;
	double qT2		= qT2;
	double s		= p.s;
	double za		= p.za;
	double zb		= p.zb;

	double Ggghza	= C_A*pow_2( (1.+za*(1.+za))/za )*gg_Hg_LO(-s*za,t,qT2*s*dt,M2);
	double Ggghzb	= C_A*pow_2( (1.+zb*(1.+zb))/zb )*gg_Hg_LO(-s*zb,u,qT2*s*du,M2);

	return C_A*C_A*(
			gghelp1
			-2.*(
					 (gghelpt*((1+dt*t+dt2*(t2))*(1+dt*t+dt2*(t2))))*(idt4*it4)
					+(gghelpu*((1+du*u+du2*(u2))*(1+du*u+du2*(u2))))*(idu4*iu4)
				)
			);
#else

	double Ggg		= C_A*gg_Hg_LO(p);
	double Gggh1	= C_A*ggh1(p);
	double Ggghza	= C_A*gghzab(p);
	double Ggghzb	= C_A*gghzab(p_tu);

	return	 C_A*(Ggg + Gggh1)
			+2.*C_A*(Ggghza + Ggghzb);
#endif
}


/*
 * 1/2 * PeTeR gg_real_regul = gg_2s(finite)
 * muOK
 */
inline double GG_2s_regul(const FO_NNLO_Kinematic_Parameter& p, const FO_Scale_Parameter &ps)
{
	double M2(p.M2);
	double M4(M2*M2);
	double M8(M4*M4);
	double x2(p.x2);
	double x4(p.x4);
	double x8(x4*x4);
	double s(p.s);
	double s2(p.s2);
	double s4(s2*s2);
	double is(p.is);
	double t(p.t);
	double t2(p.t2);
	double t4(p.t4);
	double it(p.it);
	double it2(p.it2);
	double u(p.u);
	double u2(p.u2);
	double u4(p.u4);
	double iu(p.iu);
	double iu2(p.iu2);
	double dt(p.dt);
	double dt2(dt*dt);
	double idt(p.idt);
	double idt2(idt*idt);
	double idt4(idt2*idt2);
	double du(p.du);
	double du2(du*du);
	double idu(p.idu);
	double idu2(idu*idu);
	double idu4(idu2*idu2);
	double ftu(p.ftu);
	double fx2(p.fx2);
	double M2t(p.M2t);
	double iM2t(p.iM2t);
	double M2u(p.M2u);
	double iM2u(p.iM2u);
	double idtu2(p.idtu2);
	double dtu(p.dtu);

	double fmuF(ps.fmuF);
	int 	nf(ps.nf);

	//regulpartofGgg2s
	return(
			(ftu*(C_A*C_A)*(
				((idt4+idu4+M8+x8+s4)*(-(M2*x2)+2*x2*s+t*u))/(x2*(-(M2*x2)+x2*s+t*u))
				+(2*s*(M4)*(t4+u4+M2t*M2t*M2t*M2t+(M2u*M2u*M2u*M2u)))*(iM2t*it*iM2u*iu)
				)
			)*(is*dtu)
			+C_F*(
					 (dt2*(-2*dt*nf*t*(1+dt*t)-(fmuF-fx2)*nf*(1+2*dt*t+2*(dt2)*(t2)))*(s2*t2+idtu2))*it2
					+(du2*(-2*du*nf*u*(1+du*u)-(fmuF-fx2)*nf*(1+2*du*u+2*(du2)*(u2)))*(s2*u2+idtu2))*iu2
				)
			);

}
//
///*
// * 1/2 * PeTeR gg_real = gg_2R_ns + gg_2s(finite)
// */
//inline double GG_finite(const FO_NLO_Kinematic_Parameter& p,const FO_NLO_Kinematic_Parameter& p_tu,int nf){
//#ifndef ONLY_NON_SINGULAR
//	return	GG_2s_regul(p,nf) + GG_2R_ns(p,p_tu,nf);
//#else
//	return	gg_2R_ns(p,p_tu,nf);
//#endif
//}


/*
 * 1/2 * PeTeR qg_delta = qg_2s(delta(x2))
 * muOK
 */
inline double qG_2s_delta(const FO_NNLO_Kinematic_Parameter& p, const FO_Scale_Parameter &ps)
{
	double fs(p.fs);
	double ft(p.ft);
	double f1t(p.f1t);
	double f2t(p.f2t);
	double fu(p.fu);
	double f1u(p.f1u);
	double f2u(p.f2u);
	double fA(p.fA);
	double M2u(p.M2u);

	double fmuF(ps.fmuF);
	double fmuR(ps.fmuR);
	int 	nf(ps.nf);

	double Gqg_LO = C_F*qg_Hq_LO(p);

	double Delta	= 0.;//(5*C_A-3*C_F) = C_t^{(1)} -> Hardfunc_FO;
	double beta0	= 11/6.*C_A-1/3.*nf;
	double V1		= 2*f1u+2*fs*ft-2*(fs+ft)*fu+6*zeta_2+fu*fu;
	double V2		= -7./2.+2*f1t-f2t-f2u-fs*ft+fs*fu-2*zeta_2;
	double V3		= beta0*(3*fmuR-ft-2*fu)+(67*C_A-10*nf)/9.;
	double NCminusC_F=C_A-C_F;


//	cout	<<"\t"<<"s="<<p.s	<<"\t"<<"t="<<t	<<"\t"<<"u="<<u	<<endl
//			<<"\t"<<"M2="<<M2	<<endl
//			<<"\t"<<"Gqg="<<Gqg_LO	<<endl
//			<<"\t"<<"fmuR="<<fmuR	<<"\t"<<"fmuF="<<fmuF	<<endl
//			<<"\t"<<"fs="<<fs	<<"\t"<<"ft="<<ft	<<"\t"<<"fu="<<fu	<<endl
//			<<"\t"<<"f1t="<<f1t	<<"\t"<<"f2t="<<f2t	<<endl
//			<<"\t"<<"f1u="<<f1u	<<"\t"<<"f2u="<<f2u	<<endl
//			<<"\t"<<"fA="<<fA	<<endl
//			;

	return 	C_F*NCminusC_F*M2u
			+Gqg_LO*(
					Delta
					+NCminusC_F
					+(
							-(beta0*fmuF)
							+beta0*ft
							+C_A*(2*(fA-ft)*(fA-fmuF+fs-fu)+V1)
							+V3
					)
					+0.5*C_F*(
							fA*(-3+2*fA-4*fmuF)
							-3*fmuF
							+6*fu
							+4*fmuF*fu
							+2*V2
							-2*(fu*fu)
						)
			);
}


/*
 * 1/2 * PeTeR qg_plus0 = qg_2s(log(x2)(x2)_+)
 * muOK
 */
inline double qG_2s_plus0(const FO_NNLO_Kinematic_Parameter& p, const FO_Scale_Parameter &ps)
{
	double M2(p.M2);
	double M4(M2*M2);
	double x2(p.x2);
	double x4(p.x4);
	double s(p.s);
	double s2(p.s2);
	double is(p.is);
	double t(p.t);
	double t2(p.t2);
	double u(p.u);
	double u2(p.u2);
	double iu(p.iu);
	double iu2(p.iu2);
	double dt(p.dt);
	double dt2(dt*dt);
	double idt(p.idt);
	double idt2(idt*idt);
	double du(p.du);
	double du2(du*du);
	double idu(p.idu);
	double idu2(idu*idu);
	double fs(p.fs);
	double fst(p.fst);
	double fsu(p.fsu);
	double ft(p.ft);
	double fu(p.fu);
	double dtu(p.dtu);

	double fmuF(ps.fmuF);
	int		nf(ps.nf);
//	cout	<<"\t"<<"s:"<<s	<<"\t"<<"t:"<<t	<<"\t"<<"u:"<<u	<<endl
//			<<"\t"<<"M2:"<<M2	<<"\t"<<"mx2:"<<x2	<<endl
//			<<"\t"<<"dt:"<<dt	<<"\t"<<"du:"<<du	<<endl
//			<<"\t"<<"fmuF:"<<fmuF	<<endl
//			<<"\t"<<"fs:"<<fs	<<"\t"<<"ft:"<<ft	<<"\t"<<"fu:"<<fu	<<endl
//			<<"\t"<<"fst:"<<fst	<<"\t"<<"fsu:"<<fsu	<<endl
//			;
	return	0.5*C_F*C_F*(
					3*u*(s2+t2)
					-2*du*fmuF*(-2*M2*x2*t*u+M4*(x4)+(s2+t2)*(u2))
					-2*fmuF*(du2*du)*(u2)*(-2*M2*x2*t*u+M4*(x4)+(s2+t2)*(u2))
				)*iu2
			+0.5*C_A*C_F*(
					-dt*(fs-fst+2*ft)*(
							s*u*(du2*du)
							+x2*u*(dt2*dt)*(1+du2*(x4))
							+dt2*(du2)*(x4*x2+(-1+du*u)*(s2*s)+x2*(t2)-s*(t2))
						)*(is*iu*idu2)
					+dt*(fs+fsu-2*fu)*(
							s*u*(du2*du)
							+x2*u*(dt2*dt)*(1+du2*(x4))
							+dt2*(du2)*(x4*x2+(-1+du*u)*(s2*s)+x2*(t2)-s*(t2))
						)*(is*iu*idu2)
					-(2*(2*fmuF-fs+fst-2*ft)*(1+dt2*(s2))*((1+dt*t+dt2*(t2))*(1+dt*t+dt2*(t2))))*dtu
					-(2*(fs-fst+2*ft)*(1+dt2*(s2))*((1+dt*t+dt2*(t2))*(1+dt*t+dt2*(t2))))*dtu
				)*(idt2*idt)
				;
}


/*
 * 1/2 * PeTeR qg_plus1 = qg_2s(log(x2)(x2)_+)
 */
inline double qG_2s_plus1(const FO_NNLO_Kinematic_Parameter& p)
{
	double x2(p.x2);
	double x4(p.x4);
	double s(p.s);
	double s2(p.s2);
	double is(p.is);
	double is2(p.is2);
	double t(p.t);
	double t2(p.t2);
	double u(p.u);
	double u2(p.u2);
	double iu(p.iu);
	double iu2(p.iu2);
	double dt(p.dt);
	double dt2(dt*dt);
	double idt(p.idt);
	double idt2(idt*idt);
	double du(p.du);
	double du2(du*du);
	double idu(p.idu);
	double idu2(idu*idu);
	double dtu(p.dtu);
	double idtu2(p.idtu2);

	return	(du*(C_F*C_F)*(s2)*(1+du2*(u2))*(u2+idtu2*(is2)))*iu2
			+(C_A*C_F*(
					(dt*(
							s*u*(du2*du)
							+x2*u*(dt2*dt)*(1+du2*(x4))
							+dt2*(du2)*(x4*x2+(-1+du*u)*(s2*s)+x2*(t2)-s*(t2))
						))*(iu*idu2)
					+(2*s*(1+dt2*(s2))*((1+dt*t+dt2*(t2))*(1+dt*t+dt2*(t2))))*dtu
				))*(is*idt2*idt)
				;
}



inline double qG_2s_regul(const FO_NNLO_Kinematic_Parameter& p, const FO_Scale_Parameter &ps)
{
	double M2(p.M2);
	double M4(M2*M2);
	double M8(M4*M4);
	double x2(p.x2);
	double x4(p.x4);
	double s(p.s);
	double s2(p.s2);
	double s4(s2*s2);
	double is(p.is);
	double is2(p.is2);
	double t(p.t);
	double t2(p.t2);
	double t4(p.t4);
	double it2(p.it2);
	double it4(it2*it2);
	double u(p.u);
	double u2(p.u2);
	double u4(p.u4);
	double iu(p.iu);
	double iu2(p.iu2);
	double iu4(iu2*iu2);
	double dt(p.dt);
	double dt2(dt*dt);
	double idt(p.idt);
	double du(p.du);
	double du2(du*du);
	double du4(du2*du2);
	double idu(p.idu);
	double idu2(idu*idu);
	double ftu(p.ftu);
	double fx2(p.fx2);
	double M2t(p.M2t);
	double iM2u(p.iM2u);
	double dtu(p.dtu);
	double idtu2(p.idtu2);

	double fmuF(ps.fmuF);

	//regulpartofGgq2s
	return	(C_F*C_F)*(
					-((s4*(-1-du*u)*(du2)*(u2+idtu2*(is2)))*iu2)
					-(s*(2*dt*t*(1+dt*t)+(fmuF-fx2)*(1+2*dt*t+2*(dt2)*(t2)))*(t4+dt2*t2*idtu2))*(idt*it4)
				)*(is2)
			+C_A*C_F*(
					ftu*(
							(-(M2*x2)+2*x2*s+t*u)*(
									s*(du2*du)
									+dt2*dt*(x2+du2*(x4*x2))
									+dt2*(du2*du)*(s2*s)
								)/(x2*(-(M2*x2)+x2*s+t*u)*(dt2*dt)*(du2*du))
							-(2*s*(M4)*(M2t*M2t+t2))*(iM2u*iu)
						)
					-((-(du2*(u2))+(fmuF-fx2)*(2+2*du*u+du2*(u2)))*(
							M8
							+u4
							+du4*s4*u4
							+du4*pow_2(idtu2)
						))*(iu4*idu2*idu)
				)*(is*dtu)
				;
}


///*
// * 1/2 * PeTeR qg_real = qG_2R_ns + qg_2s(finite)
// * muOK
// */
//inline double qG_finite(const FO_NLO_Kinematic_Parameter& p, const FO_NLO_Kinematic_Parameter& p_tu,int nf)
//{
//#ifndef ONLY_NON_SINGULAR
//	return qG_2s_regul(p,nf) + qG_2R_ns(p,p_tu);
//#else
//	return qG_2R_ns(p,p_tu);
//#endif
//}
//
//inline double Gq_finite(const FO_NLO_Kinematic_Parameter& p, const FO_NLO_Kinematic_Parameter& p_tu,int nf){
//	return qG_finite(p_tu,p,nf);
//}


/*
 * 1/2 * PeTeR qqbar_delta = qbq_2s(delta(x2))
 * muOK
 */
inline double qbq_2s_delta(const FO_NNLO_Kinematic_Parameter& p, const FO_Scale_Parameter &ps)
{
	double M2(p.M2);
	double s(p.s);
	double fs(p.fs);
	double ft(p.ft);
	double f1t(p.f1t);
	double f2t(p.f2t);
	double fu(p.fu);
	double f1u(p.f1u);
	double f2u(p.f2u);
	double fA(p.fA);


	double fmuF(ps.fmuF);
	double fmuR(ps.fmuR);
	int nf(ps.nf);

//	double Gqq_LO = 2*C_F*C_F*qqbar_Hg_LO(p);

	double Delta	= 0.;//(5*C_A-3*C_F) = C_t^{(1)} -> Hardfunc_FO;
	double beta0	= 11/6.*C_A-1/3.*nf;
	double W1		= (-48*zeta_2-3*(2*f2t+2*f2u-4*ft*fu+ft*ft+fu*fu))/6.;
	double W2		= -7+2*f1t+2*f1u+3*fs-(3*ft)/2.-(3*fu)/2.-4*ft*fu+12*zeta_2+ft*ft+fu*fu;
	double W3		= (67*C_A)/6.+(beta0*(6*fmuR-4*fs-ft-fu))/2.-(5*nf)/3.;
	double NCminusC_F= C_A-C_F;

//	return	Gqq_LO*(
//					Delta + C_A*W1 + C_F*W2 + W3 + NCminusC_F
//				)
//			+NCminusC_F*2*(C_F*C_F)*(s-M2)
//			+0.5*Gqq_LO*(
////					2*Delta//-> first line
//					-beta0*(2*fA-ft-fu)
////					+2*(NCminusC_F+W3)//-> first line
//					+C_F*(
//							3*(ft+fu)
//							-8*fA*(fmuF-fs+ft+fu)
//							+fmuF*(-6+4*ft+4*fu)
//							-4*(-2*ft*fu+fs*(ft+fu))
////							+2*W2//-> first line
//							+8*(fA*fA)
//						)
//					-C_A*(
//							4*fA*(fs-ft-fu)
//							+4*ft*fu
//							-2*fs*(ft+fu)
////							-2*W1//-> first line
//							+2*(fA*fA)
//							+ft*ft
//							+fu*fu
//						)
//				);

	double it(p.it);
	double t(p.t);
	double t2(p.t2);
	double iu(p.iu);
	double u(p.u);
	double u2(p.u2);
	double qT2(p.qT2);

	double Gqq0		= 2*C_F*C_F*qqbar_Hg_LO(p);
	double Gqq_hzab0= 2.*Gqq0;
//	double Gqq_hza0	= 2*C_F*C_F*(2.)*qqbar_Hg_LO(s,t,s*it*qT2);
//	double Gqq_hzb0	= 2*C_F*C_F*(2.)*qqbar_Hg_LO(s,u,s*iu*qT2);
#ifndef PLUS_X2_X2
	double fx2p0	= ft+fu-fs;
	return	 (Delta + C_A*W1 + C_F*W2 + W3)*Gqq0
			+NCminusC_F*(Gqq0 + 2.*C_F*C_F*(s-M2))
			//Plus0:
			-beta0*(2.*fA-ft-fu)*0.5*Gqq0
			-C_F*(
					 (fmuF-ft)*((fA-ft)+0.75)
					+(fmuF-fu)*((fA-fu)+0.75)
				)*(Gqq_hzab0)
			-(2*C_F-C_A)*(
					 (fx2p0-ft)*(fA-ft)
					+(fx2p0-fu)*(fA-fu)
				)*(Gqq0)
			//Plus1:
			+C_F*(
					 0.5*pow_2(fA-ft)
					+0.5*pow_2(fA-fu)
				)*(Gqq_hzab0)
			+(2*C_F-C_A)*(
					 0.5*pow_2(fA-ft)
					+0.5*pow_2(fA-fu)
				)*(Gqq0);
#else
//	double fX2(p.fX2);
//	double ftX2(ft-fX2);
//	double fuX2(fu-fX2);
//	double fmuFX2(fmuF-fX2);
//	double fx2pX2(ft+fu-fs-fX2);

	double LtX2(p.LtX2);
	double LuX2(p.LuX2);
	double LmuFX2(p.LmuX2);
	double Lx2pX2(p.Lx2pX2);
	double LtumuF(p.Ltumu);

	return	 (Delta + C_A*W1 + C_F*W2 + W3)*Gqq0
			+NCminusC_F*(Gqq0 + 2.*C_F*C_F*(s-M2))
			//Plus0:
			+beta0*0.5*( LtX2 + LuX2 )*Gqq0
			+C_F*1.5*LtumuF*(Gqq0)	//APSF[0,{q,q}]
			+C_F*(
					-0.5*pow_2(LtX2)
					-0.5*pow_2(LuX2)
					+LmuFX2*( LtX2 + LuX2 )
				)*(Gqq_hzab0)
			+(2*C_F-C_A)*(
					-0.5*pow_2(LtX2)
					-0.5*pow_2(LuX2)
					+Lx2pX2*( LtX2 + LuX2 )
				)*(Gqq0);
#endif
}


/*
 * 1/2 * PeTeR qqbar_plus0 = qbq_2s((x2)_+)
 * muOK
 */
inline double qbq_2s_plus0(const FO_NNLO_Kinematic_Parameter& p, const FO_Scale_Parameter &ps)
{
	double M2(p.M2);
	double x2(p.x2);
	double s(p.s);
	double t(p.t);
	double it2(p.it2);
	double u(p.u);
	double iu2(p.iu2);
	double dt(p.dt);
	double dt2(dt*dt);
	double idt(p.idt);
	double idt2(idt*idt);
	double du(p.du);
	double du2(du*du);
	double idu(p.idu);
	double idu2(idu*idu);
	double fst(p.fst);
	double fsu(p.fsu);
	double ft(p.ft);
	double fu(p.fu);
	double idtu2(p.idtu2);

	double fmuF(ps.fmuF);
	int 	nf(ps.nf);

	double beta0	= 11/6.*C_A-1/3.*nf;
//	double Gqq		= 2*C_F*C_F*qqbar_Hg_LO(p);
//	double Gqqhelp	= 2*C_F*C_F*(idt2*idu2)*qqbar_Hg_LO(s,dt,du);
//	double fx2p		= 0.5*(-fst-fsu+2*(ft+fu));
//
////	cout	<<"\t"<<"s="<<s	<<"\t"<<"t="<<t	<<"\t"<<"u="<<u	<<endl
////			<<"\t"<<"M2="<<M2	<<"\t"<<"mx2="<<x2	<<endl
////			<<"\t"<<"dt="<<dt	<<"\t"<<"du="<<du	<<endl
////			<<"\t"<<"fmuF="<<fmuF	<<endl
////			<<"\t"<<"ft="<<ft	<<"\t"<<"fu="<<fu	<<endl
////			<<"\t"<<"fst="<<fst	<<"\t"<<"fsu="<<fsu	<<endl
////			<<"\t"<<"dt4="<<dt2*dt2	<<"\t"<<"du4="<<du2*du2	<<endl
////			<<"\t"<<"beta0="<<beta0	<<endl
////			<<"\t"<<"Gqq="<<Gqq	<<"\t"<<"Gqqhelp="<<Gqqhelp	<<endl
////			;
//
//
//	return -beta0*Gqq
//			-(2.*C_F-C_A)*fx2p*(Gqq+Gqqhelp)
////fx2p			+0.5*C_A*(
////fx2p					(-fst-fsu+2*(ft+fu))*Gqq
////fx2p					-(fst+fsu-2*(ft+fu))*Gqqhelp
////				)
//			+C_F*(
//					Gqq*(
////fx2p							fst
////fx2p							+fsu
////fx2p							-2*(ft+fu)
//							+fmuF*(-2+(M2*x2*(-(M2*x2)+2*t*u))*(it2*iu2))
//						)
//					-Gqqhelp*(
////fx2p							-fst
////fx2p							-fsu
////	fx2p						+2*(ft+fu)
//							+fmuF*(1+dt2*du2*idtu2)
//						)
//				);


	double it(p.it);
	double t2(p.t2);
	double iu(p.iu);
	double u2(p.u2);
	double qT2(p.qT2);

	double Gqq		= 2*C_F*C_F*qqbar_Hg_LO(p);
	double Gqq_h1	= 2*C_F*C_F*qqbar_Hg_LO(s,idt,idu);
	double Gqq_hza	= 2*C_F*C_F*(1.+dt2*t2)*(idt*it)*qqbar_Hg_LO(s*t*dt,t,s*dt*qT2);
	double Gqq_hzb	= 2*C_F*C_F*(1.+du2*u2)*(idu*iu)*qqbar_Hg_LO(s*u*du,u,s*du*qT2);
#ifndef PLUS_X2_X2
	double fx2p		= 0.5*(-fst-fsu+2*(ft+fu));

	return 	-beta0*Gqq
			//Plusab1:
			-C_F*fmuF*( Gqq_hza + Gqq_hzb )
			-(2*C_F-C_A)*fx2p*( Gqq + Gqq_h1 );
#else
//	double fX2(p.fX2);
//	double fmuFX2(fmuF-fX2);
//	double fx2pX2(0.5*(-fst-fsu+2*(ft+fu))-fX2);

	double LmuFX2(p.LmuX2);
	double Lx2pX2(p.Lx2pX2);

	return 	-beta0*Gqq
			//Plusab1:
			-C_F*LmuFX2*( Gqq_hza + Gqq_hzb )
			-(2*C_F-C_A)*Lx2pX2*( Gqq + Gqq_h1 );
#endif
}


/*
 * 1/2 * PeTeR qqbar_plus1 = qbq_2s(log(x2)(x2)_+)
 */
inline double qbq_2s_plus1(const FO_NNLO_Kinematic_Parameter& p)
{
	double qT2(p.qT2);
	double qT4(qT2*qT2);
	double s(p.s);
	double s2(p.s2);
	double it2(p.it2);
	double iu2(p.iu2);
	double dt(p.dt);
	double dt2(dt*dt);
	double idt(p.idt);
	double idt2(idt*idt);
	double du(p.du);
	double du2(du*du);
	double idu(p.idu);
	double idu2(idu*idu);

//	double Gqq		= 2*C_F*C_F*qqbar_Hg_LO(p);
//	double Gqqhelp	= 2*C_F*C_F*(idt2*idu2)*qqbar_Hg_LO(s,dt,du);
//
//	return	C_A*(-Gqq-Gqqhelp)
//			+C_F*(
//					Gqqhelp*(3+dt2*(du2)*(QT4)*(s2))
//					+Gqq*(3+(QT4)*(s2)*(it2*iu2))
//				);

	double it(p.it);
	double t(p.t);
	double t2(p.t2);
	double iu(p.iu);
	double u(p.u);
	double u2(p.u2);

	double Gqq		= 2*C_F*C_F*qqbar_Hg_LO(p);
	double Gqq_h1	= 2*C_F*C_F*qqbar_Hg_LO(s,idt,idu);
	double Gqq_hza	= 2*C_F*C_F*(1.+dt2*t2)*(idt*it)*qqbar_Hg_LO(s*t*dt,t,s*dt*qT2);
	double Gqq_hzb	= 2*C_F*C_F*(1.+du2*u2)*(idu*iu)*qqbar_Hg_LO(s*u*du,u,s*du*qT2);

	return	 C_F*(Gqq_hza+Gqq_hzb)
			+(2*C_F-C_A)*(Gqq+Gqq_h1);
}


/*
 * 1/2 * (PeTeR qqbar_real_regul )  -qp_2s =  qbq_2s(finite)
 */
inline double qbq_2s_regul(const FO_NNLO_Kinematic_Parameter& p){
	double is(p.is);
	double t(p.t);
	double t2(p.t2);
	double it2(p.it2);
	double u(p.u);
	double u2(p.u2);
	double iu2(p.iu2);
	double dt(p.dt);
	double dt2(dt*dt);
	double idt(p.idt);
	double du(p.du);
	double du2(du*du);
	double idu(p.idu);
	double idtu2(p.idtu2);

	return	C_F*C_F*C_F*(
							 (2*(1+dt*t)*(t2+dt2*idtu2))*(idt*it2)
							+(2*(1+du*u)*(u2+du2*idtu2))*(idu*iu2)
						)*is;
}


/*
 * 1/2 * qqbar_different_2s = qp_2s = GS(3.29)
 * muOK
 */
inline double qp_2s_regul(const FO_NNLO_Kinematic_Parameter& p, const FO_Scale_Parameter &ps)
{
	double x2(p.x2);
	double s(p.s);
	double s2(p.s2);
	double t(p.t);
	double t2(p.t2);
	double u(p.u);
	double u2(p.u2);
	double dt(p.dt);
	double dt2(dt*dt);
	double idt(p.idt);
	double du(p.du);
	double du2(du*du);
	double idu(p.idu);
	double idu2(idu*idu);
	double fx2(p.fx2);
	double ftu(p.ftu);
	double dtu(p.dtu);

	double fmuF(ps.fmuF);

	return ((C_F*C_F)*(
					-((1+dt2*(s2))*(-(dt2*(t2))+(fmuF-fx2)*(2+2*dt*t+dt2*(t2))))*(idt*idt)
					-((1+du2*(s2))*(-(du2*(u2))+(fmuF-fx2)*(2+2*du*u+du2*(u2))))*(idu2)
					+2*ftu*((x2-s)*(x2-s)+(-2*x2+t+u)*(-2*x2+t+u))
				)
			)*dtu;
}


///*
// * 1/2 * (PeTeR qqbar_real )  - qp_2R_ns  -qp_2s = qbq_2R_ns + qbq_2s(finite)
// */
//inline double qbq_finite(const FO_NLO_Kinematic_Parameter& p, const FO_NLO_Kinematic_Parameter& p_tu,int nf){
//#ifndef ONLY_NON_SINGULAR
//	return	qbq_2s_regul(p) +qbq_2R_ns(p,p_tu,nf);
//#else
//	return	qbq_2R_ns(p,p_tu,nf);
//#endif
//}
//
///*
// * 1/2 * PeTeR qqbar_different_real = qp_2s + qp_2R_ns
// */
//inline double qp_finite(const FO_NLO_Kinematic_Parameter& p){
//#ifndef ONLY_NON_SINGULAR
//	return qp_2s(p) + qp_2R_ns(p);
//#else
//	return qp_2R_ns(p);
//#endif
//}



/*
 * 1/2 * (PeTeR qqbar_same_real )  - qp_finite = qq_2R_ns
 */
inline double qq_2R_ns(const FO_NNLO_Kinematic_Parameter& p)
{
	double iQT2(p.iqT2);
	double ix2(p.ix2);
	double is2(p.is2);

	double e4		= E4(p);

	//factor2infrontofcoefficientsA0,..,E3becauseweareusingalphas/4Pi
	return	(is2*iQT2*ix2)*(C_F*C_F/C_A*e4);
}


}  // namespace NS_Higgs

}  // namespace NS_FO

}  // namespace CuTe

#endif


