#include "FO_Lumis.h"
#include "lhafunc.h"
#include "charges.h"

//using CuTe::NS_FO::ENUM_LUMI_S;
//using CuTe::NS_FO::ENUM_LUMI_V;
using CuTe::NS_FO::Lumi;
using CuTe::NS_FO::Lumi_S;
using CuTe::NS_FO::Lumi_V;



std::ostream& CuTe::NS_FO::operator<<(std::ostream& os, const Lumi &lumi){
	os	<<lumi.to_string();
	return os;
}


std::ostream& CuTe::NS_FO::operator<<(std::ostream& os, ENUM_LUMI_S l){
	switch (l) {
	case LS_GG_LO:
		os<<"LS_GG_LO";
		break;
	case LS_Gq_LO:
		os<<"LS_Gq_LO";
		break;
	case LS_qG_LO:
		os<<"LS_qG_LO";
		break;
	case LS_qbq_LO:
		os<<"LS_qbq_LO";
		break;
	case LS_qq_NLO:
		os<<"LS_qq_NLO";
		break;
	case LS_qp_NLO:
		os<<"LS_qp_NLO";
		break;
	default:
		os<<"****WRONG*****";
		break;
	}
	return os;
}


std::ostream& CuTe::NS_FO::operator<<(std::ostream& os, ENUM_LUMI_V l){
	switch (l) {
	case LV_qG_LO:
		os<<"LV_qG_LO";
		break;
	case LV_qG_NLO:
		os<<"LV_qG_NLO";
		break;
	case LV_Gq_LO:
		os<<"LV_Gq_LO";
		break;
	case LV_Gq_NLO:
		os<<"LV_Gq_NLO";
		break;
	case LV_GG_NLO:
		os<<"LV_GG_NLO";
		break;
	case LV_qbq_LO:
		os<<"LV_qbq_LO";
		break;
	case LV_qbq_NLO_ab:
		os<<"LV_qbq_NLO_ab";
		break;
	case LV_qbq_NLO_bb:
		os<<"LV_qbq_NLO_bb";
		break;
	case LV_qbq_NLO_bc:
		os<<"LV_qbq_NLO_bc";
		break;
	case LV_qbq_NLO_cc:
		os<<"LV_qbq_NLO_cc";
		break;
	case LV_qbq_NLO_bd:
		os<<"LV_qbq_NLO_bd";
		break;
	case LV_qbq_NLO_dd:
		os<<"LV_qbq_NLO_dd";
		break;
	case LV_qbq_NLO_LL:
		os<<"LV_qbq_NLO_LL";
		break;
	case LV_qbq_NLO_LR:
		os<<"LV_qbq_NLO_LR";
		break;
	case LV_qq_NLO_aa:
		os<<"LV_qq_NLO_aa";
		break;
	case LV_qq_NLO_bb:
		os<<"LV_qq_NLO_bb";
		break;
	case LV_qq_NLO_ac:
		os<<"LV_qq_NLO_ac";
		break;
	case LV_qq_NLO_bd:
		os<<"LV_qq_NLO_bd";
		break;
	case LV_qq_NLO_ad:
		os<<"LV_qq_NLO_ad";
		break;
	case LV_qq_NLO_LL:
		os<<"LV_qq_NLO_LL";
		break;
	case LV_qq_NLO_LR:
		os<<"LV_qq_NLO_LR";
		break;
	default:
		os<<"****WRONG*****";
		break;
	}
	return os;
}

void Lumi_S::update(const VectorPDF* const Phi[2], ENUM_LUMI_UPDATE l_update){
	const VectorPDF &Phi_0(*(Phi[0]));
	const VectorPDF &Phi_1(*(Phi[1]));
	int nf(*_nf);
	//	cout << "\t-> DY_NLO::Lumi::update()" << endl;
	//	cout << Q << endl;
	//	cout << Phi_0 << endl;
	//	cout << Phi_1 << endl;
	const double g0(Phi_0[Parton_g]);
	const double g1(Phi_1[Parton_g]);

	_V[LS_GG_LO] = g0	*	g1;

	double soq0(0.),soq1(0.),qbq(0.),qq(0.),qp(0.);
	for(int q(0); q<(nf); ++q){

		const double  q0(Phi_0[Parton(1+q)]);
		const double bq0(Phi_0[Parton(-1-q)]);
		const double  q1(Phi_1[Parton(1+q)]);
		const double bq1(Phi_1[Parton(-1-q)]);

		soq0	+= q0+bq0;
		soq1	+= q1+bq1;
		qbq		+= q0*bq1 + bq0*q1;

		if(l_update!=l_update_LO){
			qq		+= q0*q1  + bq0*bq1;

			for(int p(0); p<(nf); ++p){

				const double  p1(Phi_1[Parton(1+p)]);
				const double bp1(Phi_1[Parton(-1-p)]);

				qp	+= q0*p1	+ q0*bp1	+ bq0*p1	+ bq0*bp1;
			}
		}
	}
	_V[LS_Gq_LO]	= g0	*	soq1;
	_V[LS_qG_LO]	= soq0	*	g1;
	_V[LS_qbq_LO]	= qbq;
	_V[LS_qq_NLO]	= qq;
	_V[LS_qp_NLO]	= qp;
}

Lumi_V::Lumi_V()
:Lumi(LV_Number_of_Lumis)
,C(new Charge),nf(-1)
,L(0.),bL(0.),R(0.),bR(0.),D(0.),bD(0.),QQ(0.)
,Dpp(0.),bDpp(0.),QQqp(0.)
,QQ_S(0.),QQ_bS(0.){
//	std::cerr<<"Lumi_V::Lumi_V()"<<std::endl;
}

Lumi_V::Lumi_V(const int *_nf, const PROD prod, const CKM_MATRIX *ckm, const double Theta_W, const Vector<N_FMAX+1,bool>& active_flavour)
:Lumi(LV_Number_of_Lumis)
,C(new Charge(prod,*ckm,Theta_W,active_flavour)),nf(-1)
,L(0.),bL(0.),R(0.),bR(0.),D(0.),bD(0.),QQ(0.)
,Dpp(0.),bDpp(0.),QQqp(0.)
,QQ_S(0.),QQ_bS(0.){
//	std::cerr<<"Lumi_V::Lumi_V(nf,prod,ckm,theta_W,act_flv)"<<std::endl;
	_init(_nf);
}

Lumi_V::Lumi_V(const Lumi_V &LV)
:Lumi(LV)
,C(new Charge(*(LV.C))),nf(LV.nf)
,L(LV.L),bL(LV.bL),R(LV.R),bR(LV.bR),D(LV.D),bD(LV.bD),QQ(LV.QQ)
,Dpp(LV.Dpp),bDpp(LV.bDpp),QQqp(LV.QQqp)
,QQ_S(LV.QQ_S),QQ_bS(LV.QQ_bS){
//	std::cerr<<"Lumi_V::Lumi_V(lumi)"<<std::endl;
}

Lumi_V::~Lumi_V(){
//	std::cerr<<"~Lumi_V::Lumi_V()"<<std::endl;
	delete C;
}

Lumi_V& Lumi_V::operator=(const Lumi_V &LV){
//		NS_Utils::DOUT dout("Lumi_V::operator=","LV");
	Lumi::operator =(LV);

//		Q		= (LV.Q);
	nf		= (LV.nf);
	L		= (LV.L);
	bL		= (LV.bL);
	R		= (LV.R);
	bR		= (LV.bR);
	D		= (LV.D);
	bD		= (LV.bD);
	QQ		= (LV.QQ);
	Dpp		= (LV.Dpp);
	bDpp	= (LV.bDpp);
	QQqp	= (LV.QQqp);
	QQ_S	= (LV.QQ_S);
	QQ_bS	= (LV.QQ_bS);

	*C		= *(LV.C);

	return *this;
}


//void Lumi_V::_init_C(){
//	_exit_C();
//	C	= new Charge(prod,*ckm,Theta_W);
//}
//void Lumi_V::_exit_C(){
//	delete C;	C = NULL;
//}



void Lumi_V::_init(const int *_nf){
	Lumi::_init(_nf);

	nf	= 0;
	set_nf();

	for(int i=0; i<Number_of_Lumis; ++i) _V[i]=0.;

	for(Parton q(N_FMAX); q>Parton_g; --q){
		Parton bq(-q);
		for(Parton p(N_FMAX); p>Parton_g; --p){
			Parton bp(-p);
//				cout << "i="<<q.lha()-1 <<  "\tj="<<p.lha()-1 << endl;
			 L[q.lha()-1][p.lha()-1] = C->L(q,bp);
			bL[q.lha()-1][p.lha()-1] = C->L(bq,p);
			 R[q.lha()-1][p.lha()-1] = C->R(q,bp);
			bR[q.lha()-1][p.lha()-1] = C->R(bq,p);
			 D[q.lha()-1][p.lha()-1] = C->L(q,bp)-C->R(q,bp);
			bD[q.lha()-1][p.lha()-1] = C->L(bq,p)-C->R(bq,p);
			QQ[q.lha()-1][p.lha()-1] = C->Q(q,bp);
		}
	}
//		cout << "<-Lumi::init(p)" << endl;
}

void Lumi_V::set_nf(){
	if(nf!=(*_nf)){
		nf=(*_nf);

		Dpp=bDpp=QQqp=0.;
		QQ_S = Vector<N_FMAX,double>(0.);
		QQ_bS = Vector<N_FMAX,double>(0.);

		for(Parton q(nf); q>Parton_g; --q){
			Parton bq(-q);
			for(Parton p(nf); p>Parton_g; --p){
				Parton bp(-p);
				QQqp			+=	C->Q( q,bp);
				QQ_S[q.lha()-1]	+=  C->Q( q,bp);
				QQ_bS[q.lha()-1]+=  C->Q(bq, p);
			}
			Dpp	+= C->L( q,bq)-C->R( q,bq);
			bDpp	+= C->L(bq, q)-C->R(bq, q);
		}
	}
}


void Lumi_V::update(const VectorPDF* const Phi[2], ENUM_LUMI_UPDATE l_update){
	const VectorPDF &Phi_0(*(Phi[0]));
	const VectorPDF &Phi_1(*(Phi[1]));
	set_nf();
//	cout << "\t-> DY_NLO::Lumi::update()" << endl;
//	cout << Q << endl;
//	cout << Phi_0 << endl;
//	cout << Phi_1 << endl;
	const double g0(Phi_0[Parton_g]);
	const double g1(Phi_1[Parton_g]);


//		for(double* iV(_V); iV<_V+Number_of_Lumis; ++iV) *iV=0.;
	_V.assign(Number_of_Lumis,0.);


	Vector<N_FMAX,double>::iterator itQQ_Sq(QQ_S.begin());
	Vector<N_FMAX,double>::iterator itQQ_Sbq(QQ_bS.begin());


	for(int q(0); q<nf; ++q){

		const double  q0(Phi_0[Parton(1+q)]);
		const double bq0(Phi_0[Parton(-1-q)]);
		const double  q1(Phi_1[Parton(1+q)]);
		const double bq1(Phi_1[Parton(-1-q)]);

//			const double QQ_Sq(QQ_S[q]);
//			if(QQ_Sq!=QQ_bS[q]) cout << "_______________________"<< endl;

		_V[LV_qG_LO]		+=	 q0* 	(*itQQ_Sq);
		_V[LV_qG_LO]		+=	bq0* 	(*itQQ_Sbq);

		_V[LV_Gq_LO]		+=		  	(*itQQ_Sq)	*  q1;
		_V[LV_Gq_LO]		+=		  	(*itQQ_Sbq)	* bq1;

		if(l_update!=l_update_LO){
			_V[LV_qbq_NLO_bc]	+=	 q0	*(*itQQ_Sq)		*bq1;
			_V[LV_qbq_NLO_bc]	+=	bq0	*(*itQQ_Sbq)	* q1;

			_V[LV_qbq_NLO_bd]	+=	 q0	*(*itQQ_Sbq)	*bq1;
			_V[LV_qbq_NLO_bd]	+=	bq0	*(*itQQ_Sq)		* q1;

			if(l_update==l_update_NLO_0){
				_V[LV_qG_NLO]		+=	 q0	* D[q][q] *Dpp;
				_V[LV_qG_NLO]		+=	bq0	*bD[q][q] *bDpp;

				_V[LV_Gq_NLO]		+=		  D[q][q] *Dpp	* q1;
				_V[LV_Gq_NLO]		+=		 bD[q][q] *bDpp	*bq1;
			}



			_V[LV_qbq_NLO_ab]	+=	 q0	* D[q][q] *Dpp	*bq1;
			_V[LV_qbq_NLO_ab]	+=	bq0	*bD[q][q] *bDpp	* q1;

			_V[LV_qbq_NLO_bb]	+=	 q0	*QQqp			*bq1;
			_V[LV_qbq_NLO_bb]	+=	bq0	*QQqp			* q1;

		}
		const double  L_qq( L[q][q]);
		const double bL_qq(bL[q][q]);
		const double  R_qq( R[q][q]);
		const double bR_qq(bR[q][q]);

		for(int p(0); p<nf; ++p){
//			cout << "q="<<q << "\tp="<<p << endl;

			const double  p1(Phi_1[Parton(1+p)]);
			const double bp1(Phi_1[Parton(-1-p)]);

			const double QQ_qp(QQ[q][p]);
			const double QQ_pq(QQ[p][q]);

			const double  L_pp( L[p][p]);
			const double bL_pp(bL[p][p]);

			const double  R_pp( R[p][p]);
			const double bR_pp(bR[p][p]);

			const double QQ_Sp(QQ_S[p]);
			const double QQ_Sbp(QQ_bS[p]);


			_V[LV_qbq_LO]		+=	 q0	*QQ_qp	*bp1;
			_V[LV_qbq_LO]		+=	bq0	*QQ_pq	*p1;

			if(l_update==l_update_NLO_x2){
				_V[LV_qbq_NLO_LL]	+=	 q0 *  L_qq* L_pp* bp1;
				_V[LV_qbq_NLO_LL]	+=	 q0 *  R_qq* R_pp* bp1;
				_V[LV_qbq_NLO_LL]	+=	bq0 * bL_qq*bL_pp*  p1;
				_V[LV_qbq_NLO_LL]	+=	bq0 * bR_qq*bR_pp*  p1;

				_V[LV_qbq_NLO_LR]	+=	 q0 *  L_qq* R_pp* bp1;
				_V[LV_qbq_NLO_LR]	+=	 q0 *  R_qq* L_pp* bp1;
				_V[LV_qbq_NLO_LR]	+=	bq0 * bL_qq*bR_pp*  p1;
				_V[LV_qbq_NLO_LR]	+=	bq0 * bR_qq*bL_pp*  p1;

				_V[LV_qbq_NLO_cc]	+=	 q0	*(*itQQ_Sq)		*bp1;
				_V[LV_qbq_NLO_cc]	+=	bq0	*(*itQQ_Sbq)	* p1;

				_V[LV_qbq_NLO_dd]	+=	 q0	*QQ_Sbp	*bp1;
				_V[LV_qbq_NLO_dd]	+=	bq0	*QQ_Sp	* p1;

				_V[LV_qq_NLO_ac]	+=	 q0	* QQ_qp	*  p1;
				_V[LV_qq_NLO_ac]	+=	bq0	* QQ_pq	* bp1;

				_V[LV_qq_NLO_bd]	+=	 q0	* QQ_pq	*  p1;
				_V[LV_qq_NLO_bd]	+=	bq0	* QQ_qp	* bp1;

				_V[LV_qq_NLO_ad]	+=	 q0	* QQ_qp	*  q1;
				_V[LV_qq_NLO_ad]	+=	bq0	* QQ_pq	* bq1;

				_V[LV_qq_NLO_LL]	+=	 q0 *  L_qq* L_pp*  p1;
				_V[LV_qq_NLO_LL]	+=	 q0 *  R_qq* R_pp*  p1;
				_V[LV_qq_NLO_LL]	+=	bq0 * bL_qq*bL_pp* bp1;
				_V[LV_qq_NLO_LL]	+=	bq0 * bR_qq*bR_pp* bp1;

				_V[LV_qq_NLO_LR]	+=	 q0 *  L_qq* R_pp*  p1;
				_V[LV_qq_NLO_LR]	+=	 q0 *  R_qq* L_pp*  p1;
				_V[LV_qq_NLO_LR]	+=	bq0 * bL_qq*bR_pp* bp1;
				_V[LV_qq_NLO_LR]	+=	bq0 * bR_qq*bL_pp* bp1;

				_V[LV_qq_NLO_aa]	+=	 q0	*(*itQQ_Sq)	* p1;
				_V[LV_qq_NLO_aa]	+=	bq0	*(*itQQ_Sbq)	*bp1;

				_V[LV_qq_NLO_bb]	+=	 q0	*QQ_Sp	* p1;
				_V[LV_qq_NLO_bb]	+=	bq0	*QQ_Sbp	*bp1;
			}
		}
		++itQQ_Sq;
		++itQQ_Sbq;
	}
	_V[LV_qG_LO] 	*= g1;
	_V[LV_qG_NLO]	*= g1;

	_V[LV_Gq_LO] 	*= g0;
	_V[LV_Gq_NLO] 	*= g0;

	_V[LV_GG_NLO] 	 = g0*QQqp*g1;


//	cout	<< "DY_NLO::Lumi::update:" << endl
//			<< "g0 = " << g0 << "\tg1 = " << g1 << endl
//			<< "Q_pp_bp = " << Q_pp_bp << endl;

}
