#ifndef RES_K_INTEGRAL_H
#define RES_K_INTEGRAL_H

#include "Res_Exponents.h"
#include <cstdio>

#include "gsl_integration.h"
//#include <gsl/gsl_sf_bessel.h>
//#include <gsl/gsl_integration.h>

//namespace CuTe {
//namespace NS_Resumed {
//
//
//}	//namespace NS_Resumed
//}//namespace CuTe


////Forward Declaration of gsl functions
//double gsl_sf_bessel_J0(const double x);
//double gsl_sf_bessel_K0(const double x);

#ifdef __cplusplus
extern "C" {
#endif
//Forward Declaration of gsl functions
double gsl_sf_bessel_J0(const double x);
double gsl_sf_bessel_K0(const double x);
double gsl_sf_exp(const double x);
#ifdef __cplusplus
}
#endif

namespace CuTe {

namespace NS_Resumed {

//Forward Declaration of gsl struct

using namespace NS_enums::NS_NP;

using NS_GSL_INT::GSL_Integrator;

using std::ostream;
using std::vector;

class _K_Integral_:public Pol<double> {
public:
	_exp_pol_ *gexp;
	Pol< complex<double> > kexp;
protected:

	const double *qT;
	int ord_a_or_e;

//	inline void _init_(
//			RES res, int order, bool improved, bool exponent
//			,const int *nf, const double *as, const double *LM
//			,bool Higgs_true_else_false
//			,const _AD_Gamma_PADE& Gamma_Cusp_3=_AD_Gamma_PADE()
//			){
//		gexp.init(res,order,improved,exponent,nf,as,LM,Higgs_true_else_false,Gamma_Cusp_3);
//	}
//
//	template< RES res >
//	inline void _init_(const CS_params<res> *p){
//		_init_(res,p->order,p->improved,p->exponent,&(p->nf),&(p->a),&(p->L_M),p->prod,p->Gamma_Cusp_3);
//	}

	inline void _init_(_exp_pol_ &gexp, const double *qT){
		this->gexp		= &(gexp);
		this->qT		= qT;
		this->ord_a_or_e= gexp.get_order();
	}

	inline virtual void v_update_mu(){
		cerr << "***************************************************" << endl
			 << "*** _K_Integral_::v_update_mu called -> exit(1) ***" << endl
			 << "***************************************************" << endl;
		exit(1);
	}

public:


	_K_Integral_()
	:Pol<double>(5,0.)
	,gexp(NULL),kexp(2,complex<double>(0.,0.))
	,qT(NULL),ord_a_or_e(){

	}

	_K_Integral_(const _K_Integral_& K)
	:Pol<double>(5,0.)
	,gexp(K.gexp),kexp(K.kexp)
	,qT(K.qT),ord_a_or_e(K.ord_a_or_e){
		set_equalt_to(K);
	}

	_K_Integral_& operator=(const _K_Integral_& K){
		set_equalt_to(K);
		gexp 		= (K.gexp);
		kexp 		= (K.kexp);
		qT	 		= (K.qT);
		ord_a_or_e	= (K.ord_a_or_e);
		return *this;
	}

	virtual ~_K_Integral_(){

	}

	inline virtual Pol<double>& replaceL(const Pol< Pol<double> > &P){
		cerr << "***************************************************" << endl
			 << "*** _K_Integral_::replaceL(P) called -> exit(1) ***" << endl
			 << "***************************************************" << endl;
		exit(1);
		return *this;
	}

	inline Pol<double>& replaceL_E(Pol< Pol<double> > P){
//		NS_Utils::DOUT	dout("_K_Integral_::replaceL_E","P");
//		dout()	<<"gexp:"<<*gexp<<endl;
//		dout()	<<"P:"<<P<<endl;
		P.mul_this_by_Pol(*gexp,ord_a_or_e);

		return replaceL(P);
	}

	inline virtual ostream& v_print(ostream &os)const{
		cerr << "***************************************************" << endl
			 << "*** _K_Integral_::v_print(os) called -> exit(1) ***" << endl
			 << "***************************************************" << endl;
		exit(1);
		return os;
	}

	inline void update_mu(){//const Continious_Order_Reduction *reduction=NULL){
//		NS_Utils::DOUT dout("_K_Integral_::update_mu");
//		dout()	<<"qT="<<*qT	<<endl;
		gexp->update_mu_exp(kexp);//,reduction);
		v_update_mu();
		for(int i=0; i<=_order; ++i) if(_P[i]!=_P[i]){
			cerr<<"NAN:	_K_Integral_::update_mu["<<i<<"]"<<endl;
			to_stream(cerr,1);
			return;
		}
	}

	inline virtual ostream& print_Kl(ostream & os)const{
		cerr << "****************************************************" << endl
			 << "*** _K_Integral_::print_Kl(os) called -> exit(1) ***" << endl
			 << "****************************************************" << endl;
		exit(1);
		return os;
	}

	virtual ostream& to_stream(ostream & os,const int ntabs=0)const{
		stringstream sstab;
		for(int i=0; i<ntabs; ++i){
			sstab <<"\t";
		}
		string tab(sstab.str());
		os<<tab	<<"K(ord="<<gexp->get_order()<<", pc="<<gexp->get_power_counting()<<", exp="<<gexp->get_exponent()<<")"<<endl;
		os<<tab	<<"kexp:"<<kexp<<endl;
		os<<tab	<<"gexp:"<<*gexp<<endl;
		os<<tab	<<"kres:";print_Kl(os);os<<endl;
		os<<tab	<<"this:"<<*this<<endl;
		return os;
	}

	inline const Pol< complex<double> >& get_kexp()const{ return kexp;}

};

class K_LL:public _K_Integral_{
protected:
	static const size_t intMem=1e6;
	GSL_Integrator gsl_int;
	int max_L;
public:
	vector< double > _Kl;
protected:

//	const double* q_star;
	const double* mu;
	const double* LM;//	= eta/(a*G0) = Log(M^2/mu^2)
	static const double accuracy=1e-4;

	CUTOFF cutoff;
	const double* Lambda;

	int l;
	double qT_b0_imu;
	double b02_imu2;

	bool wick_rotated;

public:
	template<CUTOFF cut>
	inline static double cut_function(double xT,double LT, double Lambda, double LM){
		return 1.;
	}

	template<CUTOFF cut>
	inline static double ln_cut_function(const double xT, const double LT, const double Lambda, const double LM){
		return 0.;
	}

protected:
//	inline double _kn_wick_(double z){
//		using ::exp;
//		if((*qT)*z > 704) return 0.;	//gsl_sf_bessel_K0(705)< min[double];
//		const complex< double > LT(2*log((*mu)*z/b_0),PI);
//
//		double _kT =z;
//		_kT	*= (-1.0*M_1_PI*gsl_sf_bessel_K0((*qT)*z));
//
//		const complex< double > tmp = kexp.of(LT);
//		const complex< double > lL_T = log(LT);
//
//		return _kT*( exp( tmp + lL_T*static_cast<double>(l) ).imag() );
//	}
	inline double _kn_wick_(const double Lp)const{
		using ::exp;
		double erg(0.);
		{
			const complex<double> LT(+Lp,PI);
			const double z(exp(+0.5*Lp)*qT_b0_imu);

			if( (0.<z)&&(z<704) ){//gsl_sf_bessel_K0(705)< min[double];
				const complex<double> k=(kexp.of(LT));
				const double re(+Lp+k.real());

				if(-708<re){	//	exp(re) < min[double]
					const double im(k.imag() + arg(LT)*static_cast<double>(l));

					erg+= gsl_sf_bessel_K0(z)*exp(re)*sin(im);
				}
			}
		}
		{
			const complex<double> LT(-Lp,PI);
			const double z(exp(-0.5*Lp)*qT_b0_imu);

			if( (0.<z)&&(z<704) ){	//gsl_sf_bessel_K0(705)< min[double];
				const complex<double> k=(kexp.of(LT));
				const double re(-Lp+k.real());

				if(-708<re){	//	exp(re) < min[double]
					const double im(k.imag() + arg(LT)*static_cast<double>(l));

					erg+= gsl_sf_bessel_K0(z)*exp(re)*sin(im);
				}
			}
		}
		erg	= -M_1_PI*0.5*b02_imu2*erg*pow(pow_2(Lp)+PI2,static_cast<double>(l)/2.);
		return erg;
	}

	inline double _kn_unwick_(const double Lp)const{
		using ::exp;
		double erg(0.);
		const double logLp(log(Lp));

		{
			const double e(+Lp+kexp.of(+Lp).real()+logLp*l);
			if(-708<e){	//	exp(e) < min[double]
				const double z(exp(+0.5*Lp)*qT_b0_imu);
				erg+= gsl_sf_bessel_J0(z)*exp(e);
			}
		}
		{
			const double e(-Lp+kexp.of(-Lp).real()+logLp*l);
			if(-708<e){	//	exp(e) < min[double]
				const double sign( (l%2) ? -1. : +1. );
				const double z(exp(-0.5*Lp)*qT_b0_imu);
				erg+= sign*gsl_sf_bessel_J0(z)*exp(e);
			}
		}
		erg	= 0.25*b02_imu2*erg;
		return erg;
	}

//	template<CUTOFF cut>
//	inline double _kn_unwick_cut(const double Lp){
//		using ::exp;
//		double erg(0.);
////		const double logLp(log(Lp));
//
//		{
//			const double xT(exp(+0.5*Lp)*b_0/(*mu));
//			const double e(+Lp+kexp.of(+Lp).real()+ln_cut_function<cut>(xT,+Lp,*Lambda,*LM));
//			if(-708<e){	//	exp(e) < min[double]
////				const double z(exp(+0.5*Lp)*qT_b0_imu);
//				erg+= gsl_sf_bessel_J0((*qT)*xT)*exp(e);
//			}
//		}
//		{
//			const double xT(exp(-0.5*Lp)*b_0/(*mu));
//			const double e(-Lp+kexp.of(-Lp).real()+ln_cut_function<cut>(xT,-Lp,*Lambda,*LM));
//			if(-708<e){	//	exp(e) < min[double]
//				const double sign( (l%2) ? -1. : +1. );
////				const double z(exp(-0.5*Lp)*qT_b0_imu);
//				erg+= sign*gsl_sf_bessel_J0((*qT)*xT)*exp(e);
//			}
//		}
//		erg	= 0.25*b02_imu2*erg*pow(+Lp,l);
//		return erg;
//	}

	template<CUTOFF cut>
	inline double _kn_(const double xT)const{
		double erg(0.);

		const double z	= (*qT)*xT;
		const double Lp	= 2.*log(z/qT_b0_imu);
		const double LLp= log(fabs(Lp));

		const double exponent = kexp.of(Lp).real() + LLp*l + ln_cut_function<cut>(xT,Lp,*Lambda,*LM);

		if(-708<exponent){	//	exp(e) < min[double]
			const double sign( ((Lp<0.)&&(l%2)) ? -1. : +1. );
			erg	 = sign;
			erg	*= ::exp(exponent);
			erg *= 0.5*xT;
			erg *= gsl_sf_bessel_J0(z);
		}

		return erg;
	}

//	template<CUTOFF cut>
//	inline double _kn_(const double xT)const{
//		using ::exp;
//		const double LT=2.*log((*mu)*xT/b_0);
//
//		double _kT = xT;
//		_kT	*= 0.5*gsl_sf_bessel_J0((*qT)*xT);
//		_kT	*= cut_function<cut>(xT,LT,*Lambda,*LM);
//		_kT	*= exp( (kexp.of(LT).real()) );
//
//		return _kT*pow(LT,l);
//	}

	inline static double _integrand_wick_(double Lp, void *params){
		K_LL *K(static_cast<K_LL*>(params));
		return K->_kn_wick_(Lp);
	}

	inline static double _integrand_unwick_(double Lp, void *params){
		K_LL *K(static_cast<K_LL*>(params));
		return K->_kn_unwick_(Lp);
	}

	template<CUTOFF cut>
	inline static double _integrand_cut_(double xT, void *params){
		K_LL *K(static_cast<K_LL*>(params));
//		return K->_kn_<cut>(xT);
		return K->_kn_<cut>(xT);
	}

	virtual bool _choose_integrand_get_with_cut(){
//		NS_Utils::DOUT dout("K_LL::_choose_integrand");



		if((*Lambda)==0.){
			if(wick_rotated){
				gsl_int.function = _integrand_wick_;
			}else{
				gsl_int.function = _integrand_unwick_;
			}
			return false;
		}else{
			switch (cutoff) {
			case CUTOFF_ENHANCED:
				gsl_int.function = _integrand_cut_<CUTOFF_ENHANCED>;
				return true;
				break;
			case CUTOFF_DIPOL:
				gsl_int.function = _integrand_cut_<CUTOFF_DIPOL>;
				return true;
				break;
			case CUTOFF_GAUSS:
				gsl_int.function = _integrand_cut_<CUTOFF_GAUSS>;
				return true;
				break;
			default:
				cerr	<<"WRONG CUTOFF -> No Cutoff"<<endl;
				if(wick_rotated){
					gsl_int.function = _integrand_wick_;
				}else{
					gsl_int.function = _integrand_unwick_;
				}
				return false;
				break;
			}
		}
	}

//	void v_update_mu();
	inline void v_update_mu(){
//		NS_Utils::DOUT dout("K_LL::v_update_mu");
//		dout()	<<"mu="<<*mu	<<"\t"<<"LM="<<*LM	<<endl;

		const double a(0.);

		b02_imu2	= b_0/(*mu);
		qT_b0_imu	= (*qT)*b02_imu2;
		b02_imu2	*= b02_imu2;

		if(*qT>10.) wick_rotated = true;

		const bool with_cut(_choose_integrand_get_with_cut());
		char numstr[32];

		for(l=0; l<=max_L; ++l){

			_Kl[l]	= gsl_int.integrate_a_to_inf(a,accuracy);

			if( (gsl_int.has_failed()) && (!with_cut) ){
//				NS_Utils::DOUT dout("K_LL::switch");
//				dout()	<<"K_LL switched from ";
//				if(wick_rotated){
//					dout(0)	<<"\tWick(K0) to ";
//				}else{
//					dout(0)	<<"\tUNwick(J0) to ";
//				}
				wick_rotated	= !wick_rotated;
//				if(wick_rotated){
//					dout(0)	<<"\tWick(K0)"<<endl;
//				}else{
//					dout(0)	<<"\tUNwick(J0)"<<endl;
//				}
//				dout()	<<"\t"<<"qT = "<<*qT	<<"\t"<<"mu = "<<*mu	<<"\t"<<"n = "<<l	<<endl;
				_choose_integrand_get_with_cut();
				_Kl[l]	= gsl_int.integrate_a_to_inf(a,accuracy);
			}

			if(gsl_int.has_failed()){
				std::sprintf(numstr, "K_LL %d", l);
				gsl_int.status(numstr);
				cerr	<<"qT="<<*qT	<<"\t"<<"mu="<<*mu	<<"\t"<<"qT_b0_imu="<<qT_b0_imu;
				if(with_cut){
					cerr	<<"\t"<<cutoff<<endl;
				}else{
					if(wick_rotated){
						cerr	<<"\tWick(K0)"<<endl;
					}else{
						cerr	<<"\tUNwick(J0)"<<endl;
					}
				}
			}

		}
	}



public:

	inline void init(_exp_pol_ &gexp
					, const double *qT, const double *mu, const double *LM//, const double *q_star
					, CUTOFF cutoff, const double *Lambda
					){

		_init_(gexp,qT);

		switch (gexp.get_power_counting()) {
		case PC_IMP:
			max_L	= 3*ord_a_or_e;	// g[2][3] ~ (e^1L^3)
			break;
		case PC_STD:
			max_L	= 3*ord_a_or_e/2;// + (ord_a_or_e%2);	// g[2][3] ~ (as^2L^3)
			break;
		case PC_NAIVE:
			max_L	= 2*ord_a_or_e;	// g[1][2] ~ (as^1L^2)
			break;
		default:
			break;
		}

		_Kl.resize(max_L+1);

		this->mu	= mu;
		this->LM	= LM;//	= eta/(a*G0) = Log(M^2/mu^2)
//		this->q_star= q_star;
		wick_rotated	= false;	//Start with J0 Integral

		this->cutoff= cutoff;
		this->Lambda= Lambda;

		_choose_integrand_get_with_cut();

		l=-1.;
	}


	inline Pol<double>& replaceL(const Pol< Pol<double> > &P){
		int N	= min_of(P.order(),ord_a_or_e);

		order(N);
		mul_this_by_scalar(0.);

		for(int n=N; n>=0; --n){
			const int L	= P[n].order();

			if(L>max_L){//FIXME DEBUG
				cerr	<<"K_LL::replaceL(P) "<<endl
						<<"\tmax_L="<<max_L	<<"\tsize="<<_Kl.size()<<endl
						<<"\t    L="<<L<<endl
						<<"\t    P:"<<P<<endl;
			}

			for(int l=0; l<=L; ++l){
				_P[n] += _Kl[l]*P[n][l];
			}
		}

		return *this;
	}

	K_LL()
	:_K_Integral_()
	,gsl_int(intMem)
//	,w(gsl_integration_workspace_alloc(intMem))
	,max_L(),_Kl(vector<double>(16,0.))
	,mu(NULL),LM(NULL)//,q_star(NULL)
	,cutoff(),Lambda(NULL)
	,l()
	,qT_b0_imu(),b02_imu2()
	,wick_rotated(false){
		gsl_int.params	= this;
	}

	K_LL(_exp_pol_ &gexp
		,const double *qT, const double *mu, const double *LM//, const double *q_star
		,CUTOFF cutoff, const double *Lambda
		)
	:_K_Integral_()
	,gsl_int(intMem)
//	,w(gsl_integration_workspace_alloc(intMem))
	,max_L(),_Kl(vector<double>(16,0.))
	,mu(NULL),LM(NULL)//,q_star(NULL)
	,cutoff(),Lambda(NULL)
	,l()
	,qT_b0_imu(),b02_imu2()
	,wick_rotated(false){
		gsl_int.params	= this;
		init(gexp,qT,mu,LM,cutoff,Lambda);
	}


	K_LL(const K_LL &K)
	:_K_Integral_(K)
	,gsl_int(intMem)
//	,w(gsl_integration_workspace_alloc(intMem))
	,max_L(K.max_L),_Kl(vector<double>(16,0.))
	,mu(K.mu),LM(K.LM)//,q_star(K.q_star)
	,cutoff(K.cutoff),Lambda(K.Lambda)
	,l(K.l)
	,qT_b0_imu(K.qT_b0_imu),b02_imu2(K.b02_imu2)
	,wick_rotated(K.wick_rotated){
//		NS_Utils::DOUT	dout("K_LL","K");
		_Kl	= K._Kl;
		gsl_int.params	= this;
	}

	inline K_LL& operator=(const K_LL &K){
//		NS_Utils::DOUT	dout("K_LL::operator=","K");

		_K_Integral_::operator =(K);
		gsl_int			= (K.gsl_int);
		gsl_int.params	= this;
		max_L			= (K.max_L);
		_Kl				=  K._Kl;
		mu				= (K.mu);
		LM				= (K.LM);
//		q_star			= (K.q_star);
		cutoff			= (K.cutoff);
		Lambda			= (K.Lambda);
		l				= (K.l);
		qT_b0_imu		= (K.qT_b0_imu);
		b02_imu2		= (K.b02_imu2);
		wick_rotated	= (K.wick_rotated);

		return *this;
	}

	inline virtual ostream& print_Kl(ostream & os)const{
		using NS_Utils::operator <<;
		os	<< _Kl;
		return os;
	}

	~K_LL(){
	}

	virtual ostream& to_stream(ostream & os,const int ntabs=0)const{
		string tab;
		for(int i=0; i<ntabs; ++i){
			tab += "\t";
		}
		os<<tab<<"K_LL"<<endl;
		return _K_Integral_::to_stream(os,ntabs+1);
	}

	inline const vector<double>& get_Kn()const{ return _Kl;}

	inline void change_NPE(const NS_enums::NS_NP::CUTOFF cutoff){
		this->cutoff	= cutoff;
	}

};

template < >
inline double K_LL::ln_cut_function<CUTOFF_GAUSS>(const double xT, const double LT, const double Lambda, const double LM){
	return -2.*pow_2(Lambda*xT);
}

template < >
inline double K_LL::ln_cut_function<CUTOFF_DIPOL>(const double xT, const double LT, const double Lambda, const double LM){
	return -2.*log(1.+pow_2(Lambda*xT));
}

template < >
inline double K_LL::ln_cut_function<CUTOFF_ENHANCED>(const double xT, const double LT, const double Lambda, const double LM){
	return -(LM+LT)*pow_2(Lambda*xT);
}

template < >
inline double K_LL::ln_cut_function<CUTOFF_GAUSS_HALF>(const double xT, const double LT, const double Lambda, const double LM){
	return -pow_2(Lambda*xT);
}

template < >
inline double K_LL::ln_cut_function<CUTOFF_DIPOL_HALF>(const double xT, const double LT, const double Lambda, const double LM){
	return -log(1.+pow_2(Lambda*xT));
}

template < >
inline double K_LL::ln_cut_function<CUTOFF_ENHANCED_HALF>(const double xT, const double LT, const double Lambda, const double LM){
	return -0.5*(LM+LT)*pow_2(Lambda*xT);
}

template < >
inline double K_LL::cut_function<CUTOFF_GAUSS>(double xT,double LT, double Lambda, double LM){
	using ::exp;
	return exp(-2.*pow_2(Lambda*xT));
}

template < >
inline double K_LL::cut_function<CUTOFF_DIPOL>(double xT,double LT, double Lambda, double LM){
	return pow_2(1./(1.+pow_2(Lambda*xT)));
}

template < >
inline double K_LL::cut_function<CUTOFF_ENHANCED>(double xT,double LT, double Lambda, double LM){
	using ::exp;
	return exp(-(LM+LT)*pow_2(Lambda*xT));
}

template < >
inline double K_LL::cut_function<CUTOFF_GAUSS_HALF>(double xT,double LT, double Lambda, double LM){
	using ::exp;
	return exp(-pow_2(Lambda*xT));
}

template < >
inline double K_LL::cut_function<CUTOFF_DIPOL_HALF>(double xT,double LT, double Lambda, double LM){
	return 1./(1.+pow_2(Lambda*xT));
}

template < >
inline double K_LL::cut_function<CUTOFF_ENHANCED_HALF>(double xT,double LT, double Lambda, double LM){
	using ::exp;
	return exp(-0.5*(LM+LT)*pow_2(Lambda*xT));
//	double erg(exp(-0.5*(LM+LT)*pow_2(Lambda*xT)));
//	if(erg>2.){	//FIXME DEBUG
//		cerr	<<"**********************************************************"<<endl
//				<<"K_LL::cut_function<CUTOFF_ENHANCED_HALF> = "<<erg<<endl
//				<<"\txT="<<xT<<"\tL="<<Lambda<<"\tLT="<<LT<<"\tLM="<<LM<<endl;
//	}
//	return erg;
}



class K_LLO:public _K_Integral_{
protected:
	static const int _N_max = 2;
	static const int L_max = 2*_N_max;
	static const int N_max = L_max+_N_max;

//	 0      _N    N= _N +L
//	0- - - - - - /
//	 |       |  /
//	 |       | /
//	L| - - - |/

	vector< Pol<double> > _Kla;

	const double* Lmu;	//log(mu^2/qT^2)
	const double* eta_B;//= C_B * Gamma_0*a*log(M^2/mu^2)



	inline void v_update_mu(){
//		NS_Utils::DOUT dout("K_LLO::v_update_mu");
//		dout()	<<"L_max="<<L_max	<<"\t"<<"N_max="<<N_max	<<"\t"<<"_N_max="<<_N_max	<<endl;
//		dout()	<<"qT="<<*qT	<<"\t"<<"Lmu="<<*Lmu	<<endl;


//		dout()	<<"eta="<<*eta	<<"\t"<<"eta_B="<<eta_B	<<endl;

//		_Kla[L][a]
		_Kla[0][0]=0.;
		_Kla[0][1]=1.;
		_Kla[0][2]=-(*Lmu);
		_Kla[0][3]=0.5*pow_2(*Lmu);
		_Kla[0][4]=(-pow_3(*Lmu)+4.*zeta_3)/6.;
		_Kla[0][5]=(*Lmu)*(pow_3(*Lmu)-16.*zeta_3)/24.;
		_Kla[0][6]=(-pow_5(*Lmu) + 40.*pow_2(*Lmu)*zeta_3 +48.*zeta_5)/120.;

		double iqT2 = 1./pow_2(*qT);

		for(int l=1; l<=L_max; ++l){
			for(int a=0; a<=(N_max-l); ++a){
				_Kla[l][a]=-(a+1.)*_Kla[l-1][a+1];
			}
			_Kla[l-1].mul_this_by_scalar(iqT2);
			_Kla[l-1].set_of(*eta_B);
		}
		_Kla[L_max].mul_this_by_scalar(iqT2);
		_Kla[L_max].set_of(*eta_B);
	}


public:

	inline void init(_exp_pol_ &gexp
					,const double *qT
					,const double *Lmu, const double *eta_B
					){
//		NS_Utils::DOUT dout("K_LLO::init","p");
//		dout()	<<"res="<<RES_LLO	<<"\t"<<"ord="<<order	<<"\timp="<<false <<endl;
		_init_(gexp,qT);

		this->Lmu	=Lmu;
		this->eta_B	=eta_B;

	}

//	template< RES res >
//	inline void init(_exp_pol_ &gexp, const CS_params<res> *p){
//		init(gexp
//			,p->order
//			,&(p->qT)
//			,&(p->L_mu),&(p->eta),&(p->C_B)
//		);
//	}

	inline Pol<double>& replaceL(const Pol< Pol<double> > &P){
//		NS_Utils::DOUT	dout("K_LLO::replaceL","P");

		int NP	= min_of(P.order(),ord_a_or_e);
		int N	= ord_a_or_e;

		order(N);
		mul_this_by_scalar(0.);

		for(int nP=0; nP<=NP; ++nP){


			int LP(P[nP].order());

//			stringstream tmp;
//			tmp<<"nP="<<nP<<"("<<NP<<")\t"<<"LP="<<LP;
//			NS_Utils::DOUT	dout(tmp.str());

			for(int lP=0; lP<=LP; ++lP){
				double P_nP_lP = P[nP][lP];

//				stringstream tmp;
//				tmp<<"lP="<<lP<<"("<<LP<<")";
//				NS_Utils::DOUT	dout(tmp.str());

				for(int n=nP; n<=N; ++n){

//					stringstream tmp;
//					tmp<<"n="<<n<<"("<<N<<")\t"<<"n-nP="<<n-nP;
//					NS_Utils::DOUT	dout(tmp.str());


					if( (n-nP)>_Kla[lP].order() ){
						cerr<<"K_LLO::replaceL"<<endl;//FIXME DEBUG
						exit(1);
					}

					_P[n] += _Kla[lP][n-nP]*P_nP_lP;


				}
			}
		}

		return *this;
	}

	inline K_LLO()
	:_K_Integral_()
	,_Kla(L_max+1,Pol<double>(N_max,0.))
	,Lmu(NULL),eta_B(NULL){
		for(int l=0; l<=L_max; ++l){
			_Kla[l].order(N_max-l);
		}
	}

	inline K_LLO(_exp_pol_ &gexp
			,const double *qT
			,const double *Lmu, const double *eta_B
			)
	:_K_Integral_()
	,_Kla(L_max+1,Pol<double>(N_max,0.))
	,Lmu(NULL),eta_B(NULL){
		for(int l=0; l<=L_max; ++l){
			_Kla[l].order(N_max-l);
		}

		init(gexp,qT,Lmu,eta_B);
	}

//	template<RES res>
//	inline K_LLO(_exp_pol_ &gexp, const CS_params<res> *p)
//	:_K_Integral_()
//	,_Kla(L_max+1,Pol<double>(N_max,0.))
//	,Lmu(NULL),eta(NULL),C_B(NULL){
//		for(int l=0; l<=L_max; ++l){
//			_Kla[l].order(N_max-l);
//		}
//		init(gexp,p);
//	}

	inline K_LLO(const K_LLO &K)
	:_K_Integral_(K)
	,_Kla(K._Kla)
	,Lmu(K.Lmu),eta_B(K.eta_B){
//		NS_Utils::DOUT	dout("K_LLO","K");
	}

	inline K_LLO& operator=(const K_LLO &K){
//		NS_Utils::DOUT	dout("K_LLO::operator=","K");
		_K_Integral_::operator =(K);
		_Kla	= (K._Kla);
		Lmu		= (K.Lmu);
		eta_B	= (K.eta_B);
		return *this;
	}

	inline virtual ostream& print_Kl(ostream & os)const{
		os	<<_Kla[0];
		for(int l=1; l<=L_max; ++l){
			os<<endl<<_Kla[l];
		}
		return os;
	}

	virtual ostream& to_stream(ostream & os,const int ntabs=0)const{
		string tab;
		for(int i=0; i<ntabs; ++i){
			tab += "\t";
		}
		os<<tab<<"K_SQT"<<endl;
		return _K_Integral_::to_stream(os,ntabs+1);
	}

};
}	//namespace NS_Resumed

}	//namespace CuTe

#endif	// RES_K_INTEGRAL_J
