#ifndef RES_PARAMETER_H_
#define RES_PARAMETER_H_

#include "enums.h"

namespace CuTe {

namespace NS_Kinematics {
	class Kinematics;
}  // namespace NS_Kinematics

namespace NS_Ibar {
	class Ibar_Parameter;
}  // namespace NS_Kinematics

namespace NS_Resumed {

using NS_Ibar::Ibar_Parameter;
using NS_Kinematics::Kinematics;
using namespace NS_enums::NS_PROD;

class Ibar_Parameter_Set{
protected:
	PROD prod;
	int order;
	bool improved;
	int order_a;

	Ibar_Parameter* I[2];

	void _exit_I();
	void _init_I();

public:
	double Jac_z;

	const Ibar_Parameter* operator[](bool i)const{ return I[static_cast<int>(i)];}

	inline void init(const PROD prod, const int order,const bool improved, const int order_a){
		_exit_I();
		this->prod		= prod;
		this->order		= order;
		this->order_a	= order_a;
		this->improved	= improved;
		_init_I();
	}

	void set_y(const Kinematics *Kin);

	void update_zz(const Kinematics *Kin);

	Ibar_Parameter_Set()
	:prod(),order(),improved(),order_a()
	,I()
	,Jac_z(){

	}
	Ibar_Parameter_Set(const Ibar_Parameter_Set &RV)
	:prod(RV.prod),order(RV.order),improved(RV.improved)
	,I()
	,Jac_z(RV.Jac_z){
		if(RV.I[0]!=NULL) _init_I();
	}
	Ibar_Parameter_Set& operator=(const Ibar_Parameter_Set &RV){
		_exit_I();
		prod		= (RV.prod);
		order		= (RV.order);
		improved	= (RV.improved);
		Jac_z		= (RV.Jac_z);

		if(RV.I[0]!=NULL) _init_I();
		return *this;

	}
	~Ibar_Parameter_Set(){
		_exit_I();
	}

	std::ostream &properties(std::ostream & os, const int ntab=0)const;


};

class Res_Kinematic_Parameters:public Ibar_Parameter_Set{
protected:
	inline void _init(PROD prod, int order,bool improved,int order_a=0);

public:
	double qT;
	const double *zz;
//protected:
//	const double *xx;	//= xi^pm / z^pm
public:

	void init(const Kinematics *Kin,PROD prod, const int order,const bool improved, const int order_a);
//	void init(const Kinematics *Kin,PROD prod, int order,bool improved);

	void set_qT(const Kinematics *Kin);

	Res_Kinematic_Parameters()
	:Ibar_Parameter_Set(),qT(),zz(NULL)
//	,xx(NULL)
	{

	}

	std::ostream &properties(std::ostream & os, const int ntab=0)const;

};


}  // namespace NS_Resumed
}  // namespace CuTe


#endif //RES_PARAMETER_H_
