#ifndef CHARGES_H_
#define CHARGES_H_

#include "enums.h"
#include "vecmath.h"
#include "utils_simple_math.h"
#include <cstdlib>
#include <complex>

namespace CuTe {


namespace NS_CHARGES {

using std::vector;
using std::complex;

using namespace CuTe::NS_enums::NS_PROD;
using CuTe::NS_Utils::NS_pow::pow_2;

using namespace NS_enums::NS_Partons;


class CKM_MATRIX{
protected:
	Matrix<N_FMAX+1,double> MX;
public:
	Vector<N_FMAX+1,double>::iterator
				 Vud,Vus,Vub
				,Vcd,Vcs,Vcb
				,Vtd,Vts,Vtb;
	Vector<N_FMAX+1,double>::const_iterator
				 CVud,CVus,CVub
				,CVcd,CVcs,CVcb
				,CVtd,CVts,CVtb;

	inline double operator() (const Parton& i,const Parton& j)const{
		return MX[i.flv()][j.flv()];
	}

	inline void init(const vector<double>& ckm){
		vector<double>::const_iterator it(ckm.begin());
		*Vud	= (*(it++));
		*Vus	= (*(it++));
		*Vub	= (*(it++));
		//
		*Vcd	= (*(it++));
		*Vcs	= (*(it++));
		*Vcb	= (*(it++));
		//
		*Vtd	= (*(it++));
		*Vts	= (*(it++));
		*Vtb	= (*(it++));
	}

	inline CKM_MATRIX()
	:MX(0.)
	,Vud(MX.begin(Parton_u.flv())+Parton_d.flv()),Vus(MX.begin(Parton_u.flv())+Parton_s.flv()),Vub(MX.begin(Parton_u.flv())+Parton_b.flv())
	,Vcd(MX.begin(Parton_c.flv())+Parton_d.flv()),Vcs(MX.begin(Parton_c.flv())+Parton_s.flv()),Vcb(MX.begin(Parton_c.flv())+Parton_b.flv())
	,Vtd(MX.begin(Parton_t.flv())+Parton_d.flv()),Vts(MX.begin(Parton_t.flv())+Parton_s.flv()),Vtb(MX.begin(Parton_t.flv())+Parton_b.flv())
	,CVud(Vud),CVus(Vus),CVub(Vub)
	,CVcd(Vcd),CVcs(Vcs),CVcb(Vcb)
	,CVtd(Vtd),CVts(Vts),CVtb(Vtb){
		*Vud	= 0.97427;
		*Vus	= 0.22534;
		*Vub	= 0.00351;
		//
		*Vcd	= 0.22520;
		*Vcs	= 0.97344;
		*Vcb	= 0.0412;
		//
		*Vtd	= 0.00867;
		*Vts	= 0.0404;
		*Vtb	= 0.999146;
//		cout	<<"CKM_MATRIX()\t"
//					<<"("<<*(CVud)<<","<<CVud<<") "
//					<<"("<<*(CVub)<<","<<CVub<<") "
//					<<"("<<*(CVus)<<","<<CVus<<") "
//					<<"("<<*(CVcd)<<","<<CVcd<<") "
//					<<"("<<*(CVcb)<<","<<CVcb<<") "
//					<<"("<<*(CVcs)<<","<<CVcs<<") "
//					<<"("<<*(CVtd)<<","<<CVtd<<") "
//					<<"("<<*(CVtb)<<","<<CVtb<<") "
//					<<"("<<*(CVts)<<","<<CVts<<") "
//				<<endl;
	}

	inline CKM_MATRIX(const CKM_MATRIX& other)
	:MX(other.MX)
	,Vud(MX.begin(Parton_u.flv())+Parton_d.flv()),Vus(MX.begin(Parton_u.flv())+Parton_s.flv()),Vub(MX.begin(Parton_u.flv())+Parton_b.flv())
	,Vcd(MX.begin(Parton_c.flv())+Parton_d.flv()),Vcs(MX.begin(Parton_c.flv())+Parton_s.flv()),Vcb(MX.begin(Parton_c.flv())+Parton_b.flv())
	,Vtd(MX.begin(Parton_t.flv())+Parton_d.flv()),Vts(MX.begin(Parton_t.flv())+Parton_s.flv()),Vtb(MX.begin(Parton_t.flv())+Parton_b.flv())
	,CVud(Vud),CVus(Vus),CVub(Vub)
	,CVcd(Vcd),CVcs(Vcs),CVcb(Vcb)
	,CVtd(Vtd),CVts(Vts),CVtb(Vtb){
//		cout	<<"CKM_MATRIX(other)\t"
//					<<"("<<*(CVud)<<","<<CVud<<"("<<other.CVud<<")) "
//					<<"("<<*(CVub)<<","<<CVub<<"("<<other.CVub<<")) "
//					<<"("<<*(CVus)<<","<<CVus<<"("<<other.CVus<<")) "
//					<<"("<<*(CVcd)<<","<<CVcd<<"("<<other.CVcd<<")) "
//					<<"("<<*(CVcb)<<","<<CVcb<<"("<<other.CVcb<<")) "
//					<<"("<<*(CVcs)<<","<<CVcs<<"("<<other.CVcs<<")) "
//					<<"("<<*(CVtd)<<","<<CVtd<<"("<<other.CVtd<<")) "
//					<<"("<<*(CVtb)<<","<<CVtb<<"("<<other.CVtb<<")) "
//					<<"("<<*(CVts)<<","<<CVts<<"("<<other.CVts<<")) "
//				<<endl;
	}

	inline CKM_MATRIX& operator=(const CKM_MATRIX& other){
//		MX=other.MX;
		*Vud	= (*(other.CVud));
		*Vus	= (*(other.CVus));
		*Vub	= (*(other.CVub));
		//
		*Vcd	= (*(other.CVcd));
		*Vcs	= (*(other.CVcs));
		*Vcb	= (*(other.CVcb));
		//
		*Vtd	= (*(other.CVtd));
		*Vts	= (*(other.CVts));
		*Vtb	= (*(other.CVtb));
//		cout	<<"CKM_MATRIX = other\t"
//					<<"("<<*(CVud)<<","<<CVud<<"("<<other.CVud<<")) "
//					<<"("<<*(CVub)<<","<<CVub<<"("<<other.CVub<<")) "
//					<<"("<<*(CVus)<<","<<CVus<<"("<<other.CVus<<")) "
//					<<"("<<*(CVcd)<<","<<CVcd<<"("<<other.CVcd<<")) "
//					<<"("<<*(CVcb)<<","<<CVcb<<"("<<other.CVcb<<")) "
//					<<"("<<*(CVcs)<<","<<CVcs<<"("<<other.CVcs<<")) "
//					<<"("<<*(CVtd)<<","<<CVtd<<"("<<other.CVtd<<")) "
//					<<"("<<*(CVtb)<<","<<CVtb<<"("<<other.CVtb<<")) "
//					<<"("<<*(CVts)<<","<<CVts<<"("<<other.CVts<<")) "
//				<<endl;
		return *this;
	}

	inline ~CKM_MATRIX(){

	}
};


std::ostream& operator<<( std::ostream& os,const CKM_MATRIX& ckm );
std::istream& operator>>( std::istream& ss, CKM_MATRIX& ckm );



namespace Pauli_Matrices {

enum e_Pauli{ Pauli_1, Pauli_2, Pauli_3, Pauli_Plus, Pauli_Minus};

template < e_Pauli i >
Matrix< 2, complex<double> > Pauli_i();

template < >
Matrix< 2, complex<double> > Pauli_i< Pauli_1 >();

template < >
Matrix< 2, complex<double> > Pauli_i< Pauli_2 >();

template < >
Matrix< 2, complex<double> > Pauli_i< Pauli_3 >();

template < >
Matrix< 2, complex<double> > Pauli_i< Pauli_Plus >();

template < >
Matrix< 2, complex<double> > Pauli_i< Pauli_Minus >();

}  // namespace Pauli_Matrices



class Charge{
public:
	static const Matrix<2,complex<double> > pauli_three;
	static const Matrix<2,complex<double> > pauli_plus;
protected:
	Vector<N_FMAX+1,bool> active_flavour;
	Matrix<N_PARTONS,bool> active_parton_combination;

	PROD prod;

	double SThetaW;		//=sin(Theta_W)
	double iS2ThetaW;	//=1./sin(2.*Theta_W)
	double TThetaW;		//=tan(Theta_W)


	Vector< N_PARTONS,double > qf;	// el. charge of flavour

	Matrix< N_PARTONS,double > r;
	Matrix< N_PARTONS,double > l;

	Matrix< N_PARTONS,double > rr;
	Matrix< N_PARTONS,double > ll;

	Matrix< N_PARTONS,double > qq;

	CKM_MATRIX ckm;

	static const Vector< N_PARTONS,double > default_qf(){
		Vector< N_PARTONS,double > qf;
		const double tmp = 1./3.;
		for(Parton q(N_FMAX); q>Parton_g; q--){
			if(q.is_downtype()){
				qf[q.i]			= -tmp;
				qf[q.bar().i]	= +tmp;
			}else{
				qf[q.i]			= +2.*tmp;
				qf[q.bar().i]	= -2.*tmp;
			}
		}
		qf[Parton_g.i] = 0.;
		return qf;
	}

	inline double l_Photon(const Parton& i, const Parton& j)const{
		if(i==-j) return qf[i.i];
		else return 0.;
	}
	inline double r_Photon(const Parton& i, const Parton& j)const{
		return l_Photon(i,j);
	}
	inline double r_W(const Parton& i, const Parton& j)const{
		return 0.;
	}
	inline double l_W(const Parton& i, const Parton& j)const{
		return pauli_plus[i.is_uptype()][j.is_uptype()].real()*M_SQRT1_2/SThetaW*ckm(j,i);
	}
	inline double l_Z(const Parton& i, const Parton& j)const{
		if(i==-j) return pauli_three[i.is_downtype()][j.is_downtype()].real()*iS2ThetaW*j.sgn()-qf[j.i]*TThetaW;
		return 0.;
	}
	inline double r_Z(const Parton& i, const Parton& j)const{
		if(i==-j) return -qf[j.i]*TThetaW;
		return 0.;
	}

	inline double r_von(const PROD prod, const Parton& i, const Parton& j)const{
		if(!(active_flavour[i.flv()]&&active_flavour[j.flv()])) return 0.;
		if( i.sgn()*j.sgn() >= 0) return 0.;
		if( j<Parton_g) return -l_von(prod,j,i);//conj(l_von(prod,j,i));
		if(prod==PROD_DY)	return r_Photon(i, j);
		if(prod==PROD_W)		return r_W(i, j);
		if(prod==PROD_Z) 		return r_Z(i, j);
		return 0.;
	}
	inline double l_von(const PROD prod, const Parton& i, const Parton& j)const{
		if(!(active_flavour[i.flv()]&&active_flavour[j.flv()])) return 0.;
		if( i.sgn()*j.sgn() >= 0) return 0.;
		if( j<Parton_g) return -r_von(prod,j,i);//conj(r_von(prod,j,i));
		if(prod==PROD_DY)	return l_Photon(i, j);
		if(prod==PROD_W)		return l_W(i, j);
		if(prod==PROD_Z) 		return l_Z(i, j);
		return 0.;
	}

	inline void _init_rlq(){
		for(Parton i(-N_FMAX); i<Parton(N_FMAX); ++i) for(Parton j(-N_FMAX); j<Parton(N_FMAX); ++j){
			r[i.i][j.i] =	r_von(prod,i,j);
			l[i.i][j.i] =	l_von(prod,i,j);

			if((l[i.i][j.i]==0.)&&(r[i.i][j.i]==0.)){
				active_parton_combination[i.i][j.i] = false;
			}else{
				active_parton_combination[i.i][j.i] = true;
			}

			rr[i.i][j.i] =	pow_2(r[i.i][j.i]);//norm(r[i.i][j.i]);
			ll[i.i][j.i] =	pow_2(l[i.i][j.i]);//norm(l[i.i][j.i]);

			qq[i.i][j.i] =	rr[i.i][j.i]+ll[i.i][j.i];


//			cout	<< "Q:"<<endl<<qq<<endl;
		}
	}


public:

	inline void init(const PROD prod,const CKM_MATRIX& ckm_, const double Theta_W, const Vector<N_FMAX+1,bool>& active_flavour){
		this->prod	= prod;

		ckm = ckm_;

		SThetaW		= sin(Theta_W);
		iS2ThetaW	= 1./sin(2.*Theta_W);
		TThetaW		= tan(Theta_W);

		this->active_flavour	= active_flavour;

		_init_rlq();
	}

	inline Charge()
	:active_flavour(false)
	,prod(PROD_WRONG_STATEMENT)
	,SThetaW(0.),iS2ThetaW(0.),TThetaW(0.)
	,qf(default_qf())
	,r(0.),l(0.)
	,rr(0.),ll(0.)
	,qq(0.)
	,ckm(){
	}

	inline Charge(const PROD prod,const CKM_MATRIX& ckm_, const double Theta_W, const Vector<N_FMAX+1,bool>& active_flavour)
	:active_flavour(active_flavour)
	,prod(prod)
	,SThetaW(sin(Theta_W)),iS2ThetaW(1./sin(2.*Theta_W)),TThetaW(tan(Theta_W))
	,qf(default_qf())
	,r(0.),l(0.)
	,rr(0.),ll(0.)
	,qq(0.)
	,ckm(ckm_){
		if(prod!=PROD_WRONG_STATEMENT)	_init_rlq();
	}

	inline ~Charge(){}

	inline const double&  q(const Parton& i)const{ return qf[i.i];}
	inline const double&  R(const Parton& i,const Parton& j)const{ return r[i.i][j.i];}
	inline const double&  L(const Parton& i,const Parton& j)const{ return l[i.i][j.i];}

	inline const double& RR(const Parton& i, const Parton& j)const{ return rr[i.i][j.i];}
	inline const double& LL(const Parton& i, const Parton& j)const{ return ll[i.i][j.i];}
	inline const double&  Q(const Parton& i, const Parton& j)const{ return qq[i.i][j.i];}

	inline const bool& is_active_flv(const Parton& p)const{return active_flavour[p.flv()];}
	inline const bool& is_active(const Parton& q,const Parton& p)const{return active_parton_combination[q.i][p.i];}
	inline PROD get_prod()const{ return prod;}

};
std::ostream& operator<<( std::ostream& os, const Charge& C );




class Q_Matrix:public Matrix<N_PARTONS,double>{
protected:
	Charge C;


	inline void _init(){
		for(Parton i(-N_FMAX); i<Parton(N_FMAX); ++i)for(Parton j(-N_FMAX); j<Parton(N_FMAX); ++j) {
			_M[i.i][j.i]=0.5*C.Q(i,j);
		}
		if( (prod()==PROD_Higgs)||(prod()==PROD_H_RS)) _M[Parton_g.i][Parton_g.i]=1.;
	}

public:

	inline void init(const PROD prod, const CKM_MATRIX& ckm, const double Theta_W, const Vector<N_FMAX+1,bool>& active_flavour){
		C.init(prod,ckm,Theta_W,active_flavour);
		_init();
	}

	inline Q_Matrix()// Matrix<N_FMAX+1,double> ckm_=Matrix<N_FMAX+1,double>(0.))
	:Matrix<N_PARTONS,double>(0.)
	,C(){
		_init();
	}

	inline Q_Matrix(const PROD prod, const CKM_MATRIX& ckm, const double Theta_W, const Vector<N_FMAX+1,bool>& active_flavour)
	:Matrix<N_PARTONS,double>(0.)
	,C(prod,ckm,Theta_W,active_flavour){
		_init();
	}

	inline const double& operator ()(const Parton& i, const Parton& j)const{
		return _M[i.i][j.i];
	}

	inline PROD prod()const{ return C.get_prod();}
	inline const bool& is_active_flv(const Parton& p)const{return C.is_active_flv(p);}
	inline const bool& is_active(const Parton& p, const Parton& q)const{return C.is_active(p,q);}

	std::ostream& to_stream(std::ostream& os, const int nf=N_FMAX, const int ntab=0)const;

};

//std::ostream& operator<<( std::ostream& os, const Q_Matrix& Q){
//	return  Q.to_stream(os);
//}

}	//NS_CHARGES
}	//CuTe

#endif //CHARGES_H_
