#include "gsl_integration.h"
#include "gsl_rand.h"

#ifdef __cplusplus
extern "C" {
#endif
	#include "gsl/gsl_integration.h"
	#include "gsl/gsl_errno.h"
#ifdef __cplusplus
}
#endif
#include <iostream>
#include "utils.h"

using CuTe::NS_Utils::GSL_ERROR_CODE;
using CuTe::NS_GSL_INT::GSL_Integrator;
using CuTe::NS_GSL_INT::GSL_Integrand;



void GSL_Integrator::_alloc_workspace(){
	workspace	= static_cast<void *>(gsl_integration_workspace_alloc(memory));
}

void GSL_Integrator::_free_workspace(){
	gsl_integration_workspace_free(static_cast<gsl_integration_workspace*>(workspace));
}


double GSL_Integrator::test_a_to_b(const double a, const double b, const double acc_rel, const int N){
	Rand rand;
	std::vector<double> values_x(N,0.);
	std::vector<double> values_y(N,0.);
	const std::vector<double>::iterator beg_x(values_x.begin());
	const std::vector<double>::iterator end_x(values_x.end());
	std::vector<double>::iterator itr_x(beg_x);
	std::vector<double>::iterator itr_y(values_y.begin());

	result	= 0.;
	const double V(b-a);

	while(itr_x!=end_x){
		(*itr_x)	 = rand(a,b);
		(*itr_y)	 = V*function((*(itr_x++)),params);
		result		+= (*(itr_y++));
	}



	return result;
}

double GSL_Integrator::integrate_a_to_b(const double a, const double b, const double acc_rel, const double acc_abs, const int key){
	gsl_function_struct gsl_f = {function,params};
	gsl_status = gsl_integration_qag(&gsl_f,a,b,acc_abs, acc_rel, memory,key, static_cast<gsl_integration_workspace*>(workspace), &result, &error);
	return result;
}

double GSL_Integrator::integrate_a_to_inf(const double a, const double acc_rel, const double acc_abs){
	gsl_function_struct gsl_f = {function,params};
	gsl_status = gsl_integration_qagiu(&gsl_f,a,acc_abs, acc_rel, memory, static_cast<gsl_integration_workspace*>(workspace), &result, &error);
	return result;
}

double GSL_Integrator::integrate_minf_to_inf(const double acc_rel, const double acc_abs){
	gsl_function_struct gsl_f = {function,params};
	gsl_status = gsl_integration_qagi(&gsl_f,acc_abs, acc_rel, memory, static_cast<gsl_integration_workspace*>(workspace), &result, &error);
	return result;
}

double GSL_Integrator::integrate_minf_to_b(const double b, const double acc_rel, const double acc_abs){
	gsl_function_struct gsl_f = {function,params};
	gsl_status = gsl_integration_qagil(&gsl_f,b,acc_abs, acc_rel, memory, static_cast<gsl_integration_workspace*>(workspace), &result, &error);
	return result;
}

bool GSL_Integrator::_status(const std::string &name)const{
	if(gsl_status!=GSL_SUCCESS){
		std::cerr	<< "-----------"<<name<<" -- integration error"<<std::endl
					<< "----------- result:"<<result<<" +- "<<error	<<" = "<< 100.*error/result<<" %"<<std::endl;
		GSL_ERROR_CODE(name,gsl_status);
		std::cerr << "----------------------------------------" <<std::endl;
		return true;
	}
	return false;
}


