
/**
* @file konst.h
* @author dwilhelm
* @date	26.12.2012
* @brief Definition of constants*
*/

#ifndef KONST_h
#define KONST_h

#include <cmath>

//! @brief Print GSL_ERROR.
//! @param status return status of a gsl function


namespace CuTe {



#ifndef PI
#define PI (M_PI)
#endif

#ifndef I_PI
#define I_PI (1./M_PI)
#endif

#ifndef PI2
#define PI2 (PI*PI)
#endif

#ifndef PI3
#define PI3 (PI2*PI)
#endif

#ifndef PI4
#define PI4 (PI2*PI2)
#endif


#ifndef gamma_E
#define gamma_E 0.577215664901532	//!<@def gamma_E Eulergamma \f$\gamma_E\f$
#endif

static const double b_0= 2*exp(-gamma_E);//!< b_0 blabla

// M_W=80.385 M_Z=91.1876 => ThetaW
//static const double ThetaW	= 0.4916940451746219;
//static const double SThetaW 	= sin(ThetaW);
////static const double SThetaW2 	= gsl_pow_2(SThetaW);
//static const double CThetaW 	= cos(ThetaW);
//static const double CThetaW2 	= gsl_pow_2(CThetaW);


#ifndef T_F
#define T_F 0.5
#endif

#ifndef N_G
#define N_G 8.0
#endif

#ifndef N_c
#define N_c 3.0
#endif

#ifndef iN_c
#define iN_c 1./N_c
#endif

#ifndef T_A
#define T_A 3.0
#endif

#ifndef C_A
#define C_A		3.0
#define C_A2	C_A*C_A
#define C_A3	C_A2*C_A
#define C_A4	C_A2*C_A2
#define C_A5	C_A3*C_A2
#endif

#ifndef C_F
#define C_F (4.0/3.0)
#define C_F2	C_F*C_F
#define C_F3	C_F2*C_F
#define C_F4	C_F2*C_F2
#define C_F5	C_F3*C_F2
#endif

#ifndef dABCdABC
#define dABCdABC 40./3.
#endif

#ifndef dABCdABCoNc
#define dABCdABCoNc dABCdABC*iN_c
#endif

#ifndef zeta_2
#define zeta_2 1.6449340668482262	//= pi^2/6
#endif

#ifndef zeta_3
#define zeta_3 1.2020569031595942
#endif

#ifndef zeta_4
#define zeta_4 1.0823232337111381	//= pi^4/90
#endif

#ifndef zeta_5
#define zeta_5 1.03692775514337
#endif

//static double ALPHA_EM = (1./127.916);
//static const double G_F = 1.1620765257672414e-5;

#ifndef GAMMA_0
#define GAMMA_0 4.
#endif

//static const double _MZ = 91.1778;
//static const double _MU0 = 10;
//static const double _Alpha0 = 0.175895;
//static const double _Alpha0MZ = 0.11707;
//static const double _Slhc_I = (7000.*7000.);
//static const double _Slhc_II = (8000.*8000.);
//static const double _STeva = (1960.*1960.);

static const double MuFac=2.;

#ifndef DEFAULT_MZ
#define DEFAULT_MZ 91.1876
#endif

#ifndef DEFAULT_MH
#define DEFAULT_MH 125.091
#endif


}  // namespace CuTe



#endif
