#ifndef PARAM_OPTIONS_H_
#define PARAM_OPTIONS_H_

#include "options.h"
#include "enums.h"
//#include "utils_simple_math.h"


namespace CuTe {
namespace NS_CS_Params {

using namespace NS_enums::NS_COL;
using namespace NS_enums::NS_PROD;
using namespace NS_enums::NS_RES;
using namespace NS_enums::NS_NP;
using namespace NS_enums::NS_MS;
using namespace NS_enums::NS_MU_OPS;
using namespace NS_enums::NS_Transition_Function;
using namespace NS_enums::NS_PC;

using NS_OPTIONS::Option;
using NS_OPTIONS::VOption;
using std::string;
using std::vector;

using NS_enums::enum_list;


class OPS_Process: public NS_OPTIONS::_Options_Set{
public:
	Option<COL> col;
	Option<PROD> prod;

	Option<double> sqrts;
	Option<double> M;

protected:

	virtual void v_init_ops(){
//		std::cout<<"\t"<<"OPS_Process::v_init_ops()"<<std::endl;
		col			= Option<COL>("collision",enum_list<COL>(),COL_PP);
		sqrts		= Option<double>("sqrts","[double] GeV",13000);
		prod		= Option<PROD>("production",enum_list<PROD>(),PROD_Higgs);
		M			= Option<double>("M","[double] GeV",125);
	}

	virtual void v_init_vec(){
//		std::cout<<"\t"<<"OPS_Process::v_init_vec()"<<std::endl;
		options.push_back(&col);
		options.push_back(&sqrts);
		options.push_back(&prod);
		options.push_back(&M);
	}

public:

	OPS_Process():_Options_Set("Process Options"){
//		std::cout<<"\t"<<"OPS_Process()"<<std::endl;
		init();
	}

	OPS_Process(const string& filename):_Options_Set("Process Options"){
//		std::cout<<"\t"<<"OPS_Process("<<filename<<")"<<std::endl;
		init(filename);
	}

	OPS_Process(const OPS_Process& OS)
	:_Options_Set(OS)
	,col(OS.col)
	,prod(OS.prod)
	,sqrts(OS.sqrts)
	,M(OS.M){
		init_vec();
	}

};

class OPS_Scheme: public NS_OPTIONS::_Options_Set{
public:
	Option<RES> resummation;
	Option<int> order;
	Option<POWER_COUNTING> Power_Counting;
	Option<int> order_epsilon;
	Option<bool> PiQuadrat;
	Option<bool> exponent;
	Option<bool> exponent_errors;
	Option<MS> MScheme;
	Option<bool> partial_results;
	VOption<double,3> transition_to_FO;
	Option<ENUM_Transition_Function_Type> transition_type;
	Option<ENUM_Transition_Function_Arg> transition_arg;

protected:

	virtual void v_init_ops(){
//		std::cout<<"\t"<<":OPS_Scheme:v_init_ops()"<<std::endl;
		resummation	= Option<RES>("resummation",enum_list<RES>(),RES_MATCHED);
		order		= Option<int>("order","[0,1,2] N^{1+order}LL / N^{order}LO",2);
		Power_Counting	= Option<POWER_COUNTING>("power_counting",enum_list<POWER_COUNTING>(),PC_STD);
//		Power_Counting.set_def_val(PC_IMP);
		order_epsilon= Option<int>("improved_order","[2*order or 2*order+1] ~\\esilon^{improved_order}");
		order_epsilon.set_def_exp("set to 2*order+1");
		PiQuadrat	= Option<bool>("pi_quadrat","[true; false]");
		PiQuadrat.set_def_val(true);
		exponent	= Option<bool>("exponent","[true; false]");
		exponent.set_def_val(true);
		exponent_errors	= Option<bool>("exponent_error","[true; false]");
		exponent_errors.set_def_val(false);
		MScheme		= Option<MS>("matching_scheme",enum_list<MS>());
		MScheme.set_def_val(MS_VAR_QT);
		partial_results =Option<bool>("partial_results","[true; false]");
		partial_results.set_def_val(false);

//		vector<bool> msb_def(3,true);
//		msb_def[1]=false;
//		MS_BOOLS		= VOption<bool,3>("MS_BOOLS","[bool,bool,bool]");
//		MS_BOOLS.set_def_val(msb_def);

		vector<double> tmp;
		tmp.push_back(1.);
		tmp.push_back(0.);
		tmp.push_back(1e-2);
		transition_to_FO	= VOption<double,3>("transition_lrR","[left,right,R]");
		transition_to_FO.set_def_val(tmp);
//		transition_to_FO.set_def_exp("1. 0. 0.01");

		transition_type =Option<ENUM_Transition_Function_Type>("transition_type",enum_list<ENUM_Transition_Function_Type>());
		transition_type.set_def_val(TFT_LOG);

		transition_arg =Option<ENUM_Transition_Function_Arg>("transition_arg",enum_list<ENUM_Transition_Function_Arg>());
		transition_arg.set_def_val(TFA_ETA);

	}

	virtual void v_init_vec(){
//		std::cout<<"\t"<<":OPS_Scheme:v_init_vec()"<<std::endl;
		options.push_back(&resummation);
		options.push_back(&order);
		options.push_back(&Power_Counting);
		options.push_back(&order_epsilon);
		options.push_back(&PiQuadrat);
		options.push_back(&exponent);
		options.push_back(&exponent_errors);
		options.push_back(&MScheme);
		options.push_back(&partial_results);
		options.push_back(&transition_to_FO);
		options.push_back(&transition_type);
		options.push_back(&transition_arg);
	}

public:

	OPS_Scheme():_Options_Set("Scheme Options"){
//		std::cout<<"\t"<<"OPS_Scheme()"<<std::endl;
		init();
	}

	OPS_Scheme(const string& filename):_Options_Set("Scheme Options"){
//		std::cout<<"\t"<<"OPS_Scheme("<<filename<<")"<<std::endl;
		init(filename);
	}

	OPS_Scheme(const OPS_Scheme& OS)
	:_Options_Set(OS)
	,resummation(OS.resummation)
	,order(OS.order)
	,Power_Counting(OS.Power_Counting)
	,order_epsilon(OS.order_epsilon)
	,PiQuadrat(OS.PiQuadrat)
	,exponent(OS.exponent)
	,exponent_errors(OS.exponent_errors)
	,MScheme(OS.MScheme)
	,partial_results(OS.partial_results)
	,transition_to_FO(OS.transition_to_FO)
	,transition_type(OS.transition_type)
	,transition_arg(OS.transition_arg){
		init_vec();
	}

};

class OPS_Scales: public NS_OPTIONS::_Options_Set{
public:
	Option<double> q_star;
	Option<double> fix_mu_to_value;
	Option<ENUM_MU_CONST_OPTION> fix_mu_to_const;
	Option<double> mu_h;
	Option<double> mu_t;
	Option<double>	scale_variation;
//	Option<bool>	Scale_Error;
//	Option<double>	mu_factor;
	Option<CUTOFF> cutoff;
	Option<double> LambdaNP;

protected:

	virtual void v_init_ops(){
//		std::cout<<"\t"<<"OPS_Scales::v_init_ops()"<<std::endl;
		scale_variation = Option<double>("scale_variation","[double] value>0: mu^{+-} = value^{+-1}*mu",2);
		scale_variation.set_def_val(2);

//		Scale_Error	= Option<bool>("scale_variation","[bool]",true);
//		Scale_Error.set_def_val(true);
//		mu_factor	= Option<double>("scale_variation_factor","[double]");
//		mu_factor.set_def_val(2);

		q_star		= Option<double>("q_star","[double] GeV");
		q_star.set_def_exp("ode solution");
		fix_mu_to_value	= Option<double>("mu_value","[double] GeV");
		fix_mu_to_value.set_def_exp("use mu");
		fix_mu_to_const = Option<ENUM_MU_CONST_OPTION>("mu",enum_list<ENUM_MU_CONST_OPTION>(),ENUM_MU_MU_STAR);
		fix_mu_to_const.set_def_val(ENUM_MU_MU_STAR);
//		fix_mu_to_const.set_def_exp("use mu option");
		mu_h		= Option<double>("mu_h","[double] GeV");
		mu_h.set_def_exp("M");
		mu_t		= Option<double>("mu_t","[double] GeV");
		mu_t.set_def_val(172.5);	//https://cds.cern.ch/record/2047636

		cutoff		= Option<CUTOFF>("NPE_model",enum_list<CUTOFF>());
		cutoff.set_def_val(CUTOFF_ENHANCED);
		LambdaNP	= Option<double>("Lambda_NP","[double] GeV");
		LambdaNP.set_def_val(0);

	}

	virtual void v_init_vec(){
//		std::cout<<"\t"<<"OPS_Scales::v_init_vec()"<<std::endl;

		options.push_back(&fix_mu_to_const);
		options.push_back(&fix_mu_to_value);
		options.push_back(&q_star);
		options.push_back(&mu_h);
		options.push_back(&mu_t);
//		options.push_back(&Scale_Error);
//		options.push_back(&mu_factor);
		options.push_back(&scale_variation);
		options.push_back(&cutoff);
		options.push_back(&LambdaNP);
	}

public:

	OPS_Scales():_Options_Set("Scale Options"){
//		std::cout<<"\t"<<"OPS_Scales()"<<std::endl;
		init();
	}

	OPS_Scales(const string& filename):_Options_Set("Scale Options"){
//		std::cout<<"\t"<<"OPS_Scales("<<filename<<")"<<std::endl;
		init(filename);
	}

	OPS_Scales(const OPS_Scales& OS)
	:_Options_Set(OS)
	,q_star(OS.q_star)
	,fix_mu_to_value(OS.fix_mu_to_value)
	,fix_mu_to_const(OS.fix_mu_to_const)
	,mu_h(OS.mu_h)
	,mu_t(OS.mu_t)
//	,Scale_Error(OS.Scale_Error)
//	,mu_factor(OS.mu_factor)
	,scale_variation(OS.scale_variation)
	,cutoff(OS.cutoff)
	,LambdaNP(OS.LambdaNP){
		init_vec();
	}

};

class OPS_SM: public NS_OPTIONS::_Options_Set{
public:
	VOption<double,9> ckm;
	Option<double> alpha_EM;
	Option<double> G_Fermi;
	Option<double> Theta_W;
	Option<double> M_t;
//	VOption<double,6> Gamma_Cusp_3;
	Option<double> Gamma_Cusp_3_Pade_Factor;
	Option<double> Anomaly_Exponent_F_3_0_Factor;
	VOption<bool,6> Active_Flavour_Charges;

protected:
	inline const vector<double> default_ckm()const{
		vector<double> ckm(9,0.);
		vector<double>::iterator it(ckm.begin());
		//http://pdg.lbl.gov/2015/reviews/rpp2014-rev-ckm-matrix.pdf
		*(it++)	= 0.97425;	//ud
		*(it++)	= 0.2253;	//us
		*(it++)	= 4.13e-3;	//ub
		//
		*(it++)	= 0.225;	//cd
		*(it++)	= 0.986;	//cs
		*(it++)	= 41.1e-3;	//cb
		//
		*(it++)	= 8.4e-3;	//td
		*(it++)	= 40.0e-3;	//ts
		*(it++)	= 1.021;	//tb
		return ckm;
	}

	virtual void v_init_ops(){
//		std::cout<<"\t"<<"OPS_SM::v_init_ops()"<<std::endl;
		ckm			= VOption<double,9>("ckm","[|Vud| |Vus| |Vub| ... |Vtb|]");
		ckm.set_def_val(default_ckm());

		alpha_EM	= Option<double>("alpha_EM","[double]");
		alpha_EM.set_def_val(1./127.916);

		G_Fermi		= Option<double>("G_Fermi","[double]");
		G_Fermi.set_def_val(1.1663787e-5);	//https://cds.cern.ch/record/2047636

		Theta_W		= Option<double>("theta_W","[double]");
		Theta_W.set_def_val(0.4916940451746219);

		M_t			= Option<double>("M_t","[double] GeV");
		M_t.set_def_val(172.5);	//https://cds.cern.ch/record/2047636
		Gamma_Cusp_3_Pade_Factor	= Option<double>("G_cusp_3_factor","[double] Gamma_Cusp_3 = Pade * Factor");
		Gamma_Cusp_3_Pade_Factor.set_def_val(1.);
		Anomaly_Exponent_F_3_0_Factor	= Option<double>("F30_factor","[double] F_3_0 = F_2_0 * (4 pi) * Factor");
		Anomaly_Exponent_F_3_0_Factor.set_def_val(0.);

		Active_Flavour_Charges = VOption<bool,6>("active_flavour_charges","[bool d u s c b t]");
		Active_Flavour_Charges.set_def_val(vector<bool>(6,true));

//		std::cout	<<Active_Flavour_Charges<<std::endl;
	}

	virtual void v_init_vec(){
//		std::cout<<"\t"<<"OPS_SM::v_init_vec()"<<std::endl;
		options.push_back(&ckm);
		options.push_back(&alpha_EM);
		options.push_back(&G_Fermi);
		options.push_back(&Theta_W);
		options.push_back(&M_t);
		options.push_back(&Gamma_Cusp_3_Pade_Factor);
		options.push_back(&Anomaly_Exponent_F_3_0_Factor);
		options.push_back(&Active_Flavour_Charges);

	}

public:

	OPS_SM():_Options_Set("SM Options"){
//		std::cout<<"\t"<<"OPS_SM()"<<std::endl;
		init();
	}

	OPS_SM(const string& filename):_Options_Set("SM Options"){
//		std::cout<<"\t"<<"OPS_SM("<<filename<<")"<<std::endl;
		init(filename);
	}

	OPS_SM(const OPS_SM& OS)
	:_Options_Set(OS)
	,ckm(OS.ckm)
	,alpha_EM(OS.alpha_EM)
	,G_Fermi(OS.G_Fermi)
	,Theta_W(OS.Theta_W)
	,M_t(OS.M_t)
	,Gamma_Cusp_3_Pade_Factor(OS.Gamma_Cusp_3_Pade_Factor)
	,Anomaly_Exponent_F_3_0_Factor(OS.Anomaly_Exponent_F_3_0_Factor)
	,Active_Flavour_Charges(OS.Active_Flavour_Charges){
//		std::cout<<"\t"<<"OPS_SM(OS)"<<std::endl;
		init_vec();
	}

};

class OPS_PDF: public NS_OPTIONS::_Options_Set{
public:
	Option<string> PDFset;
	Option<int> PDF_member;
	VOption<int,2> PDF_Dummy_a_b;
	Option<bool>	PDF_Error;
	Option<int>	LHAPDF_verbosity;


protected:

	virtual void v_init_ops(){
//		std::cout<<"\t"<<"OPS_PDF::v_init_ops()"<<std::endl;
		PDFset		= Option<string>("PDF_set","[string]","PDF4LHC15_nnlo_mc");
		PDF_Error	= Option<bool>("PDF_error","[true; false]",false);
		PDF_Error.set_def_val(false);
		PDF_member	= Option<int>("PDF_member","[int]");
		PDF_member.set_def_val(0);

		PDF_Dummy_a_b= VOption<int,2>("PDF_dummy","[int a,int b]");//phi(z) := (z)^a*(1-z)^b
		PDF_Dummy_a_b.set_def_exp("use PDF_set");
		LHAPDF_verbosity	= Option<int>("LHAPDF_verbosity","[int]");
		LHAPDF_verbosity.set_def_val(0);

	}

	virtual void v_init_vec(){
//		std::cout<<"\t"<<"OPS_PDF::v_init_vec()"<<std::endl;
		options.push_back(&PDFset);
		options.push_back(&PDF_Error);
		options.push_back(&PDF_member);

		options.push_back(&PDF_Dummy_a_b);
		options.push_back(&LHAPDF_verbosity);

	}

public:

	OPS_PDF():_Options_Set("PDF Options"){
//		std::cout<<"\t"<<"OPS_PDF()"<<std::endl;
		init();
	}

	OPS_PDF(const string& filename):_Options_Set("PDF Options"){
//		std::cout<<"\t"<<"OPS_PDF("<<filename<<")"<<std::endl;
		init(filename);
	}

	OPS_PDF(const OPS_PDF& OS)
	:_Options_Set(OS)
	,PDFset(OS.PDFset)
	,PDF_member(OS.PDF_member)
	,PDF_Dummy_a_b(OS.PDF_Dummy_a_b)
	,PDF_Error(OS.PDF_Error)
	,LHAPDF_verbosity(OS.LHAPDF_verbosity){
		init_vec();
	}

};


class SOPS_CS_Params:public NS_OPTIONS::_Set_of_Options_Sets {
public:
	OPS_Process Process;
	OPS_Scheme Scheme;
	OPS_Scales Scales;
	OPS_SM SM;
	OPS_PDF PDF;

private:

	void init(){
//		std::cout<<"SOPS_CS_Params::init()"<<std::endl;
		_all.push_back(&Process);
		_all.push_back(&Scheme);
		_all.push_back(&Scales);
		_all.push_back(&SM);
		_all.push_back(&PDF);
		_init();
	}

public:

	SOPS_CS_Params():_Set_of_Options_Sets(){
//		std::cout<<"SOPS_CS_Params()"<<std::endl;
		init();
	}

	SOPS_CS_Params(const string& filename):_Set_of_Options_Sets(){
//		std::cout<<"SOPS_CS_Params("<<filename<<")"<<std::endl;
		init();
		open(filename);
	}

	SOPS_CS_Params(const SOPS_CS_Params& OP)
	:_Set_of_Options_Sets(OP)
	,Process(OP.Process)
	,Scheme(OP.Scheme)
	,Scales(OP.Scales)
	,SM(OP.SM)
	,PDF(OP.PDF){
//		std::cout<<"SOPS_CS_Params(const SOPS_CS_Params& OP)"<<std::endl;
		init();
	}

};



}  // namespace NS_CS_Params
}  // namespace CuTe


#endif //PARAM_OPTIONS_H_
