#include "test_Convolutions.h"
#include <iomanip>
using namespace CuTe::NS_Convolutions;

inline ostream& print_V(const Vector<2*NFMAX+1,double> &V, int nf){
	cout<<"[";
	for(int i=(-nf); i<(-1); ++i) cout<<" "<<std::setw(10)<<V[NFMAX+i]<<" ,";
	cout<<" "<<std::setw(10)<<V[NFMAX-1]<<" | "<<std::setw(10)<<V[NFMAX]<<" | "<<std::setw(10)<<V[NFMAX+1];
	for(int i=2; i<=nf; ++i) cout<<" , "<<std::setw(10)<<V[NFMAX+i];
	cout<<" ]";
	cout<<"\tSum = "<<V.p_norm<1>();
	return cout;
}

inline ostream& print_rel(const Vector<2*NFMAX+1,double> &V1, const Vector<2*NFMAX+1,double> &V2, int nf){
	cout<<"[";
	for(int i=(-nf); i<(-1); ++i) cout<<" "<<std::setw(10)<<100.*(V2[NFMAX+i]/V1[NFMAX+i]-1.)<<" ,";
	cout<<" "<<std::setw(10)<<100.*(V2[NFMAX-1]/V1[NFMAX-1]-1.)<<" | "<<std::setw(10)<<100.*(V2[NFMAX]/V1[NFMAX]-1.)<<" | "<<std::setw(10)<<100.*(V2[NFMAX+1]/V1[NFMAX+1]-1.);
	for(int i=2; i<=nf; ++i) cout<<" , "<<std::setw(10)<<100.*(V2[NFMAX+i]/V1[NFMAX+i]-1.);
	cout<<" ]";
	cout<<"\tSum = "<<100.*((V2-V1).p_norm<1>()/V1.p_norm<1>());
	return cout;
}

template< int n, int l, int e >
inline void print(const Vector<2*NFMAX+1,double> &V1, const Vector<2*NFMAX+1,double> &V2, const int nf){
	cout<<"Ibar<"<<n<<","<<l<<","<<e<<">:";
//	cout<<"\t"<<std::setw(10)<<V1.p_norm<1>();
//	cout<<"\t"<<std::setw(10)<<V2.p_norm<1>();
//	cout<<"\t"<<std::setw(4)<<std::setprecision(6)<<100.*((V2-V1).p_norm<1>()/V1.p_norm<1>());
	cout<<endl;
	if(nf!=0){
		cout<<"\t";print_V(V1,nf)<<endl;
		cout<<"\t";print_V(V2,nf)<<endl;
		cout<<"\t";print_rel(V1,V2,nf)<<endl;
	}
	cout<<endl;
}

inline void test_apsfff(int nf){
	CuTe::NS_APSF::APSF_Parameter p;

	double xi	= 0.2;
	double z	= 0.6;

	double fz	= 2.0;
	double f1	= 1.5;


	p.set_x(xi);
	p.update(z);


	cout	<<"\t"<<"xi = "<<xi	<<"\t"<<"z = "<<z	<<"\t"<<"nf = "<<nf	<<endl;
	cout	<<"\t"<<"fz = "<<fz	<<"\t"<<"f1 = "<<f1	<<endl;


	cout<<"QQ  0 = "<<CuTe::NS_APSF::APSF<e_q,e_qmp,0>(fz,f1,p,nf)<<endl;
	cout<<"QQb 0 = "<<CuTe::NS_APSF::APSF<e_q,e_qbmp,0>(fz,f1,p,nf)<<endl;
	cout<<"QG  0 = "<<CuTe::NS_APSF::APSF<e_q,e_g,0>(fz,f1,p,nf)<<endl;
	cout<<"QP  0 = "<<CuTe::NS_APSF::APSF<e_q,e_qp,0>(fz,f1,p,nf)<<endl;
	cout<<"QPb 0 = "<<CuTe::NS_APSF::APSF<e_q,e_qbp,0>(fz,f1,p,nf)<<endl;
	cout<<"GQ  0 = "<<CuTe::NS_APSF::APSF<e_g,e_q,0>(fz,f1,p,nf)<<endl;
	cout<<"GG  0 = "<<CuTe::NS_APSF::APSF<e_g,e_g,0>(fz,f1,p,nf)<<endl;

	cout<<endl<<endl;

	cout<<"QQ  1 = "<<CuTe::NS_APSF::APSF<e_q,e_qmp,1>(fz,f1,p,nf)<<endl;
	cout<<"QQb 1 = "<<CuTe::NS_APSF::APSF<e_q,e_qbmp,1>(fz,f1,p,nf)<<endl;
	cout<<"QG  1 = "<<CuTe::NS_APSF::APSF<e_q,e_g,1>(fz,f1,p,nf)<<endl;
	cout<<"QP  1 = "<<CuTe::NS_APSF::APSF<e_q,e_qp,1>(fz,f1,p,nf)<<endl;
	cout<<"QPb 1 = "<<CuTe::NS_APSF::APSF<e_q,e_qbp,1>(fz,f1,p,nf)<<endl;
	cout<<"GQ  1 = "<<CuTe::NS_APSF::APSF<e_g,e_q,1>(fz,f1,p,nf)<<endl;
	cout<<"GG  1 = "<<CuTe::NS_APSF::APSF<e_g,e_g,1>(fz,f1,p,nf)<<endl;

	cout<<endl<<endl;

	cout<<"QQ  2 = "<<CuTe::NS_APSF::APSF<e_q,e_qmp,2>(fz,f1,p,nf)<<endl;
	cout<<"QQb 2 = "<<CuTe::NS_APSF::APSF<e_q,e_qbmp,2>(fz,f1,p,nf)<<endl;
	cout<<"QG  2 = "<<CuTe::NS_APSF::APSF<e_q,e_g,2>(fz,f1,p,nf)<<endl;
	cout<<"QP  2 = "<<CuTe::NS_APSF::APSF<e_q,e_qp,2>(fz,f1,p,nf)<<endl;
	cout<<"QPb 2 = "<<CuTe::NS_APSF::APSF<e_q,e_qbp,2>(fz,f1,p,nf)<<endl;
	cout<<"GQ  2 = "<<CuTe::NS_APSF::APSF<e_g,e_q,2>(fz,f1,p,nf)<<endl;
	cout<<"GG  2 = "<<CuTe::NS_APSF::APSF<e_g,e_g,2>(fz,f1,p,nf)<<endl;

	cout<<endl<<endl;
}

inline void test_ibar(int nf, double xi){
	double beta0=(11.*C_A -2.*nf)/3.;
	double beta1=34.*C_A2/3.-20.*C_A*T_F*nf/3.-4.*C_F*T_F*nf;

	Convolution_Function f;
//	f.init_function(testfunction);
//	f.init_function(testfunction_only<0>);
	f.init_function(testfunction_only<1>);
//	f.init_function(testfunction_same);
//	f.init_function(testfunction_diff);

	Convolution_Integral I;

	Vector<2*NFMAX+1,double> V1,V2,Vtmp;

	I_bar_Convolution<1,1,1> Ibar111;
	I.init_Integrand(&Ibar111)(&f);
	I.set_nf(nf);
	I.update(xi);
	V1<<I;

	APSF_Convolution<0> APSF0;
	I.init_Integrand(&APSF0)(&f);
	I.set_nf(nf);
	I.update(xi);
	V2<<I;

	V2*=-0.5;

	print<1,1,1>(V1,V2,nf);


	I_bar_Convolution<2,2,2> Ibar222;
	I.init_Integrand(&Ibar222)(&f);
	I.set_nf(nf);
	I.update(xi);
	V1<<I;

	I.init_Integrand(&Ibar111)(&APSF0)(&f);
	I.set_nf(nf);
	I.update(xi);
	Vtmp<<I;
	V2	= -Vtmp*0.25;

	I.init_Integrand(&Ibar111)(&f);
	I.set_nf(nf);
	I.update(xi);
	Vtmp<<I;
	V2	+= Vtmp*0.5*beta0;

	print<2,2,2>(V1,V2,nf);

	APSF_Convolution<0> APSF02;
	I.init_Integrand(&APSF02)(&APSF0)(&f);
	I.set_nf(nf);
	I.update(xi);
	Vtmp<<I;
	V2	= Vtmp*0.25;

	I.init_Integrand(&APSF0)(&f);
	I.set_nf(nf);
	I.update(xi);
	Vtmp<<I;
	V2	+= -Vtmp*0.5*beta0;

	V2*=0.5;

	print<2,2,2>(V1,V2,nf);


	I_bar_Convolution<2,1,3> Ibar213;
	I_bar_Convolution<1,0,2> Ibar102;
	I.init_Integrand(&Ibar213)(&f);
	I.set_nf(nf);
	I.update(xi);
	V1<<I;

	APSF_Convolution<1> APSF1;
	I.init_Integrand(&APSF1)(&f);
	I.set_nf(nf);
	I.update(xi);
	Vtmp<<I;
	V2	= -Vtmp*0.5;

	I.init_Integrand(&Ibar102)(&APSF0)(&f);
	I.set_nf(nf);
	I.update(xi);
	Vtmp<<I;
	V2	-= Vtmp*0.5;

	I.init_Integrand(&Ibar102)(&f);
	I.set_nf(nf);
	I.update(xi);
	Vtmp<<I;
	V2	+= Vtmp*beta0;

	print<2,1,3>(V1,V2,nf);


	I_bar_Convolution<3,3,3> Ibar333;
	I.init_Integrand(&Ibar333)(&f);
	I.set_nf(nf);
	I.update(xi);
	V1<<I;

	I.init_Integrand(&Ibar222)(&APSF0)(&f);
	I.set_nf(nf);
	I.update(xi);
	Vtmp<<I;
	V2	= -Vtmp*0.5;

	I.init_Integrand(&Ibar222)(&f);
	I.set_nf(nf);
	I.update(xi);
	Vtmp<<I;
	V2	+= Vtmp*2.*beta0;

	V2	*= 1/3.;

	print<3,3,3>(V1,V2,nf);


	I_bar_Convolution<4,4,4> Ibar444;
	I.init_Integrand(&Ibar444)(&f);
	I.set_nf(nf);
	I.update(xi);
	V1<<I;

	I.init_Integrand(&Ibar333)(&APSF0)(&f);
	I.set_nf(nf);
	I.update(xi);
	Vtmp<<I;
	V2	= -Vtmp*0.5;

	I.init_Integrand(&Ibar333)(&f);
	I.set_nf(nf);
	I.update(xi);
	Vtmp<<I;
	V2	+= Vtmp*3.*beta0;

	V2	*= 1/4.;

	print<4,4,4>(V1,V2,nf);


	I_bar_Convolution<5,5,5> Ibar555;
	I.init_Integrand(&Ibar555)(&f);
	I.set_nf(nf);
	I.update(xi);
	V1<<I;

	I.init_Integrand(&Ibar444)(&APSF0)(&f);
	I.set_nf(nf);
	I.update(xi);
	Vtmp<<I;
	V2	= -Vtmp*0.5;

	I.init_Integrand(&Ibar444)(&f);
	I.set_nf(nf);
	I.update(xi);
	Vtmp<<I;
	V2	+= Vtmp*4.*beta0;

	V2	*= 1/5.;

	print<5,5,5>(V1,V2,nf);


	I_bar_Convolution<3,2,4> Ibar324;
	I.init_Integrand(&Ibar324)(&f);
	I.set_nf(nf);
	I.update(xi);
	V1<<I;

//	cout<<"\t"<<"Ibar<3,2,4> o f = "<<V1.p_norm<1>()<<endl;

	I.init_Integrand(&Ibar111)(&APSF1)(&f);
	I.set_nf(nf);
	I.update(xi);
	Vtmp<<I;
	V2	= -Vtmp*0.5;

//	cout<<"\t"<<"1/3 * ( - Ibar<1,1,1> o AP<1>/2 o f ) = "<<Vtmp.p_norm<1>()*(-0.5)/3.<<endl;

	I.init_Integrand(&Ibar111)(&f);
	I.set_nf(nf);
	I.update(xi);
	Vtmp<<I;
	V2	+= Vtmp*beta1;

//	cout<<"\t"<<"1/3 * ( + beta_1 * Ibar<1,1,1> o f ) = "<<Vtmp.p_norm<1>()*beta1/3.<<endl;

	I.init_Integrand(&Ibar213)(&APSF0)(&f);
	I.set_nf(nf);
	I.update(xi);
	Vtmp<<I;
	V2	-= Vtmp*0.5;

//	cout<<"\t"<<"1/3 * ( - Ibar<2,1,3> o AP<0>/2 o f ) = "<<Vtmp.p_norm<1>()*(-0.5)/3.<<endl;

	I.init_Integrand(&Ibar213)(&f);
	I.set_nf(nf);
	I.update(xi);
	Vtmp<<I;
	V2	+= Vtmp*2.*beta0;

//	cout<<"\t"<<"1/3 * ( + 2 * beta_0 * Ibar<2,1,3> o f ) = "<<Vtmp.p_norm<1>()*2.*beta0/3.<<endl;

	V2	*= 1/2.;

	print<3,2,4>(V1,V2,nf);




	I_bar_Convolution<4,3,5> Ibar435;
	I.init_Integrand(&Ibar435)(&f);
	I.set_nf(nf);
	I.update(xi);
	V1<<I;

	I.init_Integrand(&Ibar222)(&APSF1)(&f);
	I.set_nf(nf);
	I.update(xi);
	Vtmp<<I;
	V2	= -Vtmp*0.5;

	I.init_Integrand(&Ibar222)(&f);
	I.set_nf(nf);
	I.update(xi);
	Vtmp<<I;
	V2	+= Vtmp*2.*beta1;

	I.init_Integrand(&Ibar324)(&APSF0)(&f);
	I.set_nf(nf);
	I.update(xi);
	Vtmp<<I;
	V2	+= -Vtmp*0.5;

	I.init_Integrand(&Ibar324)(&f);
	I.set_nf(nf);
	I.update(xi);
	Vtmp<<I;
	V2	+= Vtmp*3.*beta0;

	V2	*= 1/3.;

	print<4,3,5>(V1,V2,nf);



	I_bar_Convolution<2,0,4> Ibar204;
	APSF_Convolution<2> APSF2;


	I_bar_Convolution<3,1,5> Ibar315;
	I.init_Integrand(&Ibar315)(&f);
	I.set_nf(nf);
	I.update(xi);
	V1<<I;
//	cout<<"\t"<<"Ibar<3,1,5> o f = "<<endl<<"\t\t";print_V(V1,nf)<<endl;

	I.init_Integrand(&APSF2)(&f);
	I.set_nf(nf);
	I.update(xi);
	Vtmp<<I;
	Vtmp *=(-0.5);
	V2	= Vtmp;
//	cout<<"\t"<<"AP<2> o f = "<<endl<<"\t\t";print_V(Vtmp,nf)<<endl;

	I.init_Integrand(&Ibar102)(&APSF1)(&f);
	I.set_nf(nf);
	I.update(xi);
	Vtmp<<I;
	Vtmp *= (-0.5);
	V2	+= Vtmp;
//	cout<<"\t"<<"Ibar<1,0,2> o AP<1> o f = "<<endl<<"\t\t";print_V(Vtmp,nf)<<endl;

	I.init_Integrand(&Ibar102)(&f);
	I.set_nf(nf);
	I.update(xi);
	Vtmp<<I;
	Vtmp *= beta1;
	V2	+= Vtmp;
//	cout<<"\t"<<"Ibar<1,0,2> o f = "<<endl<<"\t\t";print_V(Vtmp,nf)<<endl;

	I.init_Integrand(&Ibar204)(&APSF0)(&f);
	I.set_nf(nf);
	I.update(xi);
	Vtmp<<I;
	Vtmp *=(-0.5);
	V2	+= Vtmp;
//	cout<<"\t"<<"Ibar<2,0,4> o AP<0> o f = "<<endl<<"\t\t";print_V(Vtmp,nf)<<endl;

	I.init_Integrand(&Ibar204)(&f);
	I.set_nf(nf);
	I.update(xi);
	Vtmp<<I;
	Vtmp *=(2.*beta0);
	V2	+= Vtmp;
//	cout<<"\t"<<"Ibar<2,0,4> o f = "<<endl<<"\t\t";print_V(Vtmp,nf)<<endl;

	print<3,1,5>(V1,V2,nf);
	cout<<endl;
//Ibar<1,1,1>:
//	[  -0.399097 ,  -0.399097 |   -4.11912 |  -0.399097 ,  -0.399097 ]	Sum = -5.71551
//	[  -0.399097 ,  -0.399097 |   -4.11912 |  -0.399097 ,  -0.399097 ]	Sum = -5.71551
//	[          0 ,          0 | -2.22045e-14 |          0 ,          0 ]	Sum = -1.55398e-14
//
//Ibar<2,2,2>:
//	[    -1.9179 ,    -1.9179 |   -33.5953 |    -1.9179 ,    -1.9179 ]	Sum = -41.2669
//	[    -1.9179 ,    -1.9179 |   -33.5953 |    -1.9179 ,    -1.9179 ]	Sum = -41.2669
//	[ 2.46474e-08 , 2.46474e-08 | 8.94436e-09 | 2.46474e-08 , 2.46474e-08 ]	Sum = 1.18636e-08
//
//Ibar<2,1,3>:
//	[   -3.57721 ,   -3.57721 |   -19.4106 |   -3.57721 ,   -3.57721 ]	Sum = -33.7195
//	[   -3.57721 ,   -3.57721 |   -19.4106 |   -3.57721 ,   -3.57721 ]	Sum = -33.7195
//	[ 1.00268e-08 , 1.00268e-08 | -3.46631e-09 | 1.00268e-08 , 1.00268e-08 ]	Sum = 2.25948e-09
//
//Ibar<3,3,3>:
//	[   -6.03618 ,   -6.03618 |   -183.466 |   -6.03618 ,   -6.03618 ]	Sum = -207.611
//	[   -6.03618 ,   -6.03618 |   -183.466 |   -6.03618 ,   -6.03618 ]	Sum = -207.611
//	[ 1.07461e-07 , 1.07461e-07 | -1.70355e-08 | 1.07461e-07 , 1.07461e-07 ]	Sum = -2.55676e-09
//
//Ibar<4,4,4>:
//	[    12.0302 ,    12.0302 |   -455.722 |    12.0302 ,    12.0302 ]	Sum = -407.601
//	[    12.0302 ,    12.0302 |   -455.722 |    12.0302 ,    12.0302 ]	Sum = -407.601
//	[ -3.91644e-07 , -3.91644e-07 | -6.20582e-08 | -3.91644e-07 , -3.91644e-07 ]	Sum = -2.31478e-08
//
//Ibar<5,5,5>:
//	[    499.013 ,    499.013 |    5990.58 |    499.013 ,    499.013 ]	Sum = 7986.63
//	[    499.013 ,    499.013 |    5990.58 |    499.013 ,    499.013 ]	Sum = 7986.63
//	[ -7.33756e-08 , -7.33756e-08 | -2.81532e-08 | -7.33756e-08 , -7.33756e-08 ]	Sum = -3.94554e-08
//
//Ibar<3,2,4>:
//	[   -69.3555 ,   -69.3555 |    -343.97 |   -69.3555 ,   -69.3555 ]	Sum = -621.392
//	[   -69.3555 ,   -69.3555 |    -343.97 |   -69.3555 ,   -69.3555 ]	Sum = -621.392
//	[ -1.80577e-08 , -1.80577e-08 | 4.96876e-09 | -1.80577e-08 , -1.80577e-08 ]	Sum = -5.31148e-09
//
//Ibar<4,3,5>:
//	[   -775.702 ,   -775.702 |   -2648.05 |   -775.702 ,   -775.702 ]	Sum = -5750.85
//	[   -774.664 ,   -774.664 |   -2648.05 |   -774.664 ,   -774.664 ]	Sum = -5746.7
//	[    -0.1338 ,    -0.1338 | -5.51503e-10 |    -0.1338 ,    -0.1338 ]	Sum = -0.0721904
//
//Cubacuhre accuracy goal not reached
//Ibar<3,1,5>:
//	[    -7.5766 ,    -7.5766 |    404.971 |    -7.5766 ,    -7.5766 ]	Sum = 374.665
//	[   -7.57393 ,   -7.57393 |    404.971 |   -7.57393 ,   -7.57393 ]	Sum = 374.675
//	[ -0.0352698 , -0.0352698 | -1.11794e-07 | -0.0352698 , -0.0352698 ]	Sum = 0.00285283

}

template<int n>
void test_APoF(int nf, double xi, double xx[]){
	Convolution_Function f;
	f.init_function(testfunction_diff);


	Vector<2*NFMAX+1,double> V;
	_Convolution_Integrand *I;

	cout<<"\t"<<"xi = "<<xi	<<"\t"<<"z = "<<(xi+(*xx)*(1.-xi))<<endl<<endl;

	APSF_single_Convolution<e_q,e_qmp,n>	AP2_QQ;
	I	= &AP2_QQ;
	I->init_function(&f);
	I->set_nf(nf);
	I->update_xi_z(xi,xx);
	cout<<"AP<"<<n<<">_QQ :\t";print_V(V<<(*I),nf)<<endl;

	APSF_single_Convolution<e_q,e_qbmp,n>	AP2_QQb;
	I	= &AP2_QQb;
	I->init_function(&f);
	I->set_nf(nf);
	I->update_xi_z(xi,xx);
	cout<<"AP<"<<n<<">_QQb:\t";print_V(V<<(*I),nf)<<endl;

	APSF_single_Convolution<e_q,e_g,n>		AP2_QG;
	I	= &AP2_QG;
	I->init_function(&f);
	I->set_nf(nf);
	I->update_xi_z(xi,xx);
	cout<<"AP<"<<n<<">_QG :\t";print_V(V<<(*I),nf)<<endl;

	APSF_single_Convolution<e_q,e_qp,n>		AP2_QP;
	I	= &AP2_QP;
	I->init_function(&f);
	I->set_nf(nf);
	I->update_xi_z(xi,xx);
	cout<<"AP<"<<n<<">_QP :\t";print_V(V<<(*I),nf)<<endl;

	APSF_single_Convolution<e_q,e_qbp,n>	AP2_QPb;
	I	= &AP2_QPb;
	I->init_function(&f);
	I->set_nf(nf);
	I->update_xi_z(xi,xx);
	cout<<"AP<"<<n<<">_QPb:\t";print_V(V<<(*I),nf)<<endl;

	APSF_single_Convolution<e_g,e_q,n>		AP2_GQ;
	I	= &AP2_GQ;
	I->init_function(&f);
	I->set_nf(nf);
	I->update_xi_z(xi,xx);
	cout<<"AP<"<<n<<">_GQ :\t";print_V(V<<(*I),nf)<<endl;

	APSF_single_Convolution<e_g,e_g,n>		AP2_GG;
	I	= &AP2_GG;
	I->init_function(&f);
	I->set_nf(nf);
	I->update_xi_z(xi,xx);
	cout<<"AP<"<<n<<">_GG :\t";print_V(V<<(*I),nf)<<endl;


	cout<<endl<<endl;
}

template<int n>
void test_APoF(int nf, double xi){
	Convolution_Function f;
	f.init_function(testfunction_diff);

	Convolution_Integral I;

	Vector<2*NFMAX+1,double> V;

	APSF_single_Convolution<e_q,e_qmp,n>	AP2_QQ;
	I.init_Integrand(&AP2_QQ)(&f);
	I.set_nf(nf);
	I.update(xi);
	cout<<"AP<"<<n<<">_QQ :\t";print_V(V<<I,nf)<<endl;

	APSF_single_Convolution<e_q,e_qbmp,n>	AP2_QQb;
	I.init_Integrand(&AP2_QQb)(&f);
	I.set_nf(nf);
	I.update(xi);
	cout<<"AP<"<<n<<">_QQb:\t";print_V(V<<I,nf)<<endl;

	APSF_single_Convolution<e_q,e_g,n>		AP2_QG;
	I.init_Integrand(&AP2_QG)(&f);
	I.set_nf(nf);
	I.update(xi);
	cout<<"AP<"<<n<<">_QG :\t";print_V(V<<I,nf)<<endl;

	APSF_single_Convolution<e_q,e_qp,n>		AP2_QP;
	I.init_Integrand(&AP2_QP)(&f);
	I.set_nf(nf);
	I.update(xi);
	cout<<"AP<"<<n<<">_QP :\t";print_V(V<<I,nf)<<endl;

	APSF_single_Convolution<e_q,e_qbp,n>	AP2_QPb;
	I.init_Integrand(&AP2_QPb)(&f);
	I.set_nf(nf);
	I.update(xi);
	cout<<"AP<"<<n<<">_QPb:\t";print_V(V<<I,nf)<<endl;

	APSF_single_Convolution<e_g,e_q,n>		AP2_GQ;
	I.init_Integrand(&AP2_GQ)(&f);
	I.set_nf(nf);
	I.update(xi);
	cout<<"AP<"<<n<<">_GQ :\t";print_V(V<<I,nf)<<endl;

	APSF_single_Convolution<e_g,e_g,n>		AP2_GG;
	I.init_Integrand(&AP2_GG)(&f);
	I.set_nf(nf);
	I.update(xi);
	cout<<"AP<"<<n<<">_GG :\t";print_V(V<<I,nf)<<endl;


	cout<<endl<<endl;
}

int main (int argc, char *argv[]){
	int nf		= 2;
	double xi	= 0.2;
	double xxx[3];
	xxx[0]	= 0.5;
	xxx[1]	= 0.75;
	xxx[2]	= 0.8;

//	test_apsfff(nf);
//	test_APoF<0>(nf,xi,xxx);
//	test_APoF<1>(nf,xi,xxx);
//	test_APoF<2>(nf,xi,xxx);
//	test_APoF<0>(nf,xi);
//	test_APoF<1>(nf,xi);
//	test_APoF<2>(nf,xi);
	test_ibar(nf,xi);




	return 0;
}
